/**
 * 
 */
package com.ejie.ab61.adaptadores.service;

import java.net.URL;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.HandlerResolver;
import javax.xml.ws.handler.PortInfo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.ejie.ab61.clientesWS.X43SolicitudesYAportaciones.AExtensionPoint;
import com.ejie.ab61.clientesWS.X43SolicitudesYAportaciones.TAdminInitRequest;
import com.ejie.ab61.clientesWS.X43SolicitudesYAportaciones.TSubmissionResponse;
import com.ejie.ab61.clientesWS.X43SolicitudesYAportaciones.X43PPS;
import com.ejie.ab61.clientesWS.X43SolicitudesYAportaciones.X43PPSPortType;
import com.ejie.ab61.clientesWS.X43Toolkit.Exception_Exception;
import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.comun.utilidades.config.Config;

/**
 *  
 * 
 */
@Service(value = "X43SolicitudesYAportacionesServiceImpl")
public class X43SolicitudesYAportacionesServiceImpl implements X43SolicitudesYAportacionesService {

	private static final Logger logger = LoggerFactory.getLogger(X43SolicitudesYAportacionesServiceImpl.class);
	private X43PPS service;

	private static final QName qname = new QName("com/ejie/x43s/v2/pps/ws", "X43PPS");

	/**
	 * 
	 * @return X43PPSPortType
	 */
	private X43PPSPortType getX43SolicitudesYAportacionesServicePort() {
		return this.getX43SolicitudesYAportacionesServicePort(true);
	}

	/**
	 * 
	 * @param getPort
	 *            boolean
	 * 
	 * @return X43PPSPortType
	 */
	private X43PPSPortType getX43SolicitudesYAportacionesServicePort(boolean getPort) {
		try {
			Properties prop = Config.loadProperties(Constantes.FICHERO_CONFIGURACION);

			if (this.service == null) {

				X43SolicitudesYAportacionesServiceImpl.logger.info("URL del servicio X43SolicitudesYAportaciones = "
						+ prop.getProperty("X43SolicitudesYAportaciones_URL"));

				// Creamos el servicio
				this.service = new X43PPS(new URL(prop.getProperty("X43SolicitudesYAportaciones_URL")),
						X43SolicitudesYAportacionesServiceImpl.qname);

				// definimos un handler cliente
				this.service.setHandlerResolver(new HandlerResolver() {
					@SuppressWarnings(value = "unchecked")
					public List getHandlerChain(PortInfo portInfo) {
						return Collections.singletonList(new X43SolicitudesYAportacionesServiceHandler());
					}
				});

				X43SolicitudesYAportacionesServiceImpl.logger.info(
						"getX43SolicitudesYAportacionesServicePort. Dentro de creaci�n del servicio - Definido el handler");
			}
			if (getPort) {

				X43SolicitudesYAportacionesServiceImpl.logger
						.info("getX43SolicitudesYAportacionesServicePort. Antes de obtener el proxy");

				X43PPSPortType proxy = this.service.getX43PPSPort();

				X43SolicitudesYAportacionesServiceImpl.logger
						.info("getX43SolicitudesYAportacionesServicePort. Despues de obtener el proxy");

				Map<String, Object> ctxt = ((BindingProvider) proxy).getRequestContext();

				X43SolicitudesYAportacionesServiceImpl.logger
						.info("getX43SolicitudesYAportacionesServicePort. Despues de obtener el contexto");

				ctxt.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
						// prop.getProperty("URL_WS_X43SolicitudesYAportaciones"));
						prop.getProperty("X43SolicitudesYAportaciones_URL"));

				X43SolicitudesYAportacionesServiceImpl.logger
						.info("getX43SolicitudesYAportacionesServicePort. Cargado el contexto");

				return proxy;
			}

		} catch (Exception e) {

			X43SolicitudesYAportacionesServiceImpl.logger
					.info("getX43SolicitudesYAportacionesServicePort. Excepcionnnn:" + e.getMessage());

			this.service = null;
		}
		return null;
	}

	/**
	 * Metodo que permite a la administración iniciar una solicitud bien por
	 * iniciativa propia, por ser una petición razonada o por una denuncia
	 * 
	 * @param adminInitiation
	 *            TAdminInitRequest
	 * @param ep
	 *            AExtensionPoint
	 * 
	 * @return TSubmissionResponse
	 * 
	 * @throws X43SolicitudesYAportacionesException
	 *             X43SolicitudesYAportacionesException
	 */
	public TSubmissionResponse addAdminApplication(TAdminInitRequest adminInitiation, AExtensionPoint ep)
			throws Exception_Exception {

		return this.getX43SolicitudesYAportacionesServicePort().addAdminApplication(adminInitiation, ep);

	}

}
