package com.ejie.ab61.dao.busquedas;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.busquedas.Busquedas;
import com.ejie.ab61.model.listados.Listados;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ListadosDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class BusquedasDaoImpl implements BusquedasDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDLISTADO", "IDPROCEDIMIENTO", "NOMBRE",
			"FICHERO", "FECHAREGISTRO", "USUARIOREGISTRO", "FECHAULTMODIFICACION", "USUARIOULTMODIFICACION" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Busquedas> rwMap = new RowMapper<Busquedas>() {
		public Busquedas mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			Busquedas busqueda = new Busquedas();
			busqueda.setIdBusqueda(resultSet.getInt("IDBUSQUEDA"));
			busqueda.setIdListado(resultSet.getInt("IDLISTADO"));
			busqueda.setNombre(resultSet.getString("NOMBRE"));
			busqueda.setSqlListado(resultSet.getString("SQLLISTADO"));
			busqueda.setCampos(resultSet.getString("CAMPOS"));
			return busqueda;
		}
	};

	private RowMapper<Busquedas> rwMapPK = new RowMapper<Busquedas>() {
		public Busquedas mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Busquedas(resultSet.getInt("IDBUSQUEDA"));
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Busquedas table.
	 *
	 * @param busquedas
	 *            Busquedas
	 * @return Busquedas
	 */
	public Busquedas add(Busquedas busquedas) {
		String query = "INSERT INTO BUSQUEDAS (ID_BUSQUEDA, ID_LISTADO, NOMBRE, SQL_LISTADO,CAMPOS, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (AB61.S_AB6138T00.NEXTVAL,?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, busquedas.getIdListado(), busquedas.getNombre(), busquedas.getSqlListado(),
				busquedas.getCampos(), new Date(), busquedas.getUsuarioRegistro());
		return busquedas;
	}

	//
	/**
	 * Updates a single row in the Busquedas table.
	 *
	 * @param busquedas
	 *            Busquedas
	 * @return Busquedas
	 */
	public Busquedas update(Busquedas busquedas) {
		String query = "UPDATE BUSQUEDAS SET NOMBRE=?, SQL_LISTADO=?,CAMPOS=?, FECHA_ULTMODIFICACION=?,USUARIO_ULTMODIFICACION=? WHERE ID_BUSQUEDA=?";
		this.jdbcTemplate.update(query, busquedas.getNombre(), busquedas.getSqlListado(), busquedas.getCampos(),
				new Date(), busquedas.getUsuarioUltModificacion(), busquedas.getIdBusqueda());
		return busquedas;
	}

	/**
	 * Finds a single row in the Busquedas table.
	 *
	 * @param busquedas
	 *            Busquedas
	 * @return Busquedas
	 */
	@Transactional(readOnly = true)
	public Busquedas find(Busquedas busquedas) {
		String query = "SELECT T1.ID_BUSQUEDA IDBUSQUEDA, T1.ID_LISTADO IDLISTADO, T1.NOMBRE NOMBRE, T1.SQL_LISTADO SQLLISTADO, T1.CAMPOS CAMPOS FROM BUSQUEDAS T1 WHERE t1.ID_BUSQUEDA = ? ";

		List<Busquedas> busquedasList = this.jdbcTemplate.query(query, this.rwMap, busquedas.getIdBusqueda());
		return (Busquedas) DataAccessUtils.uniqueResult(busquedasList);
	}

	/**
	 * Removes a single row in the Busquedas table.
	 *
	 * @param busquedas
	 *            Busquedas
	 * @return
	 */
	public void remove(Busquedas busquedas) {
		String query = "DELETE FROM BUSQUEDAS WHERE ID_BUSQUEDA=?";
		this.jdbcTemplate.update(query, busquedas.getIdBusqueda());
	}
	//
	// /**
	// * Finds a list of rows in the Listados table.
	// *
	// * @param listados
	// * Listados
	// * @param jqGridRequestDto
	// * JQGridRequestDto
	// * @return List<Listados>
	// */
	// @Transactional(readOnly = true)
	// public List<Listados> findAll(Listados listados, JQGridRequestDto
	// jqGridRequestDto) {
	// StringBuilder query = new StringBuilder(
	// "SELECT t1.ID_LISTADO IDLISTADO,t1.ID_PROCEDIMIENTO
	// IDPROCEDIMIENTO,t1.NOMBRE NOMBRE,t1.FICHERO FICHERO,t1.FECHA_REGISTRO
	// FECHAREGISTRO,t1.USUARIO_REGISTRO
	// USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION
	// FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION
	// ");
	// query.append("FROM LISTADOS t1 ");
	//
	// // Where clause & Params
	// Map<String, ?> mapaWhere = this.getWhereMap(listados);
	// StringBuilder where = new StringBuilder(" WHERE 1=1 ");
	// where.append(mapaWhere.get("query"));
	// query.append(where);
	//
	// List<?> params = (List<?>) mapaWhere.get("params");
	//
	// if (jqGridRequestDto != null) {
	// query = JQGridManager.getPaginationQuery(jqGridRequestDto, query,
	// BusquedasDaoImpl.ORDER_BY_WHITE_LIST);
	// }
	//
	// return (List<Listados>) this.jdbcTemplate.query(query.toString(),
	// this.rwMap, params.toArray());
	// }

	/**
	 * Finds rows in the Listados table using like.
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Listados>
	 */
	@Transactional(readOnly = true)
	public List<Busquedas> findAllLike(Busquedas busquedas, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT T1.ID_BUSQUEDA IDBUSQUEDA, T1.ID_LISTADO IDLISTADO, T1.NOMBRE NOMBRE, T1.SQL_LISTADO SQLLISTADO, T1.CAMPOS CAMPOS FROM BUSQUEDAS T1");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(busquedas, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, BusquedasDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Busquedas>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	// /**
	// * Counts rows in the Listados table.
	// *
	// * @param listados
	// * Listados
	// * @return Long
	// */
	// @Transactional(readOnly = true)
	// public Long findAllCount(Listados listados) {
	// StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM LISTADOS t1
	// ");
	//
	// // Where clause & Params
	// Map<String, ?> mapaWhere = this.getWhereMap(listados);
	// StringBuilder where = new StringBuilder(" WHERE 1=1 ");
	// where.append(mapaWhere.get("query"));
	// query.append(where);
	//
	// List<?> params = (List<?>) mapaWhere.get("params");
	//
	// return this.jdbcTemplate.queryForLong(query.toString(),
	// params.toArray());
	// }

	/**
	 * Counts rows in the Listados table using like.
	 *
	 * @param listados
	 *            Listados
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Busquedas busquedas, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM LISTADOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(busquedas, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Listados selected for rup_table
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Listados>>
	 */
	@Override
	public List<TableRowDto<Busquedas>> reorderSelection(Busquedas busquedas, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_LISTADO IDLISTADO,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.NOMBRE NOMBRE,t1.FICHERO FICHERO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM LISTADOS t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(busquedas, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Listados.class,
				filterParamList, "ID_LISTADO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Busquedas>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	// /**
	// * Returns a map with the needed value to create the conditions to filter
	// by
	// * the Listados entity
	// *
	// * @param listados
	// * Listados Bean with the criteria values to filter by.
	// * @return Map created with two keys key query stores the sql query syntax
	// * key params stores the parameter values to be used in the
	// * condition sentence.
	// */
	// // CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	// private Map<String, ?> getWhereMap(Listados listados) {
	//
	// StringBuilder where = new
	// StringBuilder(BusquedasDaoImpl.STRING_BUILDER_INIT);
	// List<Object> params = new ArrayList<Object>();
	//
	// if (listados != null && listados.getIdListado() != null) {
	// where.append(" AND t1.ID_LISTADO = ?");
	// params.add(listados.getIdListado());
	// }
	// if (listados != null && listados.getProcedimientos() != null
	// && null != listados.getProcedimientos().getIdProcedimiento()) {
	// where.append(" AND t1.ID_PROCEDIMIENTO = ?");
	// params.add(listados.getProcedimientos().getIdProcedimiento());
	// }
	// if (listados != null && listados.getNombre() != null) {
	// where.append(" AND t1.NOMBRE = ?");
	// params.add(listados.getNombre());
	// }
	// if (listados != null && listados.getFechaRegistro() != null) {
	// where.append(" AND t1.FECHA_REGISTRO = ?");
	// params.add(listados.getFechaRegistro());
	// }
	// if (listados != null && listados.getUsuarioRegistro() != null) {
	// where.append(" AND t1.USUARIO_REGISTRO = ?");
	// params.add(listados.getUsuarioRegistro());
	// }
	// if (listados != null && listados.getFechaUltmodificacion() != null) {
	// where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
	// params.add(listados.getFechaUltmodificacion());
	// }
	// if (listados != null && listados.getUsuarioUltmodificacion() != null) {
	// where.append(" AND t1.USUARIO_ULTMODIFICACION = ?");
	// params.add(listados.getUsuarioUltmodificacion());
	// }
	//
	// Map<String, Object> mapWhere = new HashMap<String, Object>();
	// mapWhere.put("query", where);
	// mapWhere.put("params", params);
	//
	// return mapWhere;
	// }
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Listados entity
	 *
	 * @param listados
	 *            Listados Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Busquedas busquedas, Boolean startsWith) {

		StringBuilder where = new StringBuilder(BusquedasDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();
		if (busquedas != null && busquedas.getIdBusqueda() != null) {
			where.append(" AND t1.ID_BUSQUEDA = ?");
			params.add(busquedas.getIdBusqueda());
		}
		if (busquedas != null && busquedas.getIdListado() != null) {
			where.append(" AND t1.ID_LISTADO = ?");
			params.add(busquedas.getIdListado());
		}
		if (busquedas != null && busquedas.getNombre() != null) {
			where.append(" AND UPPER(t1.NOMBRE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(busquedas.getNombre().toUpperCase() + "%");
			} else {
				params.add("%" + busquedas.getNombre().toUpperCase() + "%");
			}
			where.append(" AND t1.NOMBRE IS NOT NULL");
		}
		if (busquedas != null && busquedas.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(busquedas.getFechaRegistro());
		}
		if (busquedas != null && busquedas.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(busquedas.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + busquedas.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (busquedas != null && busquedas.getFechaUltModificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(busquedas.getFechaUltModificacion());
		}
		if (busquedas != null && busquedas.getUsuarioUltModificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(busquedas.getUsuarioUltModificacion().toUpperCase() + "%");
			} else {
				params.add("%" + busquedas.getUsuarioUltModificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
