package com.ejie.ab61.dao.configuracionEventos;

import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.configuracionEventos.ConfigEventos;

/**
 * ConfigEventosDao  
 * 
 *  
 */

public interface ConfigEventosDao {

	/**
	 * Inserts a single row in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos add(ConfigEventos configeventos);

	/**
	 * Updates a single row in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos update(ConfigEventos configeventos);

	/**
	 * Finds a single row in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos find(ConfigEventos configeventos);

	/**
	 * Encuentra la configuracion de un evento determinado por tipo evento,
	 * platea_type y platea_subtype
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos findConfiguracion(ConfigEventos configeventos);

	/**
	 * Encuentra la configuracion de un evento determinado por tipo evento,
	 * platea_type y platea_subtype e instancias., Para aportaciones de
	 * documentacion en N-Instancias
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos findConfiguracionPorInstancia(ConfigEventos configeventos);

	/**
	 * Encuentra la configuracion de un evento R02_TASK
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos findConfiguracionTareas(ConfigEventos configeventos);

	/**
	 * Removes a single row in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 */
	void remove(ConfigEventos configeventos);

	/**
	 * Finds a List of rows in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List
	 */
	List<ConfigEventos> findAll(ConfigEventos configeventos, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the ConfigEventos table using like.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<ConfigEventos> findAllLike(ConfigEventos configeventos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter in the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Counts rows in the ConfigEventos table using like.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	Long findAllLikeCount(ConfigEventos configeventos, Boolean startsWith);

	/**
	 * Counts rows in the ConfigEventos table.
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return Long
	 */
	Long findAllCount(ConfigEventos configeventos);

	/**
	 * Reorder selection.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ConfigEventos>>
	 */
	List<TableRowDto<ConfigEventos>> reorderSelection(ConfigEventos filterConfigEventos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Searches in the Usuario table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param searchConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ConfigEventos>>
	 */
	List<TableRowDto<ConfigEventos>> search(ConfigEventos filterConfigEventos, ConfigEventos searchConfigEventos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Filter in the hierarchical ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<ConfigEventos>>
	 */
	List<JerarquiaDto<ConfigEventos>> findAllLikeJerarquia(ConfigEventos filterConfigEventos,
			JQGridRequestDto jqGridRequestDto);

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	Long findAllLikeCountJerarquia(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto);

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<ConfigEventos>>
	 */
	List<TableRowDto<ConfigEventos>> findAllChild(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto);

	Integer getNumeroConfiguraciones(ConfigEventos configEventos);

}
