package com.ejie.ab61.dao.documentosPid;

import com.ejie.ab61.model.documentosPid.DocumentosPid;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * DocumentosPidDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class DocumentosPidDaoImpl implements DocumentosPidDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDDOCUMENTOPID", "IDTIPOPLANTILLA",
			"NOMBREDOCUMENTO", "OIDPID", "FECHAAPORTADOPID", "OIDDOKUSI", "APORTADOEXPEDIENTE", "FECHAREGISTRO",
			"USUARIOREGISTRO", "FECHAULTMODIFICACION", "USUARIOULTMODIFICACION" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<DocumentosPid> rwMap = new RowMapper<DocumentosPid>() {
		public DocumentosPid mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new DocumentosPid(resultSet.getInt("IDDOCUMENTOPID"), resultSet.getInt("IDTIPOPLANTILLA"),
					resultSet.getString("NOMBREDOCUMENTO"), resultSet.getString("OIDPID"),
					resultSet.getDate("FECHAAPORTADOPID"), resultSet.getString("OIDDOKUSI"),
					resultSet.getInt("APORTADOEXPEDIENTE"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"), resultSet.getDate("FECHAULTMODIFICACION"),
					resultSet.getString("USUARIOULTMODIFICACION"));
		}
	};

	private RowMapper<DocumentosPid> rwMapPK = new RowMapper<DocumentosPid>() {
		public DocumentosPid mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new DocumentosPid(resultSet.getInt("IDDOCUMENTOPID"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<DocumentosPid>> rwMapJerarquia = new RowMapper<JerarquiaDto<DocumentosPid>>() {
		public JerarquiaDto<DocumentosPid> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			DocumentosPid documentospid = new DocumentosPid(resultSet.getInt("IDDOCUMENTOPID"),
					resultSet.getInt("IDTIPOPLANTILLA"), resultSet.getString("NOMBREDOCUMENTO"),
					resultSet.getString("OIDPID"), resultSet.getDate("FECHAAPORTADOPID"),
					resultSet.getString("OIDDOKUSI"), resultSet.getInt("APORTADOEXPEDIENTE"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					resultSet.getDate("FECHAULTMODIFICACION"), resultSet.getString("USUARIOULTMODIFICACION"));

			JerarquiaDto<DocumentosPid> jerarquia = new JerarquiaDto<DocumentosPid>();
			jerarquia.setModel(documentospid);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the DocumentosPid table.
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @return DocumentosPid
	 */
	public DocumentosPid add(DocumentosPid documentospid) {

		int idDocumentoPid = this.jdbcTemplate.queryForInt("SELECT S_AB6152T00.NEXTVAL FROM DUAL");
		documentospid.setIdDocumentoPid(idDocumentoPid);

		String query = "INSERT INTO DOCUMENTOS_PID (ID_DOCUMENTO_PID, ID_TIPO_PLANTILLA, NOMBRE_DOCUMENTO, OID_PID, FECHA_APORTADO_PID, OID_DOKUSI, APORTADO_EXPEDIENTE, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?,?,?,sysdate,?)";
		this.jdbcTemplate.update(query, documentospid.getIdDocumentoPid(), documentospid.getIdTipoPlantilla(),
				documentospid.getNombreDocumento(), documentospid.getOidPid(), documentospid.getFechaAportadoPid(),
				documentospid.getOidDokusi(), documentospid.getAportadoExpediente(),
				documentospid.getUsuarioRegistro());
		return documentospid;
	}

	/**
	 * Updates a single row in the DocumentosPid table.
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @return DocumentosPid
	 */
	public DocumentosPid update(DocumentosPid documentospid) {
		String query = "UPDATE DOCUMENTOS_PID SET OID_PID=?, OID_DOKUSI=?, APORTADO_EXPEDIENTE=?, FECHA_ULTMODIFICACION=sysdate, USUARIO_ULTMODIFICACION=? WHERE ID_DOCUMENTO_PID=?";
		this.jdbcTemplate.update(query, documentospid.getOidPid(), documentospid.getOidDokusi(),
				documentospid.getAportadoExpediente(), documentospid.getUsuarioUltmodificacion(),
				documentospid.getIdDocumentoPid());
		return documentospid;
	}

	/**
	 * Finds a single row in the DocumentosPid table.
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @return DocumentosPid
	 */
	@Transactional(readOnly = true)
	public DocumentosPid find(DocumentosPid documentospid) {
		String query = "SELECT t1.ID_DOCUMENTO_PID IDDOCUMENTOPID, t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA, t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO, t1.OID_PID OIDPID, t1.FECHA_APORTADO_PID FECHAAPORTADOPID, t1.OID_DOKUSI OIDDOKUSI, t1.APORTADO_EXPEDIENTE APORTADOEXPEDIENTE, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM DOCUMENTOS_PID t1  WHERE t1.ID_DOCUMENTO_PID = ?  ";

		List<DocumentosPid> documentospidList = this.jdbcTemplate.query(query, this.rwMap,
				documentospid.getIdDocumentoPid());
		return (DocumentosPid) DataAccessUtils.uniqueResult(documentospidList);
	}

	@Override
	public DocumentosPid findIdByOidPid(String oidPid) {
		String query = "SELECT t1.ID_DOCUMENTO_PID IDDOCUMENTOPID FROM DOCUMENTOS_PID t1  WHERE t1.OID_PID = ?  ";

		List<DocumentosPid> documentospidList = this.jdbcTemplate.query(query, this.rwMapPK, oidPid);
		return (DocumentosPid) DataAccessUtils.uniqueResult(documentospidList);
	}

	/**
	 * Removes a single row in the DocumentosPid table.
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @return
	 */
	public void remove(DocumentosPid documentospid) {
		String query = "DELETE FROM DOCUMENTOS_PID WHERE ID_DOCUMENTO_PID=?";
		this.jdbcTemplate.update(query, documentospid.getIdDocumentoPid());
	}

	@Override
	public void removeByOidPid(String oidPid) {
		String query = "DELETE FROM DOCUMENTOS_PID WHERE OID_PID=?";
		this.jdbcTemplate.update(query, oidPid);
	}

	/**
	 * Finds a list of rows in the DocumentosPid table.
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<DocumentosPid>
	 */
	@Transactional(readOnly = true)
	public List<DocumentosPid> findAll(DocumentosPid documentospid, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_DOCUMENTO_PID IDDOCUMENTOPID,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.OID_PID OIDPID,t1.FECHA_APORTADO_PID FECHAAPORTADOPID,t1.OID_DOKUSI OIDDOKUSI,t1.APORTADO_EXPEDIENTE APORTADOEXPEDIENTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		query.append("FROM DOCUMENTOS_PID t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(documentospid);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DocumentosPidDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<DocumentosPid>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the DocumentosPid table using like.
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<DocumentosPid>
	 */
	@Transactional(readOnly = true)
	public List<DocumentosPid> findAllLike(DocumentosPid documentospid, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_DOCUMENTO_PID IDDOCUMENTOPID,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.OID_PID OIDPID,t1.FECHA_APORTADO_PID FECHAAPORTADOPID,t1.OID_DOKUSI OIDDOKUSI,t1.APORTADO_EXPEDIENTE APORTADOEXPEDIENTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		query.append("FROM DOCUMENTOS_PID t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(documentospid, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DocumentosPidDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<DocumentosPid>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the DocumentosPid table.
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(DocumentosPid documentospid) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DOCUMENTOS_PID t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(documentospid);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the DocumentosPid table using like.
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(DocumentosPid documentospid, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DOCUMENTOS_PID t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(documentospid, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of DocumentosPid selected for rup_table
	 *
	 * @param documentospid
	 *            DocumentosPid
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<DocumentosPid>>
	 */
	@Override
	public List<TableRowDto<DocumentosPid>> reorderSelection(DocumentosPid documentospid,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DOCUMENTO_PID IDDOCUMENTOPID,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.OID_PID OIDPID,t1.FECHA_APORTADO_PID FECHAAPORTADOPID,t1.OID_DOKUSI OIDDOKUSI,t1.APORTADO_EXPEDIENTE APORTADOEXPEDIENTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM DOCUMENTOS_PID t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(documentospid, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto,
				DocumentosPid.class, filterParamList, "ID_DOCUMENTO_PID");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<DocumentosPid>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            DocumentosPid
	 * @param searchParams
	 *            DocumentosPid
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<DocumentosPid>>
	 */
	@Override
	public List<TableRowDto<DocumentosPid>> search(DocumentosPid filterParams, DocumentosPid searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DOCUMENTO_PID IDDOCUMENTOPID,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.OID_PID OIDPID,t1.FECHA_APORTADO_PID FECHAAPORTADOPID,t1.OID_DOKUSI OIDDOKUSI,t1.APORTADO_EXPEDIENTE APORTADOEXPEDIENTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM DOCUMENTOS_PID t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, DocumentosPid.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_DOCUMENTO_PID");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<DocumentosPid>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterdocumentospid
	 *            DocumentosPid
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(DocumentosPid filterdocumentospid, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DOCUMENTO_PID IDDOCUMENTOPID,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.OID_PID OIDPID,t1.FECHA_APORTADO_PID FECHAAPORTADOPID,t1.OID_DOKUSI OIDDOKUSI,t1.APORTADO_EXPEDIENTE APORTADOEXPEDIENTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM DOCUMENTOS_PID t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterdocumentospid, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, DocumentosPid.class,
				query, params, "ID_DOCUMENTO_PID");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical DocumentosPid table.
	 *
	 * @param filterDocumentosPid
	 *            DocumentosPid
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<DocumentosPid>>
	 */
	@Override
	public List<JerarquiaDto<DocumentosPid>> findAllLikeJerarquia(DocumentosPid filterDocumentosPid,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_DOCUMENTO_PID IDDOCUMENTOPID,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.NOMBRE_DOCUMENTO NOMBREDOCUMENTO,t1.OID_PID OIDPID,t1.FECHA_APORTADO_PID FECHAAPORTADOPID,t1.OID_DOKUSI OIDDOKUSI,t1.APORTADO_EXPEDIENTE APORTADOEXPEDIENTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM DOCUMENTOS_PID t1 ");
		List<String> from = new ArrayList<String>();
		from.add("DOCUMENTOS_PID");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDocumentosPid, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_DOCUMENTO_PID", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					DocumentosPidDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterDocumentosPid
	 *            DocumentosPid
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(DocumentosPid filterDocumentosPid, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM DOCUMENTOS_PID t1 ");
		List<String> from = new ArrayList<String>();
		from.add("DOCUMENTOS_PID");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDocumentosPid, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_DOCUMENTO_PID",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterDocumentosPid
	 *            DocumentosPid
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<DocumentosPid>>
	 */
	@Override
	public List<TableRowDto<DocumentosPid>> findAllChild(DocumentosPid filterDocumentosPid,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM DOCUMENTOS_PID t1 ");
		List<String> from = new ArrayList<String>();
		from.add("DOCUMENTOS_PID");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDocumentosPid, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_DOCUMENTO_PID",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<DocumentosPid>(this.rwMapPK, "ID_DOCUMENTO_PID"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the DocumentosPid entity
	 *
	 * @param documentospid
	 *            DocumentosPid Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(DocumentosPid documentospid) {

		StringBuilder where = new StringBuilder(DocumentosPidDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (documentospid != null && documentospid.getIdDocumentoPid() != null) {
			where.append(" AND t1.ID_DOCUMENTO_PID = ?");
			params.add(documentospid.getIdDocumentoPid());
		}
		if (documentospid != null && documentospid.getIdTipoPlantilla() != null) {
			where.append(" AND t1.ID_TIPO_PLANTILLA = ?");
			params.add(documentospid.getIdTipoPlantilla());
		}
		if (documentospid != null && documentospid.getNombreDocumento() != null) {
			where.append(" AND t1.NOMBRE_DOCUMENTO = ?");
			params.add(documentospid.getNombreDocumento());
		}
		if (documentospid != null && documentospid.getOidPid() != null) {
			where.append(" AND t1.OID_PID = ?");
			params.add(documentospid.getOidPid());
		}
		if (documentospid != null && documentospid.getFechaAportadoPid() != null) {
			where.append(" AND t1.FECHA_APORTADO_PID = ?");
			params.add(documentospid.getFechaAportadoPid());
		}
		if (documentospid != null && documentospid.getOidDokusi() != null) {
			where.append(" AND t1.OID_DOKUSI = ?");
			params.add(documentospid.getOidDokusi());
		}
		if (documentospid != null && documentospid.getAportadoExpediente() != null) {
			where.append(" AND t1.APORTADO_EXPEDIENTE = ?");
			params.add(documentospid.getAportadoExpediente());
		}
		if (documentospid != null && documentospid.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(documentospid.getFechaRegistro());
		}
		if (documentospid != null && documentospid.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(documentospid.getUsuarioRegistro());
		}
		if (documentospid != null && documentospid.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(documentospid.getFechaUltmodificacion());
		}
		if (documentospid != null && documentospid.getUsuarioUltmodificacion() != null) {
			where.append(" AND t1.USUARIO_ULTMODIFICACION = ?");
			params.add(documentospid.getUsuarioUltmodificacion());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the DocumentosPid entity
	 *
	 * @param documentospid
	 *            DocumentosPid Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(DocumentosPid documentospid, Boolean startsWith) {

		StringBuilder where = new StringBuilder(DocumentosPidDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (documentospid != null && documentospid.getIdDocumentoPid() != null) {
			where.append(" AND t1.ID_DOCUMENTO_PID = ?");
			params.add(documentospid.getIdDocumentoPid());
		}
		if (documentospid != null && documentospid.getIdTipoPlantilla() != null) {
			where.append(" AND t1.ID_TIPO_PLANTILLA = ?");
			params.add(documentospid.getIdTipoPlantilla());
		}
		if (documentospid != null && documentospid.getNombreDocumento() != null) {
			where.append(" AND UPPER(t1.NOMBRE_DOCUMENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentospid.getNombreDocumento().toUpperCase() + "%");
			} else {
				params.add("%" + documentospid.getNombreDocumento().toUpperCase() + "%");
			}
			where.append(" AND t1.NOMBRE_DOCUMENTO IS NOT NULL");
		}
		if (documentospid != null && documentospid.getOidPid() != null) {
			where.append(" AND UPPER(t1.OID_PID) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentospid.getOidPid().toUpperCase() + "%");
			} else {
				params.add("%" + documentospid.getOidPid().toUpperCase() + "%");
			}
			where.append(" AND t1.OID_PID IS NOT NULL");
		}
		if (documentospid != null && documentospid.getFechaAportadoPid() != null) {
			where.append(" AND t1.FECHA_APORTADO_PID = ?");
			params.add(documentospid.getFechaAportadoPid());
		}
		if (documentospid != null && documentospid.getOidDokusi() != null) {
			where.append(" AND UPPER(t1.OID_DOKUSI) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentospid.getOidDokusi().toUpperCase() + "%");
			} else {
				params.add("%" + documentospid.getOidDokusi().toUpperCase() + "%");
			}
			where.append(" AND t1.OID_DOKUSI IS NOT NULL");
		}
		if (documentospid != null && documentospid.getAportadoExpediente() != null) {
			where.append(" AND t1.APORTADO_EXPEDIENTE = ?");
			params.add(documentospid.getAportadoExpediente());
		}
		if (documentospid != null && documentospid.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(documentospid.getFechaRegistro());
		}
		if (documentospid != null && documentospid.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentospid.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + documentospid.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (documentospid != null && documentospid.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(documentospid.getFechaUltmodificacion());
		}
		if (documentospid != null && documentospid.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(documentospid.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + documentospid.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
