package com.ejie.ab61.dao.expedientes;

import java.util.Date;
import java.util.List;

import com.ejie.ab61.model.expedientes.Expedientes;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ExpedientesDao  
 * 
 *  
 */

public interface ExpedientesDao {

	/**
	 * Inserts a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	Expedientes add(Expedientes expedientes);

	/**
	 * Updates a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	Expedientes update(Expedientes expedientes);

	/**
	 * Updates a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	Expedientes updateFolderNumber(Expedientes expedientes);

	/**
	 * Updates a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	Expedientes updateFolder(Expedientes expedientes);

	/**
	 * Finds a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	Expedientes find(Expedientes expedientes);

	/**
	 * Finds a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Expedientes
	 */
	Expedientes findByFolderId(String folderId);

	/**
	 * Removes a single row in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 */
	void remove(Expedientes expedientes);

	/**
	 * Finds a List of rows in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List
	 */
	List<Expedientes> findAll(Expedientes expedientes, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Expedientes table using like.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<Expedientes> findAllLike(Expedientes expedientes, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter in the Expedientes table.
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(Expedientes filterExpedientes, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Counts rows in the Expedientes table using like.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	Long findAllLikeCount(Expedientes expedientes, Boolean startsWith);

	/**
	 * Counts rows in the Expedientes table.
	 *
	 * @param expedientes
	 *            Expedientes
	 * @return Long
	 */
	Long findAllCount(Expedientes expedientes);

	/**
	 * Reorder selection.
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Expedientes>>
	 */
	List<TableRowDto<Expedientes>> reorderSelection(Expedientes filterExpedientes, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches in the Usuario table.
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param searchExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Expedientes>>
	 */
	List<TableRowDto<Expedientes>> search(Expedientes filterExpedientes, Expedientes searchExpedientes,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Filter in the hierarchical Expedientes table.
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Expedientes>>
	 */
	List<JerarquiaDto<Expedientes>> findAllLikeJerarquia(Expedientes filterExpedientes,
			JQGridRequestDto jqGridRequestDto);

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	Long findAllLikeCountJerarquia(Expedientes filterExpedientes, JQGridRequestDto jqGridRequestDto);

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterExpedientes
	 *            Expedientes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Expedientes>>
	 */
	List<TableRowDto<Expedientes>> findAllChild(Expedientes filterExpedientes, JQGridRequestDto jqGridRequestDto);

	/**
	 * obtenerTituloExpediente
	 * 
	 * @param folderID
	 * @return Expedientes
	 */
	public List<Expedientes> obtenerTituloExpediente(String folderID);

	/**
	 * Método que actualizar el tiutlo del expediente con el titulo que viene de
	 * Platea (Tramitagune)
	 * 
	 * @param resultado
	 * @param usuario
	 */
	void updateFolderTitle(Expedientes expediente);

	List<Expedientes> findExpedientesRelacionadosUbicacionRelacionExpediente(Integer relacionExpediente,
			Integer idExpediente, JQGridRequestDto jqGridRequestDto);

	Long findExpedientesRelacionadosUbicacionRelacionExpedienteCount(Integer relacionExpediente, Integer idExpediente);

	List<Expedientes> findExpedientesARelacionar(Expedientes filterExpedientes, Integer idExpediente,
			JQGridRequestDto jqGridRequestDto, String documentosTitular, boolean startsWith);

	Long findExpedientesARelacionarCount(Expedientes expedientes, boolean startsWith, Integer idExpediente,
			String documentosTitular);

	Expedientes relacionarExpedienteSinRelacion(Expedientes expediente, Integer idExpedienteOrigenAyudas);

	Expedientes relacionarExpediente(Expedientes expedientes);

	Expedientes relacionarExpedienteHistorico(Expedientes expedientes);

	Integer getNextRelacionExpediente();

	Expedientes eliminarRelacion(Expedientes expediente);

	Expedientes eliminarRelacionHistorico(Expedientes expediente);

	Expedientes findExpedientes(Expedientes expediente);

}
