package com.ejie.ab61.dao.expedientes;

import com.ejie.ab61.model.expedientes.RelExpTer;
import com.ejie.ab61.model.terceros.DatosTerceros;

import com.ejie.ab61.model.terceros.TiposTercero;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;

import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * RelExpTerDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class RelExpTerDaoImpl implements RelExpTerDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDRELEXPTER", "IDEXPEDIENTE", "IDTIPOTERCERO",
			"PARTICIPANTID", "FECHAREGISTRO", "USUARIOREGISTRO", "DTIDTERCERO", "DTDOCUMENTO", "DTIDTIPODOCUMENTO",
			"DTNOMBRE", "DTAPELLIDO1", "DTAPELLIDO2", "DTIDSEXO", "DTTELEFONO", "DTEMAIL", "DTIDIOMA",
			"DTTIPOPARTICIPANTE", "DTFECHAREGISTRO", "DTUSUARIOREGISTRO", "DTFECHAULTMODIFICACION",
			"DTUSUARIOULTMODIFICACION", "DTIDREPRESENTANTE" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<RelExpTer> rwMap = new RowMapper<RelExpTer>() {
		public RelExpTer mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			RelExpTer relExpTer = new RelExpTer();
			relExpTer.setIdRelExpTer(resultSet.getInt("IDRELEXPTER"));
			relExpTer.setIdExpediente(resultSet.getInt("IDEXPEDIENTE"));
			relExpTer.setParticipantId(resultSet.getString("PARTICIPANTID"));
			relExpTer.setFechaRegistro(resultSet.getDate("FECHAREGISTRO"));
			relExpTer.setUsuarioRegistro(resultSet.getString("USUARIOREGISTRO"));

			DatosTerceros datosTercerosAux = new DatosTerceros();
			datosTercerosAux.setIdTercero(resultSet.getInt("DTIDTERCERO"));
			datosTercerosAux.setDocumento(resultSet.getString("DTDOCUMENTO"));
			datosTercerosAux.setIdTipoDocumento(resultSet.getLong("DTIDTIPODOCUMENTO"));
			datosTercerosAux.setNombre(resultSet.getString("DTNOMBRE"));
			datosTercerosAux.setApellido1(resultSet.getString("DTAPELLIDO1"));
			datosTercerosAux.setApellido2(resultSet.getString("DTAPELLIDO2"));
			datosTercerosAux.setIdSexo(resultSet.getLong("DTIDSEXO"));
			datosTercerosAux.setTelefono(resultSet.getLong("DTTELEFONO"));
			datosTercerosAux.setEmail(resultSet.getString("DTEMAIL"));
			datosTercerosAux.setIdioma(resultSet.getInt("DTIDIOMA"));
			datosTercerosAux.setTipoParticipante(resultSet.getInt("DTTIPOPARTICIPANTE"));
			datosTercerosAux.setFechaRegistro(resultSet.getDate("DTFECHAREGISTRO"));
			datosTercerosAux.setUsuarioRegistro(resultSet.getString("DTUSUARIOREGISTRO"));
			datosTercerosAux.setFechaUltmodificacion(resultSet.getDate("DTFECHAULTMODIFICACION"));
			datosTercerosAux.setUsuarioUltmodificacion(resultSet.getString("DTUSUARIOULTMODIFICACION"));
			relExpTer.setDatosTerceros(datosTercerosAux);

			TiposTercero tiposTerceroAux = new TiposTercero();
			tiposTerceroAux.setIdTipoTercero(resultSet.getLong("IDTIPOTERCERO"));
			relExpTer.setTiposTercero(tiposTerceroAux);

			return relExpTer;
		}
	};

	private RowMapper<RelExpTer> rwMapIdTercero = new RowMapper<RelExpTer>() {
		public RelExpTer mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			RelExpTer ter = new RelExpTer();
			DatosTerceros datosTercero = new DatosTerceros();
			TiposTercero tiposTercero = new TiposTercero();
			tiposTercero.setIdTipoTercero(resultSet.getLong("IDTIPOTERCERO"));
			tiposTercero.setRole(resultSet.getString("ROLE"));
			datosTercero.setIdTercero(resultSet.getInt("DATOSTERCEROSIDTERCERO"));
			datosTercero.setTipoParticipante(resultSet.getInt("DATOSTERCEROSTIPOPARTICIPANTE"));
			ter.setDatosTerceros(datosTercero);
			ter.setTiposTercero(tiposTercero);

			return ter;
		}
	};

	private RowMapper<RelExpTer> rwMapParticipantId = new RowMapper<RelExpTer>() {
		public RelExpTer mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			RelExpTer ter = new RelExpTer();
			DatosTerceros datosTercero = new DatosTerceros();
			datosTercero.setIdTercero(resultSet.getInt("DATOSTERCEROSIDTERCERO"));
			datosTercero.setDocumento(resultSet.getString("DOCUMENTO"));
			datosTercero.setTipoParticipante(resultSet.getInt("TIPOPARTICIPANTE"));
			ter.setDatosTerceros(datosTercero);
			ter.setParticipantId(resultSet.getString("PARTICIPANTID"));
			return ter;
		}
	};

	private RowMapper<RelExpTer> rwMapParticipantes = new RowMapper<RelExpTer>() {
		public RelExpTer mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			RelExpTer ter = new RelExpTer();
			DatosTerceros datosTercero = new DatosTerceros();
			TiposTercero tiposTercero = new TiposTercero();
			tiposTercero.setIdTipoTercero(resultSet.getLong("IDTIPOTERCERO"));
			tiposTercero.setDescripcionCastellano(resultSet.getString("DESCRIPCIONCASTELLANO"));
			tiposTercero.setDescripcionEuskera(resultSet.getString("DESCRIPCIONEUSKERA"));

			datosTercero.setIdTercero(resultSet.getInt("DATOSTERCEROSIDTERCERO"));
			datosTercero.setNombre(resultSet.getString("NOMBRE"));
			datosTercero.setApellido1(resultSet.getString("APELLIDO1"));
			datosTercero.setApellido2(resultSet.getString("APELLIDO2"));
			datosTercero.setDocumento(resultSet.getString("DOCUMENTO"));
			datosTercero.setTipoParticipante(resultSet.getInt("DATOSTERCEROSTIPOPARTICIPANTE"));
			ter.setDatosTerceros(datosTercero);
			ter.setTiposTercero(tiposTercero);
			ter.setParticipantId(resultSet.getString("PARTICIPANTID"));
			ter.setOrden(resultSet.getInt("ORDEN"));

			return ter;
		}
	};

	private RowMapper<RelExpTer> rwMapParticipaciones = new RowMapper<RelExpTer>() {
		public RelExpTer mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			RelExpTer ter = new RelExpTer();
			DatosTerceros datosTercero = new DatosTerceros();
			TiposTercero tiposTercero = new TiposTercero();
			tiposTercero.setIdTipoTercero(resultSet.getLong("IDTIPOTERCERO"));
			tiposTercero.setDescripcionCastellano(resultSet.getString("DESCRIPCIONCASTELLANO"));
			tiposTercero.setDescripcionEuskera(resultSet.getString("DESCRIPCIONEUSKERA"));

			datosTercero.setIdTercero(resultSet.getInt("DATOSTERCEROSIDTERCERO"));
			datosTercero.setNombre(resultSet.getString("NOMBRE"));
			datosTercero.setApellido1(resultSet.getString("APELLIDO1"));
			datosTercero.setApellido2(resultSet.getString("APELLIDO2"));
			datosTercero.setIdSexo(resultSet.getLong("IDSEXO"));
			datosTercero.setDocumento(resultSet.getString("DOCUMENTO"));
			datosTercero.setIdTipoDocumento(resultSet.getLong("IDTIPODOCUMENTO"));
			datosTercero.setIdioma(resultSet.getInt("IDIOMA"));
			datosTercero.setTipoParticipante(resultSet.getInt("DATOSTERCEROSTIPOPARTICIPANTE"));
			ter.setDatosTerceros(datosTercero);
			ter.setTiposTercero(tiposTercero);
			ter.setParticipantId(resultSet.getString("PARTICIPANTID"));
			ter.setOrden(resultSet.getInt("ORDEN"));

			return ter;
		}
	};
	private RowMapper<RelExpTer> rwMapPK = new RowMapper<RelExpTer>() {
		public RelExpTer mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new RelExpTer(resultSet.getInt("IDRELEXPTER"));
		}
	};

	private RowMapper<RelExpTer> rwMapValorParticipacion = new RowMapper<RelExpTer>() {
		public RelExpTer mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			RelExpTer ter = new RelExpTer();
			ter.setParticipantId(resultSet.getString("PARTICIPANTID"));
			ter.setIdRelExpTer(resultSet.getInt("IDRELEXPTER"));

			DatosTerceros datosTerceroAux = new DatosTerceros();
			datosTerceroAux.setIdTercero(resultSet.getInt("IDTERCERO"));
			datosTerceroAux.setDocumento(resultSet.getString("DOCUMENTO"));

			ter.setDatosTerceros(datosTerceroAux);

			return ter;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the RelExpTer table.
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	public RelExpTer add(RelExpTer relexpter) {
		String query = "INSERT INTO REL_EXP_TER (ID_REL_EXP_TER, ID_TERCERO, ID_EXPEDIENTE, ID_TIPO_TERCERO, PARTICIPANT_ID, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (AB61.S_AB6134T00.NEXTVAL,?,?,?,?,?,?)";
		Object getDatosTercerosIdTerceroAux = null;
		if (relexpter.getDatosTerceros() != null && relexpter.getDatosTerceros().getIdTercero() != null) {
			getDatosTercerosIdTerceroAux = relexpter.getDatosTerceros().getIdTercero();
		}
		Object getTiposTerceroIdTipoTerceroAux = null;
		if (relexpter.getTiposTercero() != null && relexpter.getTiposTercero().getIdTipoTercero() != null) {
			getTiposTerceroIdTipoTerceroAux = relexpter.getTiposTercero().getIdTipoTercero();
		}
		this.jdbcTemplate.update(query, getDatosTercerosIdTerceroAux, relexpter.getIdExpediente(),
				getTiposTerceroIdTipoTerceroAux, relexpter.getParticipantId(), relexpter.getFechaRegistro(),
				relexpter.getUsuarioRegistro());
		return relexpter;
	}

	/**
	 * Updates a single row in the RelExpTer table.
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	public RelExpTer update(RelExpTer relexpter) {
		String query = "UPDATE REL_EXP_TER SET ID_TERCERO=?, ID_EXPEDIENTE=?, ID_TIPO_TERCERO=?, FECHA_REGISTRO=?, USUARIO_REGISTRO=? WHERE ID_REL_EXP_TER=?";
		Object getDatosTercerosIdTerceroAux = null;
		if (relexpter.getDatosTerceros() != null && relexpter.getDatosTerceros().getIdTercero() != null) {
			getDatosTercerosIdTerceroAux = relexpter.getDatosTerceros().getIdTercero();
		}
		Object getTiposTerceroIdTipoTerceroAux = null;
		if (relexpter.getTiposTercero() != null && relexpter.getTiposTercero().getIdTipoTercero() != null) {
			getTiposTerceroIdTipoTerceroAux = relexpter.getTiposTercero().getIdTipoTercero();
		}
		this.jdbcTemplate.update(query, getDatosTercerosIdTerceroAux, relexpter.getIdExpediente(),
				getTiposTerceroIdTipoTerceroAux, relexpter.getFechaRegistro(), relexpter.getUsuarioRegistro(),
				relexpter.getIdRelExpTer());
		return relexpter;
	}

	/**
	 * Update tipo de una relacion
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	public RelExpTer updateByParticipantId(RelExpTer relexpter) {
		String query = "UPDATE REL_EXP_TER SET ID_TIPO_TERCERO=?, FECHA_ULTMODIFICACION =?, USUARIO_ULTMODIFICACION =? WHERE PARTICIPANT_ID =?";

		Object getTiposTerceroIdTipoTerceroAux = null;
		if (relexpter.getTiposTercero() != null && relexpter.getTiposTercero().getIdTipoTercero() != null) {
			getTiposTerceroIdTipoTerceroAux = relexpter.getTiposTercero().getIdTipoTercero();
		}
		this.jdbcTemplate.update(query, getTiposTerceroIdTipoTerceroAux, relexpter.getFechaUltmodificacion(),
				relexpter.getUsuarioUltmodificacion(), relexpter.getParticipantId());
		return relexpter;
	}

	/**
	 * Finds a single row in the RelExpTer table.
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	@Transactional(readOnly = true)
	public RelExpTer find(RelExpTer relexpter) {
		StringBuilder query = new StringBuilder();

		query.append(
				"SELECT t1.ID_REL_EXP_TER IDRELEXPTER, t1.ID_EXPEDIENTE IDEXPEDIENTE, t1.ID_TIPO_TERCERO IDTIPOTERCERO, t1.PARTICIPANT_ID PARTICIPANTID, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO");
		query.append(
				", t2.ID_TERCERO DTIDTERCERO, t2.DOCUMENTO DTDOCUMENTO, t2.ID_TIPO_DOCUMENTO DTIDTIPODOCUMENTO, t2.NOMBRE DTNOMBRE, t2.APELLIDO1 DTAPELLIDO1, t2.APELLIDO2 DTAPELLIDO2");
		query.append(
				", t2.ID_SEXO DTIDSEXO, t2.TELEFONO DTTELEFONO, t2.EMAIL DTEMAIL, t2.IDIOMA DTIDIOMA, t2.TIPO_PARTICIPANTE DTTIPOPARTICIPANTE, t2.FECHA_REGISTRO DTFECHAREGISTRO");
		query.append(
				", t2.USUARIO_REGISTRO DTUSUARIOREGISTRO, t2.FECHA_ULTMODIFICACION DTFECHAULTMODIFICACION, t2.USUARIO_ULTMODIFICACION DTUSUARIOULTMODIFICACION, t2.ID_REPRESENTANTE DTIDREPRESENTANTE");
		query.append(" FROM REL_EXP_TER t1, DATOS_TERCEROS t2");
		query.append(" WHERE t1.ID_REL_EXP_TER = ? AND t1.ID_TERCERO       = t2.ID_TERCERO(+)");

		List<RelExpTer> relexpterList = this.jdbcTemplate.query(query.toString(), this.rwMap,
				relexpter.getIdRelExpTer());
		return (RelExpTer) DataAccessUtils.uniqueResult(relexpterList);
	}

	/**
	 * Finds lista de parcicipaciones de la tabla RelExpTer
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	@Transactional(readOnly = true)
	public List<RelExpTer> findParticipantId(Integer idExpediente) {
		String query = "SELECT t1.PARTICIPANT_ID AS PARTICIPANTID, t1.ID_TERCERO AS DATOSTERCEROSIDTERCERO, t2.DOCUMENTO AS DOCUMENTO, t2.TIPO_PARTICIPANTE AS TIPOPARTICIPANTE FROM AB61.REL_EXP_TER t1, AB61.DATOS_TERCEROS t2 WHERE t1.ID_TERCERO = t2.ID_TERCERO AND t1.ID_EXPEDIENTE = ? AND t1.PARTICIPANT_ID IS NOT NULL";
		List<RelExpTer> relexpterList = this.jdbcTemplate.query(query, this.rwMapParticipantId, idExpediente);

		return relexpterList;
	}

	/**
	 * Finds lista de participantes de la tabla RelExpTer
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	@Transactional(readOnly = true)
	public List<RelExpTer> findParticipantes(Integer idExpediente) {
		String query = "SELECT  T1.ID_TIPO_TERCERO AS IDTIPOTERCERO,T3.DESCRIPCION_CASTELLANO AS DESCRIPCIONCASTELLANO, T3.DESCRIPCION_EUSKERA AS DESCRIPCIONEUSKERA, T1.PARTICIPANT_ID AS PARTICIPANTID, T1.ID_TERCERO AS DATOSTERCEROSIDTERCERO , T2.NOMBRE AS NOMBRE, T2.APELLIDO1 AS APELLIDO1, T2.APELLIDO2 AS APELLIDO2, T2.TIPO_PARTICIPANTE AS DATOSTERCEROSTIPOPARTICIPANTE , T2.DOCUMENTO AS DOCUMENTO ,DECODE(  T1.ID_TIPO_TERCERO, 1,1 ,2,9,3,2,4,3,5,4,6,8,7,6,8,7,5)AS ORDEN FROM   AB61.REL_EXP_TER T1, AB61.DATOS_TERCEROS T2 , AB61.TIPOS_TERCERO T3  WHERE  T1.ID_EXPEDIENTE = ? AND T1.ID_TERCERO = T2.ID_TERCERO AND T1.ID_TIPO_TERCERO =  T3.ID_TIPO_TERCERO AND T1.PARTICIPANT_ID IS NOT NULL ORDER BY ORDEN , PARTICIPANTID , DATOSTERCEROSTIPOPARTICIPANTE  ASC";
		List<RelExpTer> relexpterList = this.jdbcTemplate.query(query, this.rwMapParticipantes, idExpediente);

		return relexpterList;
	}

	/**
	 * Finds lista de participantes de la tabla RelExpTer
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	@Transactional(readOnly = true)
	public List<RelExpTer> findParticipaciones(Integer idExpediente) {
		String query = "SELECT  T1.ID_TIPO_TERCERO AS IDTIPOTERCERO,T3.DESCRIPCION_CASTELLANO AS DESCRIPCIONCASTELLANO, T3.DESCRIPCION_EUSKERA AS DESCRIPCIONEUSKERA, T1.PARTICIPANT_ID AS PARTICIPANTID, T1.ID_TERCERO AS DATOSTERCEROSIDTERCERO , T2.NOMBRE AS NOMBRE, T2.APELLIDO1 AS APELLIDO1, T2.APELLIDO2 AS APELLIDO2, T2.ID_SEXO AS IDSEXO, T2.TIPO_PARTICIPANTE AS DATOSTERCEROSTIPOPARTICIPANTE , T2.DOCUMENTO AS DOCUMENTO, T2.ID_TIPO_DOCUMENTO AS IDTIPODOCUMENTO, T2.IDIOMA AS IDIOMA, DECODE(  T1.ID_TIPO_TERCERO, 1,1 ,2,9,3,2,4,3,5,4,6,8,7,6,8,7,5)AS ORDEN FROM   AB61.REL_EXP_TER T1, AB61.DATOS_TERCEROS T2 , AB61.TIPOS_TERCERO T3  WHERE  T1.ID_EXPEDIENTE = ? AND T1.ID_TERCERO = T2.ID_TERCERO AND T1.ID_TIPO_TERCERO =  T3.ID_TIPO_TERCERO AND T1.PARTICIPANT_ID IS NOT NULL ORDER BY ORDEN , PARTICIPANTID , DATOSTERCEROSTIPOPARTICIPANTE  ASC";
		List<RelExpTer> relexpterList = this.jdbcTemplate.query(query, this.rwMapParticipaciones, idExpediente);

		return relexpterList;
	}

	/**
	 * buscar valor de participaciones
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	@Transactional(readOnly = true)
	public List<RelExpTer> findValorParticipant(Integer idExpediente) {
		// String query = "SELECT T1.PARTICIPANT_ID PARTICIPANTID FROM
		// AB61.REL_EXP_TER T1 WHERE T1.ID_EXPEDIENTE = ? AND
		// (T1.ID_TIPO_TERCERO = 1 OR T1.ID_TIPO_TERCERO = 3) ORDER BY
		// T1.PARTICIPANT_ID";
		String query = "SELECT T1.ID_REL_EXP_TER IDRELEXPTER, T1.PARTICIPANT_ID PARTICIPANTID, t1.id_tercero IDTERCERO,t2.DOCUMENTO DOCUMENTO FROM REL_EXP_TER T1, DATOS_TERCEROS t2 WHERE t1.id_tercero = T2.ID_TERCERO AND T1.ID_EXPEDIENTE = ?  AND (T1.ID_TIPO_TERCERO = 1 OR T1.ID_TIPO_TERCERO = 3) order by T1.PARTICIPANT_ID";
		List<RelExpTer> relexpterList = this.jdbcTemplate.query(query, this.rwMapValorParticipacion, idExpediente);

		return relexpterList;
	}

	/**
	 * Busca los id terceros en la tabla de Rel_Exp_Ter
	 *
	 * @param idExpediente
	 *            Integer
	 * 
	 * @return List
	 */
	@Transactional(readOnly = true)
	public List<RelExpTer> findIdTerceros(Integer idExpediente) {

		String query = "SELECT  T1.ID_TERCERO   AS DATOSTERCEROSIDTERCERO, T1.PARTICIPANT_ID PARTICIPANTID, T2.TIPO_PARTICIPANTE AS DATOSTERCEROSTIPOPARTICIPANTE,  T3.ROLE  AS ROLE , T1.ID_TIPO_TERCERO   AS IDTIPOTERCERO FROM AB61.REL_EXP_TER T1, AB61.DATOS_TERCEROS T2, AB61.TIPOS_TERCERO T3 WHERE T1.ID_EXPEDIENTE = ? AND T2.ID_TERCERO      = T1.ID_TERCERO AND T1.ID_TIPO_TERCERO = T3.ID_TIPO_TERCERO AND PARTICIPANT_ID IS NOT NULL ";
		List<RelExpTer> relexpterList = this.jdbcTemplate.query(query, this.rwMapIdTercero, idExpediente);
		return relexpterList;

	}

	/**
	 * Update valor de participacionid
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return RelExpTer
	 */
	public RelExpTer updateByParticipantValue(RelExpTer relexpter) {
		String query = "UPDATE REL_EXP_TER SET PARTICIPANT_ID =?, FECHA_ULTMODIFICACION =?, USUARIO_ULTMODIFICACION =? WHERE ID_TERCERO =?";

		this.jdbcTemplate.update(query, relexpter.getParticipantId(), relexpter.getFechaUltmodificacion(),
				relexpter.getUsuarioUltmodificacion(), relexpter.getDatosTerceros().getIdTercero());
		return relexpter;
	}

	/**
	 * Removes a single row in the RelExpTer table.
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return
	 */
	public void remove(RelExpTer relexpter) {
		String query = "DELETE FROM REL_EXP_TER WHERE ID_REL_EXP_TER=?";
		this.jdbcTemplate.update(query, relexpter.getIdRelExpTer());
	}

	/**
	 * Remove a single row in the RelExpTer dado un idTercero
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return
	 */
	public void removeidTercero(Integer idTercero) {
		String query = "DELETE FROM REL_EXP_TER WHERE ID_TERCERO = ?";
		this.jdbcTemplate.update(query, idTercero);
	}

	/**
	 * Finds a list of rows in the RelExpTer table.
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<RelExpTer>
	 */
	@Transactional(readOnly = true)
	public List<RelExpTer> findAll(RelExpTer relexpter, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_REL_EXP_TER IDRELEXPTER,t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_TIPO_TERCERO IDTIPOTERCERO, t1.PARTICIPANT_ID PARTICIPANTID,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_TERCERO DTIDTERCERO,t2.DOCUMENTO DTDOCUMENTO,t2.ID_TIPO_DOCUMENTO DTIDTIPODOCUMENTO,t2.NOMBRE DTNOMBRE,t2.APELLIDO1 DTAPELLIDO1,t2.APELLIDO2 DTAPELLIDO2,t2.ID_SEXO DTIDSEXO,t2.TELEFONO DTTELEFONO,t2.EMAIL DTEMAIL,t2.IDIOMA DTIDIOMA,t2.TIPO_PARTICIPANTE DTTIPOPARTICIPANTE,t2.FECHA_REGISTRO DTFECHAREGISTRO,t2.USUARIO_REGISTRO DTUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DTFECHAULTMODIFICACION,t2.USUARIO_ULTMODIFICACION DTUSUARIOULTMODIFICACION,t2.ID_REPRESENTANTE DTIDREPRESENTANTE ");
		query.append("FROM REL_EXP_TER t1 ,DATOS_TERCEROS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(relexpter);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_TERCERO= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, RelExpTerDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<RelExpTer>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the RelExpTer table using like.
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<RelExpTer>
	 */
	@Transactional(readOnly = true)
	public List<RelExpTer> findAllLike(RelExpTer relexpter, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_REL_EXP_TER IDRELEXPTER,t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_TIPO_TERCERO IDTIPOTERCERO, t1.PARTICIPANT_ID PARTICIPANTID,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_TERCERO DTIDTERCERO,t2.DOCUMENTO DTDOCUMENTO,t2.ID_TIPO_DOCUMENTO DTIDTIPODOCUMENTO,t2.NOMBRE DTNOMBRE,t2.APELLIDO1 DTAPELLIDO1,t2.APELLIDO2 DTAPELLIDO2,t2.ID_SEXO DTIDSEXO,t2.TELEFONO DTTELEFONO,t2.EMAIL DTEMAIL,t2.IDIOMA DTIDIOMA,t2.TIPO_PARTICIPANTE DTTIPOPARTICIPANTE,t2.FECHA_REGISTRO DTFECHAREGISTRO,t2.USUARIO_REGISTRO DTUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DTFECHAULTMODIFICACION,t2.USUARIO_ULTMODIFICACION DTUSUARIOULTMODIFICACION,t2.ID_REPRESENTANTE DTIDREPRESENTANTE ");
		query.append("FROM REL_EXP_TER t1 ,DATOS_TERCEROS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(relexpter, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_TERCERO= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, RelExpTerDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<RelExpTer>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Busca los id terceros en la tabla de Rel_Exp_Ter
	 *
	 * @param idExpediente
	 *            Integer
	 * @param partipant_id
	 *            String
	 * @param role
	 *            String
	 * 
	 * @return List
	 */
	@Transactional(readOnly = true)
	public List<RelExpTer> findIdTerceros(Integer idExpediente, String partipant_id) {

		String query = "SELECT T1.ID_TERCERO  AS DATOSTERCEROSIDTERCERO, T2.TIPO_PARTICIPANTE AS DATOSTERCEROSTIPOPARTICIPANTE, T3.ROLE AS ROLE , T1.ID_TIPO_TERCERO   AS IDTIPOTERCERO FROM AB61.REL_EXP_TER T1, AB61.DATOS_TERCEROS T2, AB61.TIPOS_TERCERO T3 WHERE T1.ID_EXPEDIENTE = ? AND T1.PARTICIPANT_ID  = ? AND T2.ID_TERCERO = T1.ID_TERCERO AND T1.ID_TIPO_TERCERO = T3.ID_TIPO_TERCERO";
		List<RelExpTer> relexpterList = this.jdbcTemplate.query(query, this.rwMapIdTercero, idExpediente, partipant_id);
		return relexpterList;

	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the RelExpTer table.
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(RelExpTer relexpter) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM REL_EXP_TER t1 , DATOS_TERCEROS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(relexpter);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_TERCERO= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the RelExpTer table using like.
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(RelExpTer relexpter, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM REL_EXP_TER t1 ,DATOS_TERCEROS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(relexpter, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_TERCERO= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of RelExpTer selected for rup_table
	 *
	 * @param relexpter
	 *            RelExpTer
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<RelExpTer>>
	 */
	@Override
	public List<TableRowDto<RelExpTer>> reorderSelection(RelExpTer relexpter, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_REL_EXP_TER IDRELEXPTER,t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_TIPO_TERCERO IDTIPOTERCERO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		// FROM
		query.append("FROM REL_EXP_TER t1 ,DATOS_TERCEROS t2 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(relexpter, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_TERCERO= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// ParÃ¡metros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenaciÃ³n
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, RelExpTer.class,
				filterParamList, "ID_REL_EXP_TER");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<RelExpTer>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            RelExpTer
	 * @param searchParams
	 *            RelExpTer
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<RelExpTer>>
	 */
	@Override
	public List<TableRowDto<RelExpTer>> search(RelExpTer filterParams, RelExpTer searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_REL_EXP_TER IDRELEXPTER,t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_TIPO_TERCERO IDTIPOTERCERO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		// FROM
		query.append("FROM REL_EXP_TER t1 ,DATOS_TERCEROS t2 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_TERCERO= t2.ID_TERCERO(+) ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, RelExpTer.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_REL_EXP_TER");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<RelExpTer>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterrelexpter
	 *            RelExpTer
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(RelExpTer filterrelexpter, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_REL_EXP_TER IDRELEXPTER,t1.ID_EXPEDIENTE IDEXPEDIENTE,t1.ID_TIPO_TERCERO IDTIPOTERCERO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		// FROM
		query.append("FROM REL_EXP_TER t1 ,DATOS_TERCEROS t2 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterrelexpter, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, RelExpTer.class,
				query, params, "ID_REL_EXP_TER");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterRelExpTer
	 *            RelExpTer
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<RelExpTer>>
	 */
	@Override
	public List<TableRowDto<RelExpTer>> findAllChild(RelExpTer filterRelExpTer, JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM REL_EXP_TER t1
		// ,DATOS_TERCEROS t2 ");
		List<String> from = new ArrayList<String>();
		from.add("REL_EXP_TER");
		from.add("DATOS_TERCEROS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquÃ­a
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterRelExpTer, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_REL_EXP_TER",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<RelExpTer>(this.rwMapPK, "ID_REL_EXP_TER"), params.toArray());
	}

	/*
	 * MÃ‰TODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the RelExpTer entity
	 *
	 * @param relexpter
	 *            RelExpTer Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - GeneraciÃ³n de cÃ³digo de UDA
	private Map<String, ?> getWhereMap(RelExpTer relexpter) {

		StringBuilder where = new StringBuilder(RelExpTerDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (relexpter != null && relexpter.getIdRelExpTer() != null) {
			where.append(" AND t1.ID_REL_EXP_TER = ?");
			params.add(relexpter.getIdRelExpTer());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getIdTercero() != null) {
			where.append(" AND t2.ID_TERCERO = ?");
			params.add(relexpter.getDatosTerceros().getIdTercero());
		}

		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getDocumento() != null) {
			where.append(" AND t2.DOCUMENTO = ?");
			params.add(relexpter.getDatosTerceros().getDocumento());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getIdTipoDocumento() != null) {
			where.append(" AND t2.ID_TIPO_DOCUMENTO = ?");
			params.add(relexpter.getDatosTerceros().getIdTipoDocumento());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getNombre() != null) {
			where.append(" AND t2.NOMBRE = ?");
			params.add(relexpter.getDatosTerceros().getNombre());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getApellido1() != null) {
			where.append(" AND t2.APELLIDO1 = ?");
			params.add(relexpter.getDatosTerceros().getApellido1());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getApellido2() != null) {
			where.append(" AND t2.APELLIDO2 = ?");
			params.add(relexpter.getDatosTerceros().getApellido2());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getIdSexo() != null) {
			where.append(" AND t2.ID_SEXO = ?");
			params.add(relexpter.getDatosTerceros().getIdSexo());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getTelefono() != null) {
			where.append(" AND t2.TELEFONO = ?");
			params.add(relexpter.getDatosTerceros().getTelefono());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getEmail() != null) {
			where.append(" AND t2.EMAIL = ?");
			params.add(relexpter.getDatosTerceros().getEmail());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getIdioma() != null) {
			where.append(" AND t2.IDIOMA = ?");
			params.add(relexpter.getDatosTerceros().getIdioma());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getTipoParticipante() != null) {
			where.append(" AND t2.TIPO_PARTICIPANTE = ?");
			params.add(relexpter.getDatosTerceros().getTipoParticipante());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getFechaRegistro() != null) {
			where.append(" AND t2.FECHA_REGISTRO = ?");
			params.add(relexpter.getDatosTerceros().getFechaRegistro());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getUsuarioRegistro() != null) {
			where.append(" AND t2.USUARIO_REGISTRO = ?");
			params.add(relexpter.getDatosTerceros().getUsuarioRegistro());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getFechaUltmodificacion() != null) {
			where.append(" AND t2.FECHA_ULTMODIFICACION = ?");
			params.add(relexpter.getDatosTerceros().getFechaUltmodificacion());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getUsuarioUltmodificacion() != null) {
			where.append(" AND t2.USUARIO_ULTMODIFICACION = ?");
			params.add(relexpter.getDatosTerceros().getUsuarioUltmodificacion());
		}
		if (relexpter != null && relexpter.getIdExpediente() != null) {
			where.append(" AND t1.ID_EXPEDIENTE = ?");
			params.add(relexpter.getIdExpediente());
		}
		if (relexpter != null && relexpter.getTiposTercero() != null) {
			where.append(" AND t1.ID_TIPO_TERCERO = ?");
			params.add(relexpter.getTiposTercero().getIdTipoTercero());
		}
		if (relexpter != null && relexpter.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(relexpter.getFechaRegistro());
		}
		if (relexpter != null && relexpter.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(relexpter.getUsuarioRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - GeneraciÃ³n de cÃ³digo de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the RelExpTer entity
	 *
	 * @param relexpter
	 *            RelExpTer Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - GeneraciÃ³n de cÃ³digo de UDA
	private Map<String, Object> getWhereLikeMap(RelExpTer relexpter, Boolean startsWith) {

		StringBuilder where = new StringBuilder(RelExpTerDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (relexpter != null && relexpter.getIdRelExpTer() != null) {
			where.append(" AND t1.ID_REL_EXP_TER = ?");
			params.add(relexpter.getIdRelExpTer());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getIdTercero() != null) {
			where.append(" AND t2.ID_TERCERO = ?");
			params.add(relexpter.getDatosTerceros().getIdTercero());
		}

		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getDocumento() != null) {
			where.append(" AND UPPER(t2.DOCUMENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(relexpter.getDatosTerceros().getDocumento().toUpperCase() + "%");
			} else {
				params.add("%" + relexpter.getDatosTerceros().getDocumento().toUpperCase() + "%");
			}
			where.append(" AND t2.DOCUMENTO IS NOT NULL");
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getIdTipoDocumento() != null) {
			where.append(" AND t2.ID_TIPO_DOCUMENTO = ?");
			params.add(relexpter.getDatosTerceros().getIdTipoDocumento());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getNombre() != null) {
			where.append(" AND UPPER(t2.NOMBRE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(relexpter.getDatosTerceros().getNombre().toUpperCase() + "%");
			} else {
				params.add("%" + relexpter.getDatosTerceros().getNombre().toUpperCase() + "%");
			}
			where.append(" AND t2.NOMBRE IS NOT NULL");
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getApellido1() != null) {
			where.append(" AND UPPER(t2.APELLIDO1) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(relexpter.getDatosTerceros().getApellido1().toUpperCase() + "%");
			} else {
				params.add("%" + relexpter.getDatosTerceros().getApellido1().toUpperCase() + "%");
			}
			where.append(" AND t2.APELLIDO1 IS NOT NULL");
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getApellido2() != null) {
			where.append(" AND UPPER(t2.APELLIDO2) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(relexpter.getDatosTerceros().getApellido2().toUpperCase() + "%");
			} else {
				params.add("%" + relexpter.getDatosTerceros().getApellido2().toUpperCase() + "%");
			}
			where.append(" AND t2.APELLIDO2 IS NOT NULL");
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getIdSexo() != null) {
			where.append(" AND t2.ID_SEXO = ?");
			params.add(relexpter.getDatosTerceros().getIdSexo());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getTelefono() != null) {
			where.append(" AND t2.TELEFONO = ?");
			params.add(relexpter.getDatosTerceros().getTelefono());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getEmail() != null) {
			where.append(" AND UPPER(t2.EMAIL) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(relexpter.getDatosTerceros().getEmail().toUpperCase() + "%");
			} else {
				params.add("%" + relexpter.getDatosTerceros().getEmail().toUpperCase() + "%");
			}
			where.append(" AND t2.EMAIL IS NOT NULL");
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getIdioma() != null) {
			where.append(" AND t2.IDIOMA = ?");
			params.add(relexpter.getDatosTerceros().getIdioma());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getTipoParticipante() != null) {
			where.append(" AND t2.TIPO_PARTICIPANTE = ?");
			params.add(relexpter.getDatosTerceros().getTipoParticipante());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getFechaRegistro() != null) {
			where.append(" AND t2.FECHA_REGISTRO = ?");
			params.add(relexpter.getDatosTerceros().getFechaRegistro());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t2.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(relexpter.getDatosTerceros().getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + relexpter.getDatosTerceros().getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t2.USUARIO_REGISTRO IS NOT NULL");
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getFechaUltmodificacion() != null) {
			where.append(" AND t2.FECHA_ULTMODIFICACION = ?");
			params.add(relexpter.getDatosTerceros().getFechaUltmodificacion());
		}
		if (relexpter != null && relexpter.getDatosTerceros() != null
				&& relexpter.getDatosTerceros().getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t2.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(relexpter.getDatosTerceros().getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + relexpter.getDatosTerceros().getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t2.USUARIO_ULTMODIFICACION IS NOT NULL");
		}
		if (relexpter != null && relexpter.getIdExpediente() != null) {
			where.append(" AND t1.ID_EXPEDIENTE = ?");
			params.add(relexpter.getIdExpediente());
		}
		if (relexpter != null && relexpter.getTiposTercero() != null) {
			where.append(" AND t1.ID_TIPO_TERCERO = ?");
			params.add(relexpter.getTiposTercero().getIdTipoTercero());
		}
		if (relexpter != null && relexpter.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(relexpter.getFechaRegistro());
		}
		if (relexpter != null && relexpter.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(relexpter.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + relexpter.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - GeneraciÃ³n de cÃ³digo de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
