package com.ejie.ab61.dao.listados;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.listados.Listados;
import com.ejie.ab61.model.listados.RelLisPro;
import com.ejie.ab61.model.procedimientos.Procedimientos;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ListadosDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class ListadosDaoImpl implements ListadosDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDLISTADO", "NOMBRE", "FICHERO", "FECHAREGISTRO",
			"NOMBREBUSQUEDA", "USUARIOREGISTRO", "FECHAULTMODIFICACION", "USUARIOULTMODIFICACION",
			"DESCRIPCIONCASTELLANO", "DESCRIPCIONEUSKERA" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Listados> rwMap = new RowMapper<Listados>() {
		public Listados mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			Listados listado = new Listados();
			listado.setIdListado(resultSet.getInt("IDLISTADO"));
			listado.setNombre(resultSet.getString("NOMBRE"));
			listado.setNombreBusquedas(resultSet.getString("NOMBREBUSQUEDA"));
			return listado;
		}
	};

	private RowMapper<Listados> rwMapKey = new RowMapper<Listados>() {
		public Listados mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			Listados listado = new Listados();
			listado.setIdListado(resultSet.getInt("IDLISTADO"));
			listado.setNombre(resultSet.getString("NOMBRE"));
			listado.setFicheroArray(resultSet.getBytes("FICHERO"));
			listado.setNombreFichero(resultSet.getString("NOMBREFICHERO"));
			return listado;
		}
	};

	private RowMapper<RelLisPro> rwMapKeyRelLisPro = new RowMapper<RelLisPro>() {
		public RelLisPro mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			RelLisPro relLisPro = new RelLisPro();
			relLisPro.setIdRelLisPro(resultSet.getInt("IDRELLISPRO"));
			relLisPro.setDescripcionCastellano(resultSet.getString("DESCRIPCIONCASTELLANO"));
			relLisPro.setDescripcionEuskera(resultSet.getString("DESCRIPCIONEUSKERA"));

			Procedimientos procedimientosAux = new Procedimientos();
			procedimientosAux.setIdProcedimiento(resultSet.getInt("IDPROCEDIMIENTO"));
			relLisPro.setProcedimientos(procedimientosAux);

			Listados listadosAux = new Listados();
			listadosAux.setIdListado(resultSet.getInt("IDLISTADO"));
			relLisPro.setListados(listadosAux);

			return relLisPro;
		}
	};

	private RowMapper<RelLisPro> rwMapListadosRelLisPro = new RowMapper<RelLisPro>() {
		public RelLisPro mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			RelLisPro relLisPro = new RelLisPro();
			relLisPro.setIdRelLisPro(resultSet.getInt("IDRELLISPRO"));
			relLisPro.setDescripcionCastellano(resultSet.getString("DESCRIPCIONCASTELLANO"));
			relLisPro.setDescripcionEuskera(resultSet.getString("DESCRIPCIONEUSKERA"));

			return relLisPro;
		}
	};

	private RowMapper<Listados> rwMapCombo = new RowMapper<Listados>() {
		public Listados mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			Listados listado = new Listados();
			listado.setIdListado(resultSet.getInt("IDLISTADO"));
			listado.setNombre(resultSet.getString("NOMBRE"));
			return listado;
		}
	};

	private RowMapper<Listados> rwMapPK = new RowMapper<Listados>() {
		public Listados mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Listados(resultSet.getInt("IDLISTADO"));
		}
	};

	private RowMapper<RelLisPro> rwMapPKRelLisPro = new RowMapper<RelLisPro>() {
		public RelLisPro mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new RelLisPro(resultSet.getInt("IDRELLISPRO"));
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Finds el siguiente al ultimo id de la tabla in the LISTADOS table.
	 *
	 * @return LISTADOS
	 */
	@Transactional(readOnly = true)
	public Listados findUltimoId() {
		String query = "SELECT AB61.S_AB6108T00.NEXTVAL IDLISTADO FROM dual";
		List<Listados> idListado = this.jdbcTemplate.query(query, this.rwMapPK);
		return (Listados) DataAccessUtils.uniqueResult(idListado);

	}

	/**
	 * Finds el siguiente al ultimo id de la tabla in the REL_LIS_PRO table.
	 *
	 * @return RelLisPro
	 */
	@Transactional(readOnly = true)
	public RelLisPro findUltimoIdRelLisPro() {
		String query = "SELECT AB61.S_AB6105T00.NEXTVAL IDRELLISPRO FROM dual";
		List<RelLisPro> idRelLisPro = this.jdbcTemplate.query(query, this.rwMapPKRelLisPro);
		return (RelLisPro) DataAccessUtils.uniqueResult(idRelLisPro);

	}

	/**
	 * Inserts a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	public Listados add(Listados listados) {
		Listados listadosAux = new Listados();
		listadosAux = findUltimoId();
		listados.setIdListado(listadosAux.getIdListado());
		String query = "INSERT INTO LISTADOS (ID_LISTADO, NOMBRE, NOMBRE_FICHERO, FICHERO, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, listados.getIdListado(), listados.getNombre(), listados.getNombreFichero(),
				listados.getFicheroArray(), new Date(), listados.getUsuarioRegistro());
		return listados;
	}

	/**
	 * Updates a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	public Listados update(Listados listados) {
		String query = "UPDATE LISTADOS SET NOMBRE=?, NOMBRE_FICHERO=?, FICHERO=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_LISTADO=?";
		this.jdbcTemplate.update(query, listados.getNombre(), listados.getNombreFichero(), listados.getFicheroArray(),
				new Date(), listados.getUsuarioUltmodificacion(), listados.getIdListado());
		return listados;
	}

	@Override
	public Listados updateSinFichero(Listados listados) {
		String query = "UPDATE LISTADOS SET NOMBRE=?, NOMBRE_FICHERO=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_LISTADO=?";
		this.jdbcTemplate.update(query, listados.getNombre(), listados.getNombreFichero(), new Date(),
				listados.getUsuarioUltmodificacion(), listados.getIdListado());
		return listados;
	}

	/**
	 * Finds a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Listados
	 */
	@Transactional(readOnly = true)
	public Listados find(Listados listados) {
		String query = "SELECT t1.ID_LISTADO IDLISTADO, t1.NOMBRE_FICHERO NOMBREFICHERO, t1.NOMBRE NOMBRE, t1.FICHERO FICHERO, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM LISTADOS t1  WHERE t1.ID_LISTADO = ?  ";

		List<Listados> listadosList = this.jdbcTemplate.query(query, this.rwMapKey, listados.getIdListado());
		return (Listados) DataAccessUtils.uniqueResult(listadosList);
	}

	/**
	 * Removes a single row in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return
	 */
	public void remove(Listados listados) {
		String query = "DELETE FROM LISTADOS WHERE ID_LISTADO=?";
		this.jdbcTemplate.update(query, listados.getIdListado());
	}

	/**
	 * Finds a list of rows in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Listados>
	 */
	@Transactional(readOnly = true)
	public List<Listados> findAll(Listados listados, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder("SELECT  t1.ID_LISTADO IDLISTADO, t1.NOMBRE NOMBRE");
		query.append(" FROM LISTADOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(listados);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ListadosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Listados>) this.jdbcTemplate.query(query.toString(), this.rwMapCombo, params.toArray());
	}

	/**
	 * Finds rows in the Listados table using like.
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Listados>
	 */
	@Transactional(readOnly = true)
	public List<Listados> findAllLike(Listados listados, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT T1.ID_LISTADO IDLISTADO, T1.NOMBRE NOMBRE, (SELECT LISTAGG(CONCAT(T2.NOMBRE,'|')) WITHIN GROUP (ORDER BY T2.ID_BUSQUEDA) AS CONCATDATA FROM BUSQUEDAS T2 WHERE T2.ID_LISTADO = T1.ID_LISTADO) as NOMBREBUSQUEDA FROM LISTADOS T1");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(listados, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ListadosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Listados>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Listados table.
	 *
	 * @param listados
	 *            Listados
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Listados listados) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM LISTADOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(listados);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Listados table using like.
	 *
	 * @param listados
	 *            Listados
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Listados listados, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM LISTADOS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(listados, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Listados selected for rup_table
	 *
	 * @param listados
	 *            Listados
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Listados>>
	 */
	@Override
	public List<TableRowDto<Listados>> reorderSelection(Listados listados, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_LISTADO IDLISTADO,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.NOMBRE NOMBRE,t1.FICHERO FICHERO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM LISTADOS t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(listados, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Listados.class,
				filterParamList, "ID_LISTADO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Listados>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	public Listados borrarFichero(Listados listados) {
		String query = "UPDATE LISTADOS SET FICHERO=?,NOMBRE_FICHERO=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_LISTADO=?";
		this.jdbcTemplate.update(query, listados.getFicheroArray(), "", new Date(),
				listados.getUsuarioUltmodificacion(), listados.getIdListado());
		return listados;
	}
	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Listados entity
	 *
	 * @param listados
	 *            Listados Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Listados listados) {

		StringBuilder where = new StringBuilder(ListadosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (listados != null && listados.getIdListado() != null) {
			where.append(" AND t1.ID_LISTADO = ?");
			params.add(listados.getIdListado());
		}
		if (listados != null && listados.getNombre() != null) {
			where.append(" AND t1.NOMBRE = ?");
			params.add(listados.getNombre());
		}
		if (listados != null && listados.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(listados.getFechaRegistro());
		}
		if (listados != null && listados.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(listados.getUsuarioRegistro());
		}
		if (listados != null && listados.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(listados.getFechaUltmodificacion());
		}
		if (listados != null && listados.getUsuarioUltmodificacion() != null) {
			where.append(" AND t1.USUARIO_ULTMODIFICACION = ?");
			params.add(listados.getUsuarioUltmodificacion());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Listados entity
	 *
	 * @param listados
	 *            Listados Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Listados listados, Boolean startsWith) {

		StringBuilder where = new StringBuilder(ListadosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (listados != null && listados.getIdListado() != null) {
			where.append(" AND t1.ID_LISTADO = ?");
			params.add(listados.getIdListado());
		}

		if (listados != null && listados.getNombre() != null) {
			where.append(" AND UPPER(t1.NOMBRE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(listados.getNombre().toUpperCase() + "%");
			} else {
				params.add("%" + listados.getNombre().toUpperCase() + "%");
			}
			where.append(" AND t1.NOMBRE IS NOT NULL");
		}
		if (listados != null && listados.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(listados.getFechaRegistro());
		}
		if (listados != null && listados.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(listados.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + listados.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (listados != null && listados.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(listados.getFechaUltmodificacion());
		}
		if (listados != null && listados.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(listados.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + listados.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;

	/**
	 * Finds rows in the RelLisPro table using like.
	 *
	 * @param relLisPro
	 *            RelLisPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<RelLisPro>
	 */
	@Transactional(readOnly = true)
	public List<RelLisPro> findListadosRelLisPro(RelLisPro relLisPro, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder();

		query.append(
				"SELECT t1.ID_REL_LIS_PRO as IDRELLISPRO, t1.DESCRIPCION_CASTELLANO AS DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA AS DESCRIPCIONEUSKERA");
		query.append(" FROM REL_LIS_PRO t1");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereListadosRelLisPro(relLisPro);
		StringBuilder where = new StringBuilder(" WHERE 1 = 1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ListadosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<RelLisPro>) this.jdbcTemplate.query(query.toString(), this.rwMapListadosRelLisPro,
				params.toArray());
	}

	/**
	 * Counts rows in the RelLisPro table.
	 *
	 * @param relLisPro
	 *            RelLisPro
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findListadosRelLisProCount(RelLisPro relLisPro) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM REL_LIS_PRO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereListadosRelLisPro(relLisPro);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the RelLisPro entity
	 *
	 * @param relLisPro
	 *            RelLisPro Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereListadosRelLisPro(RelLisPro relLisPro) {

		StringBuilder where = new StringBuilder(ListadosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (relLisPro != null && relLisPro.getProcedimientos() != null
				&& relLisPro.getProcedimientos().getIdProcedimiento() != null) {
			where.append(" AND t1.ID_PROCEDIMIENTO = ?");
			params.add(relLisPro.getProcedimientos().getIdProcedimiento());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Finds a single row in the RelLisPro table.
	 *
	 * @param idRelLisPro
	 *            Integer
	 * 
	 * @return RelLisPro
	 */
	@Transactional(readOnly = true)
	public RelLisPro findRelLisPro(Integer idRelLisPro) {
		StringBuilder query = new StringBuilder();

		query.append("SELECT ID_REL_LIS_PRO AS IDRELLISPRO");
		query.append(", ID_LISTADO AS IDLISTADO, ID_PROCEDIMIENTO AS IDPROCEDIMIENTO");
		query.append(", DESCRIPCION_CASTELLANO AS DESCRIPCIONCASTELLANO, DESCRIPCION_EUSKERA AS DESCRIPCIONEUSKERA");

		query.append(" FROM REL_LIS_PRO");

		query.append(" WHERE ID_REL_LIS_PRO = ?");

		List<RelLisPro> relLisProList = this.jdbcTemplate.query(query.toString(), this.rwMapKeyRelLisPro, idRelLisPro);
		return DataAccessUtils.uniqueResult(relLisProList);
	}

	/**
	 * Inserts a single row in the RelLisPro table.
	 *
	 * @param relLisPro
	 *            RelLisPro
	 * @return RelLisPro
	 */
	@Override()
	public RelLisPro addRelLisPro(RelLisPro relLisPro) {
		RelLisPro relLisProAux = findUltimoIdRelLisPro();
		relLisPro.setIdRelLisPro(relLisProAux.getIdRelLisPro());

		String query = "INSERT INTO REL_LIS_PRO (ID_REL_LIS_PRO, ID_LISTADO, ID_PROCEDIMIENTO, DESCRIPCION_CASTELLANO, DESCRIPCION_EUSKERA, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?, ?, ?, ?, ?, ?, ?)";
		this.jdbcTemplate.update(query, relLisPro.getIdRelLisPro(), relLisPro.getListados().getIdListado(),
				relLisPro.getProcedimientos().getIdProcedimiento(), relLisPro.getDescripcionCastellano(),
				relLisPro.getDescripcionEuskera(), new Date(), relLisPro.getUsuarioRegistro());
		return relLisPro;
	}

	/**
	 * Updates a single row in the RelLisPro table.
	 *
	 * @param relLisPro
	 *            RelLisPro
	 * @return RelLisPro
	 */
	@Override()
	public RelLisPro updateRelLisPro(RelLisPro relLisPro) {
		String query = "UPDATE REL_LIS_PRO SET DESCRIPCION_CASTELLANO = ?, DESCRIPCION_EUSKERA = ?, ID_LISTADO = ?, FECHA_ULTMODIFICACION = ?, USUARIO_ULTMODIFICACION = ? WHERE ID_REL_LIS_PRO = ?";
		this.jdbcTemplate.update(query, relLisPro.getDescripcionCastellano(), relLisPro.getDescripcionEuskera(),
				relLisPro.getListados().getIdListado(), new Date(), relLisPro.getUsuarioUltmodificacion(),
				relLisPro.getIdRelLisPro());
		return relLisPro;
	}

	/**
	 * Removes a single row in the RelLisPro table.
	 *
	 * @param idRelLisPro
	 *            Integer
	 * 
	 * @return
	 */
	public void removeRelLisPro(Integer idRelLisPro) {
		String query = "DELETE FROM REL_LIS_PRO WHERE ID_REL_LIS_PRO = ?";
		this.jdbcTemplate.update(query, idRelLisPro);
	}
}
