package com.ejie.ab61.dao.parametros;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.parametros.Parametros;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * ParametrosDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class ParametrosDaoImpl implements ParametrosDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDPARAMETRO", "IDBUSQUEDA", "NOMBRE",
			"TEXTOMOSTRAR", "FECHAREGISTRO", "USUARIOREGISTRO", "FECHAULTMODIFICACION", "USUARIOULTMODIFICACION",
			"TEXTOEUS" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Parametros> rwMap = new RowMapper<Parametros>() {
		public Parametros mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Parametros(resultSet.getInt("IDPARAMETRO"), resultSet.getInt("IDBUSQUEDA"),
					resultSet.getString("NOMBRE"), resultSet.getString("TEXTOMOSTRAR"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					resultSet.getDate("FECHAULTMODIFICACION"), resultSet.getString("USUARIOULTMODIFICACION"),
					resultSet.getString("TEXTOEUS"));
		}
	};

	private RowMapper<Parametros> rwMapPK = new RowMapper<Parametros>() {
		public Parametros mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Parametros(resultSet.getInt("IDPARAMETRO"));
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Parametros table.
	 *
	 * @param parametros
	 *            Parametros
	 * @return Parametros
	 */
	public Parametros add(Parametros parametros) {
		String query = "INSERT INTO PARAMETROS (ID_PARAMETRO, ID_BUSQUEDA, NOMBRE, TEXTO_MOSTRAR, TEXTO_MOSTRAR_EUS, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (AB61.S_AB6109T00.NEXTVAL,?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, parametros.getIdBusqueda(), parametros.getNombre(),
				parametros.getTextoMostrar(), parametros.getTextoMostrarEus(), new Date(),
				parametros.getUsuarioRegistro());
		return parametros;
	}

	/**
	 * Updates a single row in the Parametros table.
	 *
	 * @param parametros
	 *            Parametros
	 * @return Parametros
	 */
	public Parametros update(Parametros parametros) {
		String query = "UPDATE PARAMETROS SET ID_BUSQUEDA=?, NOMBRE=?, TEXTO_MOSTRAR=?, TEXTO_MOSTRAR_EUS=?, FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_PARAMETRO=?";
		this.jdbcTemplate.update(query, parametros.getIdBusqueda(), parametros.getNombre(),
				parametros.getTextoMostrar(), parametros.getTextoMostrarEus(), new Date(),
				parametros.getUsuarioUltmodificacion(), parametros.getIdParametro());
		return parametros;
	}

	/**
	 * Finds a single row in the Parametros table.
	 *
	 * @param parametros
	 *            Parametros
	 * @return Parametros
	 */
	@Transactional(readOnly = true)
	public Parametros find(Parametros parametros) {
		String query = "SELECT t1.ID_PARAMETRO IDPARAMETRO, t1.ID_BUSQUEDA IDBUSQUEDA, t1.NOMBRE NOMBRE, t1.TEXTO_MOSTRAR TEXTOMOSTRAR, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION, t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION, t1.TEXTO_MOSTRAR_EUS TEXTOEUS FROM PARAMETROS t1  WHERE t1.ID_PARAMETRO = ?  ";

		List<Parametros> parametrosList = this.jdbcTemplate.query(query, this.rwMap, parametros.getIdParametro());
		return (Parametros) DataAccessUtils.uniqueResult(parametrosList);
	}

	/**
	 * Removes a single row in the Parametros table.
	 *
	 * @param parametros
	 *            Parametros
	 * @return
	 */
	public void remove(Parametros parametros) {
		String query = "DELETE FROM PARAMETROS WHERE ID_PARAMETRO=?";
		this.jdbcTemplate.update(query, parametros.getIdParametro());
	}

	/**
	 * Elimina todos los parametros de una busqueda
	 *
	 * @param parametros
	 *            Parametros
	 * @return
	 */
	public void removeList(Integer idBusqueda) {
		String query = "DELETE FROM PARAMETROS WHERE ID_BUSQUEDA=?";
		this.jdbcTemplate.update(query, idBusqueda);
	}

	/**
	 * Finds a list of rows in the Parametros table.
	 *
	 * @param parametros
	 *            Parametros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Parametros>
	 */
	@Transactional(readOnly = true)
	public List<Parametros> findAll(Parametros parametros, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_PARAMETRO IDPARAMETRO,t1.ID_BUSQUEDA IDBUSQUEDA,t1.NOMBRE NOMBRE,t1.TEXTO_MOSTRAR TEXTOMOSTRAR,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION, t1.TEXTO_MOSTRAR_EUS TEXTOEUS ");
		query.append("FROM PARAMETROS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(parametros);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ParametrosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Parametros>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Parametros table using like.
	 *
	 * @param parametros
	 *            Parametros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Parametros>
	 */
	@Transactional(readOnly = true)
	public List<Parametros> findAllLike(Parametros parametros, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_PARAMETRO IDPARAMETRO,t1.ID_BUSQUEDA IDBUSQUEDA,t1.NOMBRE NOMBRE,t1.TEXTO_MOSTRAR TEXTOMOSTRAR,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION,t1.TEXTO_MOSTRAR_EUS TEXTOEUS ");
		query.append("FROM PARAMETROS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(parametros, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, ParametrosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Parametros>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Parametros table.
	 *
	 * @param parametros
	 *            Parametros
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Parametros parametros) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM PARAMETROS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(parametros);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Parametros table using like.
	 *
	 * @param parametros
	 *            Parametros
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Parametros parametros, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM PARAMETROS t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(parametros, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Parametros selected for rup_table
	 *
	 * @param parametros
	 *            Parametros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Parametros>>
	 */
	@Override
	public List<TableRowDto<Parametros>> reorderSelection(Parametros parametros, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_PARAMETRO IDPARAMETRO,t1.ID_BUSQUEDA IDBUSQUEDA,t1.NOMBRE NOMBRE,t1.TEXTO_MOSTRAR TEXTOMOSTRAR,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION ");
		// FROM
		query.append("FROM PARAMETROS t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(parametros, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Parametros.class,
				filterParamList, "ID_PARAMETRO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Parametros>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}
	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Parametros entity
	 *
	 * @param parametros
	 *            Parametros Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Parametros parametros) {

		StringBuilder where = new StringBuilder(ParametrosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (parametros != null && parametros.getIdParametro() != null) {
			where.append(" AND t1.ID_PARAMETRO = ?");
			params.add(parametros.getIdParametro());
		}
		if (parametros != null && parametros.getIdBusqueda() != null) {
			where.append(" AND t1.ID_BUSQUEDA = ?");
			params.add(parametros.getIdBusqueda());
		}
		if (parametros != null && parametros.getNombre() != null) {
			where.append(" AND t1.NOMBRE = ?");
			params.add(parametros.getNombre());
		}
		if (parametros != null && parametros.getTextoMostrar() != null) {
			where.append(" AND t1.TEXTO_MOSTRAR = ?");
			params.add(parametros.getTextoMostrar());
		}
		if (parametros != null && parametros.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(parametros.getFechaRegistro());
		}
		if (parametros != null && parametros.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(parametros.getUsuarioRegistro());
		}
		if (parametros != null && parametros.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(parametros.getFechaUltmodificacion());
		}
		if (parametros != null && parametros.getUsuarioUltmodificacion() != null) {
			where.append(" AND t1.USUARIO_ULTMODIFICACION = ?");
			params.add(parametros.getUsuarioUltmodificacion());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Parametros entity
	 *
	 * @param parametros
	 *            Parametros Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Parametros parametros, Boolean startsWith) {

		StringBuilder where = new StringBuilder(ParametrosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (parametros != null && parametros.getIdParametro() != null) {
			where.append(" AND t1.ID_PARAMETRO = ?");
			params.add(parametros.getIdParametro());
		}
		where.append(" AND t1.ID_BUSQUEDA = ?");
		params.add(parametros.getIdBusqueda());

		if (parametros != null && parametros.getNombre() != null) {
			where.append(" AND UPPER(t1.NOMBRE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(parametros.getNombre().toUpperCase() + "%");
			} else {
				params.add("%" + parametros.getNombre().toUpperCase() + "%");
			}
			where.append(" AND t1.NOMBRE IS NOT NULL");
		}
		if (parametros != null && parametros.getTextoMostrar() != null) {
			where.append(" AND UPPER(t1.TEXTO_MOSTRAR) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(parametros.getTextoMostrar().toUpperCase() + "%");
			} else {
				params.add("%" + parametros.getTextoMostrar().toUpperCase() + "%");
			}
			where.append(" AND t1.TEXTO_MOSTRAR IS NOT NULL");
		}
		if (parametros != null && parametros.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(parametros.getFechaRegistro());
		}
		if (parametros != null && parametros.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(parametros.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + parametros.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (parametros != null && parametros.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(parametros.getFechaUltmodificacion());
		}
		if (parametros != null && parametros.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(parametros.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + parametros.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
