package com.ejie.ab61.dao.plantillas;

import com.ejie.ab61.model.plantillas.RelTipPlaPro;
import com.ejie.ab61.model.plantillas.TiposPlantilla;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * RelTipPlaProDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class RelTipPlaProDaoImpl implements RelTipPlaProDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDRELTIPPLAPRO", "IDTIPOPLANTILLA",
			"IDPROCEDIMIENTO", "FECHAREGISTRO", "USUARIOREGISTRO" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<RelTipPlaPro> rwMap = new RowMapper<RelTipPlaPro>() {
		public RelTipPlaPro mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new RelTipPlaPro(resultSet.getInt("IDRELTIPPLAPRO"), resultSet.getInt("IDTIPOPLANTILLA"),
					resultSet.getInt("IDPROCEDIMIENTO"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"));
		}
	};

	private RowMapper<RelTipPlaPro> rwMapPK = new RowMapper<RelTipPlaPro>() {
		public RelTipPlaPro mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new RelTipPlaPro(resultSet.getInt("IDRELTIPPLAPRO"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<RelTipPlaPro>> rwMapJerarquia = new RowMapper<JerarquiaDto<RelTipPlaPro>>() {
		public JerarquiaDto<RelTipPlaPro> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			RelTipPlaPro reltipplapro = new RelTipPlaPro(resultSet.getInt("IDRELTIPPLAPRO"),
					resultSet.getInt("IDTIPOPLANTILLA"), resultSet.getInt("IDPROCEDIMIENTO"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"));

			JerarquiaDto<RelTipPlaPro> jerarquia = new JerarquiaDto<RelTipPlaPro>();
			jerarquia.setModel(reltipplapro);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the RelTipPlaPro table.
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @return RelTipPlaPro
	 */
	public RelTipPlaPro add(RelTipPlaPro reltipplapro) {

		reltipplapro.setIdRelTipPlaPro(this.jdbcTemplate.queryForInt("SELECT S_AB6131T00.NEXTVAL FROM DUAL"));

		String query = "INSERT INTO REL_TIP_PLA_PRO (ID_REL_TIP_PLA_PRO, ID_TIPO_PLANTILLA, ID_PROCEDIMIENTO, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,sysdate,?)";
		this.jdbcTemplate.update(query, reltipplapro.getIdRelTipPlaPro(), reltipplapro.getIdTipoPlantilla(),
				reltipplapro.getIdProcedimiento(), reltipplapro.getUsuarioRegistro());
		return reltipplapro;
	}

	/**
	 * Updates a single row in the RelTipPlaPro table.
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @return RelTipPlaPro
	 */
	public RelTipPlaPro update(RelTipPlaPro reltipplapro) {
		String query = "UPDATE REL_TIP_PLA_PRO SET ID_TIPO_PLANTILLA=?, ID_PROCEDIMIENTO=?, FECHA_REGISTRO=?, USUARIO_REGISTRO=? WHERE ID_REL_TIP_PLA_PRO=?";
		this.jdbcTemplate.update(query, reltipplapro.getIdTipoPlantilla(), reltipplapro.getIdProcedimiento(),
				reltipplapro.getFechaRegistro(), reltipplapro.getUsuarioRegistro(), reltipplapro.getIdRelTipPlaPro());
		return reltipplapro;
	}

	/**
	 * Finds a single row in the RelTipPlaPro table.
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @return RelTipPlaPro
	 */
	@Transactional(readOnly = true)
	public RelTipPlaPro find(RelTipPlaPro reltipplapro) {
		String query = "SELECT t1.ID_REL_TIP_PLA_PRO IDRELTIPPLAPRO, t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA, t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO FROM REL_TIP_PLA_PRO t1  WHERE t1.ID_REL_TIP_PLA_PRO = ?  ";

		List<RelTipPlaPro> reltipplaproList = this.jdbcTemplate.query(query, this.rwMap,
				reltipplapro.getIdRelTipPlaPro());
		return (RelTipPlaPro) DataAccessUtils.uniqueResult(reltipplaproList);
	}

	@Override
	public List<RelTipPlaPro> findByTipoPlantilla(TiposPlantilla tipoPlantilla) {
		String query = "SELECT t1.ID_REL_TIP_PLA_PRO IDRELTIPPLAPRO, t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA, t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO FROM REL_TIP_PLA_PRO t1  WHERE t1.ID_TIPO_PLANTILLA = ?  ";

		return this.jdbcTemplate.query(query, this.rwMap, tipoPlantilla.getIdTipoPlantilla());
	}

	/**
	 * Removes a single row in the RelTipPlaPro table.
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @return
	 */
	public void remove(RelTipPlaPro reltipplapro) {
		String query = "DELETE FROM REL_TIP_PLA_PRO WHERE ID_REL_TIP_PLA_PRO=?";
		this.jdbcTemplate.update(query, reltipplapro.getIdRelTipPlaPro());
	}

	@Override
	public void removeByTipoPlantilla(TiposPlantilla tiposPlantilla) {
		String query = "DELETE FROM REL_TIP_PLA_PRO WHERE ID_TIPO_PLANTILLA = ?";
		this.jdbcTemplate.update(query, tiposPlantilla.getIdTipoPlantilla());
	}

	/**
	 * Finds a list of rows in the RelTipPlaPro table.
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<RelTipPlaPro>
	 */
	@Transactional(readOnly = true)
	public List<RelTipPlaPro> findAll(RelTipPlaPro reltipplapro, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_REL_TIP_PLA_PRO IDRELTIPPLAPRO,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		query.append("FROM REL_TIP_PLA_PRO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(reltipplapro);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, RelTipPlaProDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<RelTipPlaPro>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the RelTipPlaPro table using like.
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<RelTipPlaPro>
	 */
	@Transactional(readOnly = true)
	public List<RelTipPlaPro> findAllLike(RelTipPlaPro reltipplapro, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_REL_TIP_PLA_PRO IDRELTIPPLAPRO,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		query.append("FROM REL_TIP_PLA_PRO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(reltipplapro, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, RelTipPlaProDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<RelTipPlaPro>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the RelTipPlaPro table.
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(RelTipPlaPro reltipplapro) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM REL_TIP_PLA_PRO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(reltipplapro);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the RelTipPlaPro table using like.
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(RelTipPlaPro reltipplapro, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM REL_TIP_PLA_PRO t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(reltipplapro, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of RelTipPlaPro selected for rup_table
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<RelTipPlaPro>>
	 */
	@Override
	public List<TableRowDto<RelTipPlaPro>> reorderSelection(RelTipPlaPro reltipplapro,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_REL_TIP_PLA_PRO IDRELTIPPLAPRO,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM REL_TIP_PLA_PRO t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(reltipplapro, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, RelTipPlaPro.class,
				filterParamList, "ID_REL_TIP_PLA_PRO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<RelTipPlaPro>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            RelTipPlaPro
	 * @param searchParams
	 *            RelTipPlaPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<RelTipPlaPro>>
	 */
	@Override
	public List<TableRowDto<RelTipPlaPro>> search(RelTipPlaPro filterParams, RelTipPlaPro searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_REL_TIP_PLA_PRO IDRELTIPPLAPRO,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM REL_TIP_PLA_PRO t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, RelTipPlaPro.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_REL_TIP_PLA_PRO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<RelTipPlaPro>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterreltipplapro
	 *            RelTipPlaPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(RelTipPlaPro filterreltipplapro, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_REL_TIP_PLA_PRO IDRELTIPPLAPRO,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM REL_TIP_PLA_PRO t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterreltipplapro, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, RelTipPlaPro.class,
				query, params, "ID_REL_TIP_PLA_PRO");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical RelTipPlaPro table.
	 *
	 * @param filterRelTipPlaPro
	 *            RelTipPlaPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<RelTipPlaPro>>
	 */
	@Override
	public List<JerarquiaDto<RelTipPlaPro>> findAllLikeJerarquia(RelTipPlaPro filterRelTipPlaPro,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_REL_TIP_PLA_PRO IDRELTIPPLAPRO,t1.ID_TIPO_PLANTILLA IDTIPOPLANTILLA,t1.ID_PROCEDIMIENTO IDPROCEDIMIENTO,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM REL_TIP_PLA_PRO t1 ");
		List<String> from = new ArrayList<String>();
		from.add("REL_TIP_PLA_PRO");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterRelTipPlaPro, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_REL_TIP_PLA_PRO", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					RelTipPlaProDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterRelTipPlaPro
	 *            RelTipPlaPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(RelTipPlaPro filterRelTipPlaPro, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM REL_TIP_PLA_PRO t1 ");
		List<String> from = new ArrayList<String>();
		from.add("REL_TIP_PLA_PRO");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterRelTipPlaPro, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_REL_TIP_PLA_PRO",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterRelTipPlaPro
	 *            RelTipPlaPro
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<RelTipPlaPro>>
	 */
	@Override
	public List<TableRowDto<RelTipPlaPro>> findAllChild(RelTipPlaPro filterRelTipPlaPro,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM REL_TIP_PLA_PRO t1 ");
		List<String> from = new ArrayList<String>();
		from.add("REL_TIP_PLA_PRO");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterRelTipPlaPro, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_REL_TIP_PLA_PRO",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<RelTipPlaPro>(this.rwMapPK, "ID_REL_TIP_PLA_PRO"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the RelTipPlaPro entity
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(RelTipPlaPro reltipplapro) {

		StringBuilder where = new StringBuilder(RelTipPlaProDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (reltipplapro != null && reltipplapro.getIdRelTipPlaPro() != null) {
			where.append(" AND t1.ID_REL_TIP_PLA_PRO = ?");
			params.add(reltipplapro.getIdRelTipPlaPro());
		}
		if (reltipplapro != null && reltipplapro.getIdTipoPlantilla() != null) {
			where.append(" AND t1.ID_TIPO_PLANTILLA = ?");
			params.add(reltipplapro.getIdTipoPlantilla());
		}
		if (reltipplapro != null && reltipplapro.getIdProcedimiento() != null) {
			where.append(" AND t1.ID_PROCEDIMIENTO = ?");
			params.add(reltipplapro.getIdProcedimiento());
		}
		if (reltipplapro != null && reltipplapro.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(reltipplapro.getFechaRegistro());
		}
		if (reltipplapro != null && reltipplapro.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(reltipplapro.getUsuarioRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the RelTipPlaPro entity
	 *
	 * @param reltipplapro
	 *            RelTipPlaPro Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(RelTipPlaPro reltipplapro, Boolean startsWith) {

		StringBuilder where = new StringBuilder(RelTipPlaProDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (reltipplapro != null && reltipplapro.getIdRelTipPlaPro() != null) {
			where.append(" AND t1.ID_REL_TIP_PLA_PRO = ?");
			params.add(reltipplapro.getIdRelTipPlaPro());
		}
		if (reltipplapro != null && reltipplapro.getIdTipoPlantilla() != null) {
			where.append(" AND t1.ID_TIPO_PLANTILLA = ?");
			params.add(reltipplapro.getIdTipoPlantilla());
		}
		if (reltipplapro != null && reltipplapro.getIdProcedimiento() != null) {
			where.append(" AND t1.ID_PROCEDIMIENTO = ?");
			params.add(reltipplapro.getIdProcedimiento());
		}
		if (reltipplapro != null && reltipplapro.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(reltipplapro.getFechaRegistro());
		}
		if (reltipplapro != null && reltipplapro.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(reltipplapro.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + reltipplapro.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
