package com.ejie.ab61.dao.procedimientos;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.model.procedimientos.Aplicaciones;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * AplicacionesDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class AplicacionesDaoImpl implements AplicacionesDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDAPLICACION", "DESCRIPCIONCASTELLANO",
			"DESCRIPCIONEUSKERA", "URLAPLICACION", "FECHAREGISTRO", "USUARIOREGISTRO" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Aplicaciones> rwMap = new RowMapper<Aplicaciones>() {
		public Aplicaciones mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Aplicaciones(resultSet.getInt("IDAPLICACION"), resultSet.getString("DESCRIPCIONCASTELLANO"),
					resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getString("URLAPLICACION"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"));
		}
	};

	private RowMapper<Aplicaciones> rwMapPK = new RowMapper<Aplicaciones>() {
		public Aplicaciones mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Aplicaciones(resultSet.getInt("IDAPLICACION"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<Aplicaciones>> rwMapJerarquia = new RowMapper<JerarquiaDto<Aplicaciones>>() {
		public JerarquiaDto<Aplicaciones> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Aplicaciones aplicaciones = new Aplicaciones(resultSet.getInt("IDAPLICACION"),
					resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA"),
					resultSet.getString("URLAPLICACION"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"));

			JerarquiaDto<Aplicaciones> jerarquia = new JerarquiaDto<Aplicaciones>();
			jerarquia.setModel(aplicaciones);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @return Aplicaciones
	 */
	public Aplicaciones add(Aplicaciones aplicaciones) {
		String query = "INSERT INTO APLICACIONES (ID_APLICACION, DESCRIPCION_CASTELLANO, DESCRIPCION_EUSKERA, URL_APLICACION, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, aplicaciones.getIdAplicacion(), aplicaciones.getDescripcionCastellano(),
				aplicaciones.getDescripcionEuskera(), aplicaciones.getUrlAplicacion(), aplicaciones.getFechaRegistro(),
				aplicaciones.getUsuarioRegistro());
		return aplicaciones;
	}

	/**
	 * Updates a single row in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @return Aplicaciones
	 */
	public Aplicaciones update(Aplicaciones aplicaciones) {
		String query = "UPDATE APLICACIONES SET DESCRIPCION_CASTELLANO=?, DESCRIPCION_EUSKERA=?, URL_APLICACION=?, FECHA_REGISTRO=?, USUARIO_REGISTRO=? WHERE ID_APLICACION=?";
		this.jdbcTemplate.update(query, aplicaciones.getDescripcionCastellano(), aplicaciones.getDescripcionEuskera(),
				aplicaciones.getUrlAplicacion(), aplicaciones.getFechaRegistro(), aplicaciones.getUsuarioRegistro(),
				aplicaciones.getIdAplicacion());
		return aplicaciones;
	}

	/**
	 * Finds a single row in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @return Aplicaciones
	 */
	@Transactional(readOnly = true)
	public Aplicaciones find(Aplicaciones aplicaciones) {
		String query = "SELECT t1.ID_APLICACION IDAPLICACION, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA, t1.URL_APLICACION URLAPLICACION, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO FROM APLICACIONES t1  WHERE t1.ID_APLICACION = ?  ";

		List<Aplicaciones> aplicacionesList = this.jdbcTemplate.query(query, this.rwMap,
				aplicaciones.getIdAplicacion());
		return (Aplicaciones) DataAccessUtils.uniqueResult(aplicacionesList);
	}

	/**
	 * Removes a single row in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @return
	 */
	public void remove(Aplicaciones aplicaciones) {
		String query = "DELETE FROM APLICACIONES WHERE ID_APLICACION=?";
		this.jdbcTemplate.update(query, aplicaciones.getIdAplicacion());
	}

	/**
	 * Finds a list of rows in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Aplicaciones>
	 */
	@Transactional(readOnly = true)
	public List<Aplicaciones> findAll(Aplicaciones aplicaciones, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_APLICACION IDAPLICACION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.URL_APLICACION URLAPLICACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		query.append("FROM APLICACIONES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(aplicaciones);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, AplicacionesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Aplicaciones>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Aplicaciones table using like.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Aplicaciones>
	 */
	@Transactional(readOnly = true)
	public List<Aplicaciones> findAllLike(Aplicaciones aplicaciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_APLICACION IDAPLICACION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.URL_APLICACION URLAPLICACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		query.append("FROM APLICACIONES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(aplicaciones, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, AplicacionesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Aplicaciones>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Aplicaciones aplicaciones) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM APLICACIONES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(aplicaciones);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Aplicaciones table using like.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Aplicaciones aplicaciones, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM APLICACIONES t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(aplicaciones, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Aplicaciones selected for rup_table
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Aplicaciones>>
	 */
	@Override
	public List<TableRowDto<Aplicaciones>> reorderSelection(Aplicaciones aplicaciones,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_APLICACION IDAPLICACION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.URL_APLICACION URLAPLICACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM APLICACIONES t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(aplicaciones, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Aplicaciones.class,
				filterParamList, "ID_APLICACION");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Aplicaciones>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            Aplicaciones
	 * @param searchParams
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Aplicaciones>>
	 */
	@Override
	public List<TableRowDto<Aplicaciones>> search(Aplicaciones filterParams, Aplicaciones searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_APLICACION IDAPLICACION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.URL_APLICACION URLAPLICACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM APLICACIONES t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, Aplicaciones.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_APLICACION");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Aplicaciones>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filteraplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(Aplicaciones filteraplicaciones, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_APLICACION IDAPLICACION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.URL_APLICACION URLAPLICACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		query.append("FROM APLICACIONES t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filteraplicaciones, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, Aplicaciones.class,
				query, params, "ID_APLICACION");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical Aplicaciones table.
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Aplicaciones>>
	 */
	@Override
	public List<JerarquiaDto<Aplicaciones>> findAllLikeJerarquia(Aplicaciones filterAplicaciones,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_APLICACION IDAPLICACION,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.URL_APLICACION URLAPLICACION,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM APLICACIONES t1 ");
		List<String> from = new ArrayList<String>();
		from.add("APLICACIONES");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterAplicaciones, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_APLICACION", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					AplicacionesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(Aplicaciones filterAplicaciones, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM APLICACIONES t1 ");
		List<String> from = new ArrayList<String>();
		from.add("APLICACIONES");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterAplicaciones, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_APLICACION",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Aplicaciones>>
	 */
	@Override
	public List<TableRowDto<Aplicaciones>> findAllChild(Aplicaciones filterAplicaciones,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM APLICACIONES t1 ");
		List<String> from = new ArrayList<String>();
		from.add("APLICACIONES");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterAplicaciones, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_APLICACION",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<Aplicaciones>(this.rwMapPK, "ID_APLICACION"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Aplicaciones entity
	 *
	 * @param aplicaciones
	 *            Aplicaciones Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Aplicaciones aplicaciones) {

		StringBuilder where = new StringBuilder(AplicacionesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (aplicaciones != null && aplicaciones.getIdAplicacion() != null) {
			where.append(" AND t1.ID_APLICACION = ?");
			params.add(aplicaciones.getIdAplicacion());
		}
		if (aplicaciones != null && aplicaciones.getDescripcionCastellano() != null) {
			where.append(" AND t1.DESCRIPCION_CASTELLANO = ?");
			params.add(aplicaciones.getDescripcionCastellano());
		}
		if (aplicaciones != null && aplicaciones.getDescripcionEuskera() != null) {
			where.append(" AND t1.DESCRIPCION_EUSKERA = ?");
			params.add(aplicaciones.getDescripcionEuskera());
		}
		if (aplicaciones != null && aplicaciones.getUrlAplicacion() != null) {
			where.append(" AND t1.URL_APLICACION = ?");
			params.add(aplicaciones.getUrlAplicacion());
		}
		if (aplicaciones != null && aplicaciones.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(aplicaciones.getFechaRegistro());
		}
		if (aplicaciones != null && aplicaciones.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(aplicaciones.getUsuarioRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Aplicaciones entity
	 *
	 * @param aplicaciones
	 *            Aplicaciones Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Aplicaciones aplicaciones, Boolean startsWith) {

		StringBuilder where = new StringBuilder(AplicacionesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (aplicaciones != null && aplicaciones.getIdAplicacion() != null) {
			where.append(" AND t1.ID_APLICACION = ?");
			params.add(aplicaciones.getIdAplicacion());
		}
		if (aplicaciones != null && aplicaciones.getDescripcionCastellano() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION_CASTELLANO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(aplicaciones.getDescripcionCastellano().toUpperCase() + "%");
			} else {
				params.add("%" + aplicaciones.getDescripcionCastellano().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION_CASTELLANO IS NOT NULL");
		}
		if (aplicaciones != null && aplicaciones.getDescripcionEuskera() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION_EUSKERA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(aplicaciones.getDescripcionEuskera().toUpperCase() + "%");
			} else {
				params.add("%" + aplicaciones.getDescripcionEuskera().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION_EUSKERA IS NOT NULL");
		}
		if (aplicaciones != null && aplicaciones.getUrlAplicacion() != null) {
			where.append(" AND UPPER(t1.URL_APLICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(aplicaciones.getUrlAplicacion().toUpperCase() + "%");
			} else {
				params.add("%" + aplicaciones.getUrlAplicacion().toUpperCase() + "%");
			}
			where.append(" AND t1.URL_APLICACION IS NOT NULL");
		}
		if (aplicaciones != null && aplicaciones.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(aplicaciones.getFechaRegistro());
		}
		if (aplicaciones != null && aplicaciones.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(aplicaciones.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + aplicaciones.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
