package com.ejie.ab61.dao.procedimientos;

import com.ejie.ab61.model.procedimientos.Areas;
import com.ejie.ab61.model.procedimientos.Departamentos;
import com.ejie.ab61.model.procedimientos.Direcciones;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * AreasDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class AreasDaoImpl implements AreasDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDAREA", "DESCRIPCIONCASTELLANO",
			"DESCRIPCIONEUSKERA", "FECHAREGISTRO", "USUARIOREGISTRO", "DIRECCIONESIDDIRECCION",
			"DIRECCIONESDESCCASTELLANO", "DIRECCIONESDESCEUSKERA", "DIRECCIONESFECHAREGISTRO",
			"DIRECCIONESUSUARIOREGISTRO", "DIRECCIONESIDDEPARTAMENTO" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Areas> rwMap = new RowMapper<Areas>() {
		public Areas mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Areas(resultSet.getInt("IDAREA"), resultSet.getString("DESCRIPCIONCASTELLANO"),
					resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"),
					new Direcciones(resultSet.getInt("DireccionesIDDIRECCION"),
							resultSet.getString("DireccionesDESCCASTELLANO"),
							resultSet.getString("DireccionesDESCEUSKERA"),
							resultSet.getDate("DireccionesFECHAREGISTRO"),
							resultSet.getString("DireccionesUSUARIOREGISTRO"), new Departamentos(
									resultSet.getInt("DireccionesIDDEPARTAMENTO"), null, null, null, null, null)));
		}
	};

	private RowMapper<Areas> rwMapPK = new RowMapper<Areas>() {
		public Areas mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Areas(resultSet.getInt("IDAREA"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<Areas>> rwMapJerarquia = new RowMapper<JerarquiaDto<Areas>>() {
		public JerarquiaDto<Areas> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Areas areas = new Areas(resultSet.getInt("IDAREA"), resultSet.getString("DESCRIPCIONCASTELLANO"),
					resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"),
					new Direcciones(resultSet.getInt("DireccionesIDDIRECCION"),
							resultSet.getString("DireccionesDESCCASTELLANO"),
							resultSet.getString("DireccionesDESCEUSKERA"),
							resultSet.getDate("DireccionesFECHAREGISTRO"),
							resultSet.getString("DireccionesUSUARIOREGISTRO"), new Departamentos(
									resultSet.getInt("DireccionesIDDEPARTAMENTO"), null, null, null, null, null)));

			JerarquiaDto<Areas> jerarquia = new JerarquiaDto<Areas>();
			jerarquia.setModel(areas);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Areas table.
	 *
	 * @param areas
	 *            Areas
	 * @return Areas
	 */
	public Areas add(Areas areas) {
		String query = "INSERT INTO AREAS (ID_AREA, ID_DIRECCION, DESCRIPCION_CASTELLANO, DESCRIPCION_EUSKERA, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?,?)";
		Object getDireccionesIdDireccionAux = null;
		if (areas.getDirecciones() != null && areas.getDirecciones().getIdDireccion() != null) {
			getDireccionesIdDireccionAux = areas.getDirecciones().getIdDireccion();
		}
		this.jdbcTemplate.update(query, areas.getIdArea(), getDireccionesIdDireccionAux,
				areas.getDescripcionCastellano(), areas.getDescripcionEuskera(), areas.getFechaRegistro(),
				areas.getUsuarioRegistro());
		return areas;
	}

	/**
	 * Updates a single row in the Areas table.
	 *
	 * @param areas
	 *            Areas
	 * @return Areas
	 */
	public Areas update(Areas areas) {
		String query = "UPDATE AREAS SET ID_DIRECCION=?, DESCRIPCION_CASTELLANO=?, DESCRIPCION_EUSKERA=?, FECHA_REGISTRO=?, USUARIO_REGISTRO=? WHERE ID_AREA=?";
		Object getDireccionesIdDireccionAux = null;
		if (areas.getDirecciones() != null && areas.getDirecciones().getIdDireccion() != null) {
			getDireccionesIdDireccionAux = areas.getDirecciones().getIdDireccion();
		}
		this.jdbcTemplate.update(query, getDireccionesIdDireccionAux, areas.getDescripcionCastellano(),
				areas.getDescripcionEuskera(), areas.getFechaRegistro(), areas.getUsuarioRegistro(), areas.getIdArea());
		return areas;
	}

	/**
	 * Finds a single row in the Areas table.
	 *
	 * @param areas
	 *            Areas
	 * @return Areas
	 */
	@Transactional(readOnly = true)
	public Areas find(Areas areas) {
		String query = "SELECT t1.ID_AREA IDAREA, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO, t2.ID_DIRECCION DIRECCIONESIDDIRECCION, t2.DESCRIPCION_CASTELLANO DIRECCIONESDESCCASTELLANO, t2.DESCRIPCION_EUSKERA DIRECCIONESDESCEUSKERA, t2.FECHA_REGISTRO DIRECCIONESFECHAREGISTRO, t2.USUARIO_REGISTRO DIRECCIONESUSUARIOREGISTRO, t2.ID_DEPARTAMENTO DIRECCIONESIDDEPARTAMENTO FROM AREAS t1 , DIRECCIONES t2  WHERE t1.ID_AREA = ?   AND t1.ID_DIRECCION= t2.ID_DIRECCION(+)";

		List<Areas> areasList = this.jdbcTemplate.query(query, this.rwMap, areas.getIdArea());
		return (Areas) DataAccessUtils.uniqueResult(areasList);
	}

	/**
	 * Removes a single row in the Areas table.
	 *
	 * @param areas
	 *            Areas
	 * @return
	 */
	public void remove(Areas areas) {
		String query = "DELETE FROM AREAS WHERE ID_AREA=?";
		this.jdbcTemplate.update(query, areas.getIdArea());
	}

	/**
	 * Finds a list of rows in the Areas table.
	 *
	 * @param areas
	 *            Areas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Areas>
	 */
	@Transactional(readOnly = true)
	public List<Areas> findAll(Areas areas, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_AREA IDAREA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DIRECCION DIRECCIONESIDDIRECCION,t2.DESCRIPCION_CASTELLANO DIRECCIONESDESCCASTELLANO,t2.DESCRIPCION_EUSKERA DIRECCIONESDESCEUSKERA,t2.FECHA_REGISTRO DIRECCIONESFECHAREGISTRO,t2.USUARIO_REGISTRO DIRECCIONESUSUARIOREGISTRO,t2.ID_DEPARTAMENTO DIRECCIONESIDDEPARTAMENTO ");
		query.append("FROM AREAS t1 ,DIRECCIONES t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(areas);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DIRECCION= t2.ID_DIRECCION(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, AreasDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Areas>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Areas table using like.
	 *
	 * @param areas
	 *            Areas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Areas>
	 */
	@Transactional(readOnly = true)
	public List<Areas> findAllLike(Areas areas, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_AREA IDAREA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DIRECCION DIRECCIONESIDDIRECCION,t2.DESCRIPCION_CASTELLANO DIRECCIONESDESCCASTELLANO,t2.DESCRIPCION_EUSKERA DIRECCIONESDESCEUSKERA,t2.FECHA_REGISTRO DIRECCIONESFECHAREGISTRO,t2.USUARIO_REGISTRO DIRECCIONESUSUARIOREGISTRO,t2.ID_DEPARTAMENTO DIRECCIONESIDDEPARTAMENTO ");
		query.append("FROM AREAS t1 ,DIRECCIONES t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(areas, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DIRECCION= t2.ID_DIRECCION(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, AreasDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Areas>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Areas table.
	 *
	 * @param areas
	 *            Areas
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Areas areas) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM AREAS t1 , DIRECCIONES t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(areas);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DIRECCION= t2.ID_DIRECCION(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Areas table using like.
	 *
	 * @param areas
	 *            Areas
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Areas areas, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM AREAS t1 ,DIRECCIONES t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(areas, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DIRECCION= t2.ID_DIRECCION(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Areas selected for rup_table
	 *
	 * @param areas
	 *            Areas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Areas>>
	 */
	@Override
	public List<TableRowDto<Areas>> reorderSelection(Areas areas, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_AREA IDAREA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DIRECCION DIRECCIONESIDDIRECCION,t2.DESCRIPCION_CASTELLANO DIRECCIONESDESCCASTELLANO,t2.DESCRIPCION_EUSKERA DIRECCIONESDESCEUSKERA,t2.FECHA_REGISTRO DIRECCIONESFECHAREGISTRO,t2.USUARIO_REGISTRO DIRECCIONESUSUARIOREGISTRO,t2.ID_DEPARTAMENTO DIRECCIONESIDDEPARTAMENTO ");
		// FROM
		query.append("FROM AREAS t1 ,DIRECCIONES t2 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(areas, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DIRECCION= t2.ID_DIRECCION(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Areas.class,
				filterParamList, "ID_AREA");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Areas>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            Areas
	 * @param searchParams
	 *            Areas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Areas>>
	 */
	@Override
	public List<TableRowDto<Areas>> search(Areas filterParams, Areas searchParams, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_AREA IDAREA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DIRECCION DIRECCIONESIDDIRECCION,t2.DESCRIPCION_CASTELLANO DIRECCIONESDESCCASTELLANO,t2.DESCRIPCION_EUSKERA DIRECCIONESDESCEUSKERA,t2.FECHA_REGISTRO DIRECCIONESFECHAREGISTRO,t2.USUARIO_REGISTRO DIRECCIONESUSUARIOREGISTRO,t2.ID_DEPARTAMENTO DIRECCIONESIDDEPARTAMENTO ");
		// FROM
		query.append("FROM AREAS t1 ,DIRECCIONES t2 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_DIRECCION= t2.ID_DIRECCION(+) ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, Areas.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_AREA");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Areas>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterareas
	 *            Areas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(Areas filterareas, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_AREA IDAREA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DIRECCION DIRECCIONESIDDIRECCION,t2.DESCRIPCION_CASTELLANO DIRECCIONESDESCCASTELLANO,t2.DESCRIPCION_EUSKERA DIRECCIONESDESCEUSKERA,t2.FECHA_REGISTRO DIRECCIONESFECHAREGISTRO,t2.USUARIO_REGISTRO DIRECCIONESUSUARIOREGISTRO,t2.ID_DEPARTAMENTO DIRECCIONESIDDEPARTAMENTO ");
		// FROM
		query.append("FROM AREAS t1 ,DIRECCIONES t2 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterareas, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, Areas.class, query,
				params, "ID_AREA");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical Areas table.
	 *
	 * @param filterAreas
	 *            Areas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Areas>>
	 */
	@Override
	public List<JerarquiaDto<Areas>> findAllLikeJerarquia(Areas filterAreas, JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_AREA IDAREA,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t2.ID_DIRECCION DIRECCIONESIDDIRECCION,t2.DESCRIPCION_CASTELLANO DIRECCIONESDESCCASTELLANO,t2.DESCRIPCION_EUSKERA DIRECCIONESDESCEUSKERA,t2.FECHA_REGISTRO DIRECCIONESFECHAREGISTRO,t2.USUARIO_REGISTRO DIRECCIONESUSUARIOREGISTRO,t2.ID_DEPARTAMENTO DIRECCIONESIDDEPARTAMENTO ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM AREAS t1 ,DIRECCIONES t2
		// ");
		List<String> from = new ArrayList<String>();
		from.add("AREAS");
		from.add("DIRECCIONES");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterAreas, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_AREA", "ID_PADRE", "NOMBRE",
				from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					AreasDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterAreas
	 *            Areas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(Areas filterAreas, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM AREAS t1 ,DIRECCIONES t2
		// ");
		List<String> from = new ArrayList<String>();
		from.add("AREAS");
		from.add("DIRECCIONES");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterAreas, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_AREA", "ID_PADRE",
				from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterAreas
	 *            Areas
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Areas>>
	 */
	@Override
	public List<TableRowDto<Areas>> findAllChild(Areas filterAreas, JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM AREAS t1 ,DIRECCIONES t2
		// ");
		List<String> from = new ArrayList<String>();
		from.add("AREAS");
		from.add("DIRECCIONES");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterAreas, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_AREA",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(), new RowNumResultSetExtractor<Areas>(this.rwMapPK, "ID_AREA"),
				params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Areas entity
	 *
	 * @param areas
	 *            Areas Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Areas areas) {

		StringBuilder where = new StringBuilder(AreasDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (areas != null && areas.getIdArea() != null) {
			where.append(" AND t1.ID_AREA = ?");
			params.add(areas.getIdArea());
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getIdDireccion() != null) {
			where.append(" AND t2.ID_DIRECCION = ?");
			params.add(areas.getDirecciones().getIdDireccion());
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getDepartamentos() != null
				&& areas.getDirecciones().getDepartamentos().getIdDepartamento() != null) {
			where.append(" AND t2.ID_DEPARTAMENTO = ?");
			params.add(areas.getDirecciones().getDepartamentos().getIdDepartamento());
		}
		if (areas != null && areas.getDirecciones() != null
				&& areas.getDirecciones().getDescripcionCastellano() != null) {
			where.append(" AND t2.DESCRIPCION_CASTELLANO = ?");
			params.add(areas.getDirecciones().getDescripcionCastellano());
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getDescripcionEuskera() != null) {
			where.append(" AND t2.DESCRIPCION_EUSKERA = ?");
			params.add(areas.getDirecciones().getDescripcionEuskera());
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getFechaRegistro() != null) {
			where.append(" AND t2.FECHA_REGISTRO = ?");
			params.add(areas.getDirecciones().getFechaRegistro());
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getUsuarioRegistro() != null) {
			where.append(" AND t2.USUARIO_REGISTRO = ?");
			params.add(areas.getDirecciones().getUsuarioRegistro());
		}
		if (areas != null && areas.getDescripcionCastellano() != null) {
			where.append(" AND t1.DESCRIPCION_CASTELLANO = ?");
			params.add(areas.getDescripcionCastellano());
		}
		if (areas != null && areas.getDescripcionEuskera() != null) {
			where.append(" AND t1.DESCRIPCION_EUSKERA = ?");
			params.add(areas.getDescripcionEuskera());
		}
		if (areas != null && areas.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(areas.getFechaRegistro());
		}
		if (areas != null && areas.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(areas.getUsuarioRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Areas entity
	 *
	 * @param areas
	 *            Areas Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Areas areas, Boolean startsWith) {

		StringBuilder where = new StringBuilder(AreasDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (areas != null && areas.getIdArea() != null) {
			where.append(" AND t1.ID_AREA = ?");
			params.add(areas.getIdArea());
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getIdDireccion() != null) {
			where.append(" AND t2.ID_DIRECCION = ?");
			params.add(areas.getDirecciones().getIdDireccion());
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getDepartamentos() != null
				&& areas.getDirecciones().getDepartamentos().getIdDepartamento() != null) {
			where.append(" AND t2.ID_DEPARTAMENTO = ?");
			params.add(areas.getDirecciones().getDepartamentos().getIdDepartamento());
		}
		if (areas != null && areas.getDirecciones() != null
				&& areas.getDirecciones().getDescripcionCastellano() != null) {
			where.append(" AND UPPER(t2.DESCRIPCION_CASTELLANO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(areas.getDirecciones().getDescripcionCastellano().toUpperCase() + "%");
			} else {
				params.add("%" + areas.getDirecciones().getDescripcionCastellano().toUpperCase() + "%");
			}
			where.append(" AND t2.DESCRIPCION_CASTELLANO IS NOT NULL");
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getDescripcionEuskera() != null) {
			where.append(" AND UPPER(t2.DESCRIPCION_EUSKERA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(areas.getDirecciones().getDescripcionEuskera().toUpperCase() + "%");
			} else {
				params.add("%" + areas.getDirecciones().getDescripcionEuskera().toUpperCase() + "%");
			}
			where.append(" AND t2.DESCRIPCION_EUSKERA IS NOT NULL");
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getFechaRegistro() != null) {
			where.append(" AND t2.FECHA_REGISTRO = ?");
			params.add(areas.getDirecciones().getFechaRegistro());
		}
		if (areas != null && areas.getDirecciones() != null && areas.getDirecciones().getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t2.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(areas.getDirecciones().getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + areas.getDirecciones().getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t2.USUARIO_REGISTRO IS NOT NULL");
		}
		if (areas != null && areas.getDescripcionCastellano() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION_CASTELLANO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(areas.getDescripcionCastellano().toUpperCase() + "%");
			} else {
				params.add("%" + areas.getDescripcionCastellano().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION_CASTELLANO IS NOT NULL");
		}
		if (areas != null && areas.getDescripcionEuskera() != null) {
			where.append(" AND UPPER(t1.DESCRIPCION_EUSKERA) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(areas.getDescripcionEuskera().toUpperCase() + "%");
			} else {
				params.add("%" + areas.getDescripcionEuskera().toUpperCase() + "%");
			}
			where.append(" AND t1.DESCRIPCION_EUSKERA IS NOT NULL");
		}
		if (areas != null && areas.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(areas.getFechaRegistro());
		}
		if (areas != null && areas.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(areas.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + areas.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
