package com.ejie.ab61.dao.procedimientos;

import com.ejie.ab61.model.procedimientos.Departamentos;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

/**
 * DepartamentosDao  
 *  
 */

public interface DepartamentosDao {
    
    /**
     * Inserts a single row in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return Departamentos
     */
    Departamentos add(Departamentos departamentos);

    /**
     * Updates a single row in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return Departamentos
     */
    Departamentos update(Departamentos departamentos);

    /**
     * Finds a single row in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return Departamentos
     */
    Departamentos find(Departamentos departamentos);

    /**
     * Removes a single row in the Departamentos table.
     *
     * @param departamentos Departamentos
     */
    void remove(Departamentos departamentos);

    /**
     * Finds a List of rows in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @return List
     */
    List<Departamentos> findAll(Departamentos departamentos, JQGridRequestDto jqGridRequestDto);

	/**
     * Finds rows in the Departamentos table using like.
     *
     * @param departamentos Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     * @return List
     */
	List<Departamentos> findAllLike(Departamentos departamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
	
	
	/*
	 * OPERACIONES RUP_TABLE
	 */
	 
	/**
	 * Filter in the Departamentos table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 */	
	 void removeMultiple(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
	
	
    /**
     * Counts rows in the Departamentos table using like.
     *
     * @param departamentos Departamentos
     * @param startsWith Boolean
     * @return Long
     */
    Long findAllLikeCount(Departamentos departamentos, Boolean startsWith);
    
    /**
     * Counts rows in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return Long
     */
    Long findAllCount(Departamentos departamentos);
    
    /**
	 * Reorder selection.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return List<TableRowDto<Departamentos>>
	 */	
    List<TableRowDto<Departamentos>> reorderSelection(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
    
     /**
	 * Searches in the Usuario table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param searchDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return List<TableRowDto<Departamentos>>	 
	 */	
    List<TableRowDto<Departamentos>> search(Departamentos filterDepartamentos, Departamentos searchDepartamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

    /*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */
    
    /**
	 * Filter in the hierarchical Departamentos table.
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return List<JerarquiaDto<Departamentos>>
	 */	
	List<JerarquiaDto<Departamentos>> findAllLikeJerarquia(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto);
	
	/**
	 * Count rows in hierarchy
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return Long
	 */	
	Long findAllLikeCountJerarquia(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto);
	
	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterDepartamentos Departamentos
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return List<TableRowDto<Departamentos>>
	 */	
	List<TableRowDto<Departamentos>> findAllChild(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto);
	
	
}

