package com.ejie.ab61.dao.procedimientos;

import com.ejie.ab61.model.procedimientos.Departamentos;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * DepartamentosDaoImpl  
 *  
 */

@Repository
@Transactional
public class DepartamentosDaoImpl implements DepartamentosDao {


    public static final String[] ORDER_BY_WHITE_LIST = new String[] {"IDDEPARTAMENTO", "DESCRIPCIONCASTELLANO", "DESCRIPCIONEUSKERA", "FECHAREGISTRO", "USUARIOREGISTRO"};

    private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Departamentos> rwMap = new RowMapper<Departamentos>() {
		public Departamentos mapRow(ResultSet resultSet, int rowNum) throws SQLException {
           return new Departamentos(
               resultSet.getInt("IDDEPARTAMENTO"), resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO")
           ); } } ;

	private RowMapper<Departamentos> rwMapPK = new RowMapper<Departamentos>() {
		public Departamentos mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Departamentos(
				resultSet.getInt("IDDEPARTAMENTO")
           ); } } ;

	/**
     * Rowmapper para Jerarquía
     *
     * @param dataSource DataSource
     * @return
     */
	private RowMapper<JerarquiaDto< Departamentos>> rwMapJerarquia = new RowMapper<JerarquiaDto<Departamentos>>() {
		public JerarquiaDto<Departamentos> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Departamentos departamentos = new Departamentos(resultSet.getInt("IDDEPARTAMENTO"), resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"));

			JerarquiaDto<Departamentos> jerarquia = new JerarquiaDto<Departamentos>();
			jerarquia.setModel(departamentos);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};


	/**
     * Method use to set the datasource.
     *
     * @param dataSource DataSource
     * @return
     */
    @Resource
    public void setDataSource(DataSource dataSource) {
    	this.jdbcTemplate = new JdbcTemplate(dataSource);
    }

	/*
	 * OPERACIONES CRUD
	 */

    /**
     * Inserts a single row in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return Departamentos
     */
	public Departamentos add(Departamentos departamentos) {
    	String query = "INSERT INTO DEPARTAMENTOS (ID_DEPARTAMENTO, DESCRIPCION_CASTELLANO, DESCRIPCION_EUSKERA, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?)";
		this.jdbcTemplate.update(query, departamentos.getIdDepartamento(), departamentos.getDescripcionCastellano(), departamentos.getDescripcionEuskera(), departamentos.getFechaRegistro(), departamentos.getUsuarioRegistro());
		return departamentos;
	}

    /**
     * Updates a single row in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return Departamentos
     */
    public Departamentos update(Departamentos departamentos) {
		String query = "UPDATE DEPARTAMENTOS SET DESCRIPCION_CASTELLANO=?, DESCRIPCION_EUSKERA=?, FECHA_REGISTRO=?, USUARIO_REGISTRO=? WHERE ID_DEPARTAMENTO=?";
		this.jdbcTemplate.update(query, departamentos.getDescripcionCastellano(), departamentos.getDescripcionEuskera(), departamentos.getFechaRegistro(), departamentos.getUsuarioRegistro(), departamentos.getIdDepartamento());
		return departamentos;
	}

    /**
     * Finds a single row in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return Departamentos
     */
    @Transactional (readOnly = true)
    public Departamentos find(Departamentos departamentos) {
		String query = "SELECT t1.ID_DEPARTAMENTO IDDEPARTAMENTO, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO FROM DEPARTAMENTOS t1  WHERE t1.ID_DEPARTAMENTO = ?  ";

		List<Departamentos> departamentosList = this.jdbcTemplate.query(query, this.rwMap, departamentos.getIdDepartamento());
		return (Departamentos) DataAccessUtils.uniqueResult(departamentosList);
    }

    /**
     * Removes a single row in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return
     */
    public void remove(Departamentos departamentos) {
		String query = "DELETE FROM DEPARTAMENTOS WHERE ID_DEPARTAMENTO=?";
		this.jdbcTemplate.update(query, departamentos.getIdDepartamento());
    }

   /**
    * Finds a list of rows in the Departamentos table.
    *
    * @param departamentos Departamentos
    * @param jqGridRequestDto JQGridRequestDto
    * @return List<Departamentos>
    */
	@Transactional (readOnly = true)
    public List<Departamentos> findAll(Departamentos departamentos, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder("SELECT  t1.ID_DEPARTAMENTO IDDEPARTAMENTO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		query.append("FROM DEPARTAMENTOS t1 ");

		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(departamentos); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DepartamentosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Departamentos>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Departamentos table using like.
     *
     * @param departamentos Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     * @return List<Departamentos>
     */
	@Transactional (readOnly = true)
    public List<Departamentos> findAllLike(Departamentos departamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT  t1.ID_DEPARTAMENTO IDDEPARTAMENTO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
        query.append("FROM DEPARTAMENTOS t1 ");

		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(departamentos,startsWith); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DepartamentosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Departamentos>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

    /**
     * Counts rows in the Departamentos table.
     *
     * @param departamentos Departamentos
     * @return Long
     */
    @Transactional (readOnly = true)
    public Long findAllCount(Departamentos departamentos) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DEPARTAMENTOS t1 ");

		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(departamentos); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Departamentos table using like.
     *
     * @param departamentos Departamentos
     * @param startsWith Boolean
     * @return Long
     */
	@Transactional (readOnly = true)
    public Long findAllLikeCount(Departamentos departamentos, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DEPARTAMENTOS t1 ");

		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(departamentos,startsWith); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Departamentos selected for rup_table
     *
     * @param departamentos Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     * @return List<TableRowDto<Departamentos>>
     */
	@Override
	public List<TableRowDto<Departamentos>> reorderSelection(Departamentos departamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder("SELECT t1.ID_DEPARTAMENTO IDDEPARTAMENTO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
        query.append("FROM DEPARTAMENTOS t1 ");
		// FILTRADO
		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(departamentos,startsWith); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL =  JQGridManager.getReorderQuery(query, jqGridRequestDto, Departamentos.class, filterParamList, "ID_DEPARTAMENTO" );

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(), new RowNumResultSetExtractor<Departamentos>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
     *
     * @param filterParams Departamentos
     * @param searchParams Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     * @return List<TableRowDto<Departamentos>>
     */
	@Override
	public List<TableRowDto<Departamentos>> search(Departamentos filterParams, Departamentos searchParams, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder("SELECT t1.ID_DEPARTAMENTO IDDEPARTAMENTO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
        query.append("FROM DEPARTAMENTOS t1 ");
        //TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, Departamentos.class, filterParamList, searchSQL, searchParamList, from_alias, "ID_DEPARTAMENTO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(), new RowNumResultSetExtractor<Departamentos>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
     *
     * @param filterdepartamentos Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     */
	@Override
	public void removeMultiple(Departamentos filterdepartamentos, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder("SELECT t1.ID_DEPARTAMENTO IDDEPARTAMENTO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
        query.append("FROM DEPARTAMENTOS t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterdepartamentos, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, Departamentos.class, query, params, "ID_DEPARTAMENTO");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical Departamentos table.
     *
     * @param filterDepartamentos Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @return List<JerarquiaDto<Departamentos>>
     */
	@Override
	public List<JerarquiaDto<Departamentos>> findAllLikeJerarquia(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder("SELECT t1.ID_DEPARTAMENTO IDDEPARTAMENTO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
        //StringBuilder from = new StringBuilder("FROM DEPARTAMENTOS t1 ");
        List<String> from = new ArrayList<String>();
        from.add("DEPARTAMENTOS");
        //TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		//FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDepartamentos, false);

		//JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_DEPARTAMENTO", "ID_PADRE", "NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y condiciones de negocio
		 */
//		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins, businessFilters, businessParams);

		//PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query, DepartamentosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
     *
     * @param filterDepartamentos Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @return Long
     */
	@Override
	public Long findAllLikeCountJerarquia(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto) {
		// FROM
        //StringBuilder from = new StringBuilder("FROM DEPARTAMENTOS t1 ");
        List<String> from = new ArrayList<String>();
        from.add("DEPARTAMENTOS");

        //TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		//FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDepartamentos, false);

		//JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_DEPARTAMENTO", "ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y condiciones de negocio
		 */
//		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID", "ID_PADRE", from, from_alias, joins, businessFilters, businessParams);


		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
     *
     * @param filterDepartamentos Departamentos
     * @param jqGridRequestDto JQGridRequestDto
     * @return List<TableRowDto<Departamentos>>
     */
	@Override
	public List<TableRowDto<Departamentos>> findAllChild(Departamentos filterDepartamentos, JQGridRequestDto jqGridRequestDto) {

		// FROM
        //StringBuilder from = new StringBuilder("FROM DEPARTAMENTOS t1 ");
        List<String> from = new ArrayList<String>();
        from.add("DEPARTAMENTOS");

        //TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		//FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDepartamentos, false);

		//MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_DEPARTAMENTO", "ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y condiciones de negocio
		 */
//		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID", "ID_PADRE", from, from_alias, joins, businessFilters, businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(), new RowNumResultSetExtractor<Departamentos>(this.rwMapPK, "ID_DEPARTAMENTO"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Departamentos entity
	 *
	 * @param departamentos Departamentos
	 *            Bean with the criteria values to filter by.
	 * @return Map created with two keys
	 *         key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap (Departamentos departamentos){

		StringBuilder where = new StringBuilder(DepartamentosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (departamentos  != null  && departamentos.getIdDepartamento() != null ) {
			where.append(" AND t1.ID_DEPARTAMENTO = ?");
			params.add(departamentos.getIdDepartamento());
		}
		if (departamentos  != null  && departamentos.getDescripcionCastellano() != null ) {
			where.append(" AND t1.DESCRIPCION_CASTELLANO = ?");
			params.add(departamentos.getDescripcionCastellano());
		}
		if (departamentos  != null  && departamentos.getDescripcionEuskera() != null ) {
			where.append(" AND t1.DESCRIPCION_EUSKERA = ?");
			params.add(departamentos.getDescripcionEuskera());
		}
		if (departamentos  != null  && departamentos.getFechaRegistro() != null ) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(departamentos.getFechaRegistro());
		}
		if (departamentos  != null  && departamentos.getUsuarioRegistro() != null ) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(departamentos.getUsuarioRegistro());
		}

		Map<String,Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Departamentos entity
	 *
	 * @param departamentos Departamentos
	 *            Bean with the criteria values to filter by.
     * @param startsWith Boolean
	 * @return Map created with two keys
	 *         key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap (Departamentos departamentos, Boolean startsWith){

		StringBuilder where = new StringBuilder(DepartamentosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (departamentos  != null  && departamentos.getIdDepartamento() != null ) {
			where.append(" AND t1.ID_DEPARTAMENTO = ?");
			params.add(departamentos.getIdDepartamento());
	     }
		if (departamentos  != null  && departamentos.getDescripcionCastellano() != null ) {
			where.append(" AND UPPER(t1.DESCRIPCION_CASTELLANO) like ? ESCAPE  '\\'");
			if (startsWith){
				params.add(departamentos.getDescripcionCastellano().toUpperCase() +"%");
			}else{
				params.add("%"+departamentos.getDescripcionCastellano().toUpperCase() +"%");
			}
			where.append(" AND t1.DESCRIPCION_CASTELLANO IS NOT NULL");
	     }
		if (departamentos  != null  && departamentos.getDescripcionEuskera() != null ) {
			where.append(" AND UPPER(t1.DESCRIPCION_EUSKERA) like ? ESCAPE  '\\'");
			if (startsWith){
				params.add(departamentos.getDescripcionEuskera().toUpperCase() +"%");
			}else{
				params.add("%"+departamentos.getDescripcionEuskera().toUpperCase() +"%");
			}
			where.append(" AND t1.DESCRIPCION_EUSKERA IS NOT NULL");
	     }
		if (departamentos  != null  && departamentos.getFechaRegistro() != null ) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(departamentos.getFechaRegistro());
	     }
		if (departamentos  != null  && departamentos.getUsuarioRegistro() != null ) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith){
				params.add(departamentos.getUsuarioRegistro().toUpperCase() +"%");
			}else{
				params.add("%"+departamentos.getUsuarioRegistro().toUpperCase() +"%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
	     }

		Map<String,Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	 public static final int STRING_BUILDER_INIT = 4096;}

