package com.ejie.ab61.dao.solicitudes;

import com.ejie.ab61.comun.utilidades.ClobUtils;
import com.ejie.ab61.comun.utilidades.SecuenciasSQL;
import com.ejie.ab61.model.solicitudes.MensajesSolicitud;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * MensajesSolicitudDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class MensajesSolicitudDaoImpl implements MensajesSolicitudDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDMENSAJE", "TIPOMENSAJE", "MENSAJE",
			"FECHAREGISTRO" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<MensajesSolicitud> rwMap = new RowMapper<MensajesSolicitud>() {
		public MensajesSolicitud mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new MensajesSolicitud(resultSet.getInt("IDMENSAJE"), resultSet.getString("TIPOMENSAJE"),
					resultSet.getClob("MENSAJE"), resultSet.getDate("FECHAREGISTRO"));
		}
	};

	private RowMapper<MensajesSolicitud> rwMapPK = new RowMapper<MensajesSolicitud>() {
		public MensajesSolicitud mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new MensajesSolicitud(resultSet.getInt("IDMENSAJE"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<MensajesSolicitud>> rwMapJerarquia = new RowMapper<JerarquiaDto<MensajesSolicitud>>() {
		public JerarquiaDto<MensajesSolicitud> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			MensajesSolicitud mensajessolicitud = new MensajesSolicitud(resultSet.getInt("IDMENSAJE"),
					resultSet.getString("TIPOMENSAJE"), resultSet.getClob("MENSAJE"),
					resultSet.getDate("FECHAREGISTRO"));

			JerarquiaDto<MensajesSolicitud> jerarquia = new JerarquiaDto<MensajesSolicitud>();
			jerarquia.setModel(mensajessolicitud);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the MensajesSolicitud table.
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @return MensajesSolicitud
	 */
	public MensajesSolicitud add(MensajesSolicitud mensajessolicitud) {
		int idMensaje = this.jdbcTemplate.queryForInt(SecuenciasSQL.getSelectNext("AB61.S_AB6117T00"));
		String query = "INSERT INTO MENSAJES_SOLICITUD (ID_MENSAJE, TIPOMENSAJE, MENSAJE, FECHA_REGISTRO, COD_PROCEDURE_ID, PLATEA_TYPE, PLATEA_SUBTYPE) VALUES (?,?,?,?,?,?,?)";
		this.jdbcTemplate.update(query, idMensaje, mensajessolicitud.getTipomensaje(),
				ClobUtils.clobToString(mensajessolicitud.getMensaje()), mensajessolicitud.getFechaRegistro(),
				mensajessolicitud.getCodProcedureId(), mensajessolicitud.getPlateaType(),
				mensajessolicitud.getPlateaSubtype());
		mensajessolicitud.setIdMensaje(idMensaje);
		return mensajessolicitud;
	}

	/**
	 * Updates a single row in the MensajesSolicitud table.
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @return MensajesSolicitud
	 */
	public MensajesSolicitud update(MensajesSolicitud mensajessolicitud) {
		String query = "UPDATE MENSAJES_SOLICITUD SET TIPOMENSAJE=?, MENSAJE=?, FECHA_REGISTRO=? WHERE ID_MENSAJE=?";
		this.jdbcTemplate.update(query, mensajessolicitud.getTipomensaje(), mensajessolicitud.getMensaje(),
				mensajessolicitud.getFechaRegistro(), mensajessolicitud.getIdMensaje());
		return mensajessolicitud;
	}

	/**
	 * Finds a single row in the MensajesSolicitud table.
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @return MensajesSolicitud
	 */
	@Transactional(readOnly = true)
	public MensajesSolicitud find(MensajesSolicitud mensajessolicitud) {
		String query = "SELECT t1.ID_MENSAJE IDMENSAJE, t1.TIPOMENSAJE TIPOMENSAJE, t1.MENSAJE MENSAJE, t1.FECHA_REGISTRO FECHAREGISTRO FROM MENSAJES_SOLICITUD t1  WHERE t1.ID_MENSAJE = ?  ";

		List<MensajesSolicitud> mensajessolicitudList = this.jdbcTemplate.query(query, this.rwMap,
				mensajessolicitud.getIdMensaje());
		return (MensajesSolicitud) DataAccessUtils.uniqueResult(mensajessolicitudList);
	}

	/**
	 * Removes a single row in the MensajesSolicitud table.
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @return
	 */
	public void remove(MensajesSolicitud mensajessolicitud) {
		String query = "DELETE FROM MENSAJES_SOLICITUD WHERE ID_MENSAJE=?";
		this.jdbcTemplate.update(query, mensajessolicitud.getIdMensaje());
	}

	/**
	 * Finds a list of rows in the MensajesSolicitud table.
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<MensajesSolicitud>
	 */
	@Transactional(readOnly = true)
	public List<MensajesSolicitud> findAll(MensajesSolicitud mensajessolicitud, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_MENSAJE IDMENSAJE,t1.TIPOMENSAJE TIPOMENSAJE,t1.MENSAJE MENSAJE,t1.FECHA_REGISTRO FECHAREGISTRO ");
		query.append("FROM MENSAJES_SOLICITUD t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(mensajessolicitud);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query,
					MensajesSolicitudDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<MensajesSolicitud>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the MensajesSolicitud table using like.
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<MensajesSolicitud>
	 */
	@Transactional(readOnly = true)
	public List<MensajesSolicitud> findAllLike(MensajesSolicitud mensajessolicitud, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_MENSAJE IDMENSAJE,t1.TIPOMENSAJE TIPOMENSAJE,t1.MENSAJE MENSAJE,t1.FECHA_REGISTRO FECHAREGISTRO ");
		query.append("FROM MENSAJES_SOLICITUD t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(mensajessolicitud, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query,
					MensajesSolicitudDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<MensajesSolicitud>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the MensajesSolicitud table.
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(MensajesSolicitud mensajessolicitud) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM MENSAJES_SOLICITUD t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(mensajessolicitud);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the MensajesSolicitud table using like.
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(MensajesSolicitud mensajessolicitud, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM MENSAJES_SOLICITUD t1 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(mensajessolicitud, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of MensajesSolicitud selected for rup_table
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<MensajesSolicitud>>
	 */
	@Override
	public List<TableRowDto<MensajesSolicitud>> reorderSelection(MensajesSolicitud mensajessolicitud,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_MENSAJE IDMENSAJE,t1.TIPOMENSAJE TIPOMENSAJE,t1.MENSAJE MENSAJE,t1.FECHA_REGISTRO FECHAREGISTRO ");
		// FROM
		query.append("FROM MENSAJES_SOLICITUD t1 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(mensajessolicitud, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto,
				MensajesSolicitud.class, filterParamList, "ID_MENSAJE");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<MensajesSolicitud>(this.rwMapPK, jqGridRequestDto),
				filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            MensajesSolicitud
	 * @param searchParams
	 *            MensajesSolicitud
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<MensajesSolicitud>>
	 */
	@Override
	public List<TableRowDto<MensajesSolicitud>> search(MensajesSolicitud filterParams, MensajesSolicitud searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_MENSAJE IDMENSAJE,t1.TIPOMENSAJE TIPOMENSAJE,t1.MENSAJE MENSAJE,t1.FECHA_REGISTRO FECHAREGISTRO ");
		// FROM
		query.append("FROM MENSAJES_SOLICITUD t1 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto,
				MensajesSolicitud.class, filterParamList, searchSQL, searchParamList, from_alias, "ID_MENSAJE");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<MensajesSolicitud>(this.rwMapPK, jqGridRequestDto),
				filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filtermensajessolicitud
	 *            MensajesSolicitud
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(MensajesSolicitud filtermensajessolicitud, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_MENSAJE IDMENSAJE,t1.TIPOMENSAJE TIPOMENSAJE,t1.MENSAJE MENSAJE,t1.FECHA_REGISTRO FECHAREGISTRO ");
		// FROM
		query.append("FROM MENSAJES_SOLICITUD t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filtermensajessolicitud, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto,
				MensajesSolicitud.class, query, params, "ID_MENSAJE");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical MensajesSolicitud table.
	 *
	 * @param filterMensajesSolicitud
	 *            MensajesSolicitud
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<MensajesSolicitud>>
	 */
	@Override
	public List<JerarquiaDto<MensajesSolicitud>> findAllLikeJerarquia(MensajesSolicitud filterMensajesSolicitud,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_MENSAJE IDMENSAJE,t1.TIPOMENSAJE TIPOMENSAJE,t1.MENSAJE MENSAJE,t1.FECHA_REGISTRO FECHAREGISTRO ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM MENSAJES_SOLICITUD t1
		// ");
		List<String> from = new ArrayList<String>();
		from.add("MENSAJES_SOLICITUD");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterMensajesSolicitud, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_MENSAJE", "ID_PADRE", "NOMBRE",
				from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					MensajesSolicitudDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterMensajesSolicitud
	 *            MensajesSolicitud
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(MensajesSolicitud filterMensajesSolicitud,
			JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM MENSAJES_SOLICITUD t1
		// ");
		List<String> from = new ArrayList<String>();
		from.add("MENSAJES_SOLICITUD");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterMensajesSolicitud, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_MENSAJE",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterMensajesSolicitud
	 *            MensajesSolicitud
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<MensajesSolicitud>>
	 */
	@Override
	public List<TableRowDto<MensajesSolicitud>> findAllChild(MensajesSolicitud filterMensajesSolicitud,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM MENSAJES_SOLICITUD t1
		// ");
		List<String> from = new ArrayList<String>();
		from.add("MENSAJES_SOLICITUD");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterMensajesSolicitud, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_MENSAJE",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<MensajesSolicitud>(this.rwMapPK, "ID_MENSAJE"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the MensajesSolicitud entity
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(MensajesSolicitud mensajessolicitud) {

		StringBuilder where = new StringBuilder(MensajesSolicitudDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (mensajessolicitud != null && mensajessolicitud.getIdMensaje() != null) {
			where.append(" AND t1.ID_MENSAJE = ?");
			params.add(mensajessolicitud.getIdMensaje());
		}
		if (mensajessolicitud != null && mensajessolicitud.getTipomensaje() != null) {
			where.append(" AND t1.TIPOMENSAJE = ?");
			params.add(mensajessolicitud.getTipomensaje());
		}
		if (mensajessolicitud != null && mensajessolicitud.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(mensajessolicitud.getFechaRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the MensajesSolicitud entity
	 *
	 * @param mensajessolicitud
	 *            MensajesSolicitud Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(MensajesSolicitud mensajessolicitud, Boolean startsWith) {

		StringBuilder where = new StringBuilder(MensajesSolicitudDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (mensajessolicitud != null && mensajessolicitud.getIdMensaje() != null) {
			where.append(" AND t1.ID_MENSAJE = ?");
			params.add(mensajessolicitud.getIdMensaje());
		}

		if (mensajessolicitud != null && mensajessolicitud.getTipomensaje() != null) {
			where.append(" AND UPPER(t1.TIPOMENSAJE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(mensajessolicitud.getTipomensaje().toUpperCase() + "%");
			} else {
				params.add("%" + mensajessolicitud.getTipomensaje().toUpperCase() + "%");
			}
			where.append(" AND t1.TIPOMENSAJE IS NOT NULL");
		}
		if (mensajessolicitud != null && mensajessolicitud.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(mensajessolicitud.getFechaRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
