package com.ejie.ab61.dao.solicitudes;

import com.ejie.ab61.model.solicitudes.Solicitudes;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

/**
 * SolicitudesDao  
 * 
 *  
 */

public interface SolicitudesDao {

	/**
	 * Inserts a single row in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return Solicitudes
	 */
	Solicitudes add(Solicitudes solicitudes);

	/**
	 * Updates a single row in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return Solicitudes
	 */
	Solicitudes update(Solicitudes solicitudes);

	/**
	 * Finds a single row in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return Solicitudes
	 */
	Solicitudes find(Solicitudes solicitudes);

	/**
	 * Removes a single row in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 */
	void remove(Solicitudes solicitudes);

	/**
	 * Finds a List of rows in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List
	 */
	List<Solicitudes> findAll(Solicitudes solicitudes, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Solicitudes table using like.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List
	 */
	List<Solicitudes> findAllLike(Solicitudes solicitudes, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Filter in the Solicitudes table.
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Counts rows in the Solicitudes table using like.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	Long findAllLikeCount(Solicitudes solicitudes, Boolean startsWith);

	/**
	 * Counts rows in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return Long
	 */
	Long findAllCount(Solicitudes solicitudes);

	/**
	 * Reorder selection.
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Solicitudes>>
	 */
	List<TableRowDto<Solicitudes>> reorderSelection(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches in the Usuario table.
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param searchSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Solicitudes>>
	 */
	List<TableRowDto<Solicitudes>> search(Solicitudes filterSolicitudes, Solicitudes searchSolicitudes,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Filter in the hierarchical Solicitudes table.
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Solicitudes>>
	 */
	List<JerarquiaDto<Solicitudes>> findAllLikeJerarquia(Solicitudes filterSolicitudes,
			JQGridRequestDto jqGridRequestDto);

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	Long findAllLikeCountJerarquia(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto);

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Solicitudes>>
	 */
	List<TableRowDto<Solicitudes>> findAllChild(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto);

	Solicitudes findSolicitud(Integer idMensaje);

}
