package com.ejie.ab61.dao.solicitudes;

import com.ejie.ab61.comun.utilidades.ClobUtils;
import com.ejie.ab61.comun.utilidades.SecuenciasSQL;
import com.ejie.ab61.model.solicitudes.MensajesSolicitud;
import com.ejie.ab61.model.solicitudes.Solicitudes;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * SolicitudesDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class SolicitudesDaoImpl implements SolicitudesDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDSOLICITUD", "SOLICITUD", "FECHAREGISTRO",
			"MENSAJESSOLICITUDIDMENSAJE", "MENSAJESSOLICITUDTIPOMENSAJE", "MENSAJESSOLICITUDMENSAJE",
			"MENSAJESSOLICITUDFECHAREGISTRO" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<Solicitudes> rwMap = new RowMapper<Solicitudes>() {
		public Solicitudes mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Solicitudes(resultSet.getInt("IDSOLICITUD"), resultSet.getClob("SOLICITUD"),
					resultSet.getDate("FECHAREGISTRO"),
					new MensajesSolicitud(resultSet.getInt("MensajesSolicitudIDMENSAJE"),
							resultSet.getString("MensajesSolicitudTIPOMENSAJE"),
							resultSet.getClob("MensajesSolicitudMENSAJE"),
							resultSet.getDate("MensajesSolicitudFECHAREGISTRO")));
		}
	};

	private RowMapper<Solicitudes> rwMapFind = new RowMapper<Solicitudes>() {
		public Solicitudes mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Solicitudes solicitud = new Solicitudes();
			solicitud.setIdSolicitud(resultSet.getInt("IDSOLICITUD"));
			solicitud.setSolicitud(resultSet.getClob("SOLICITUD"));
			solicitud.setFechaRegistro(resultSet.getDate("FECHAREGISTRO"));

			return solicitud;
		}
	};

	private RowMapper<Solicitudes> rwMapPK = new RowMapper<Solicitudes>() {
		public Solicitudes mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new Solicitudes(resultSet.getInt("IDSOLICITUD"));
		}
	};

	/**
	 * Rowmapper para Jerarquía
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<Solicitudes>> rwMapJerarquia = new RowMapper<JerarquiaDto<Solicitudes>>() {
		public JerarquiaDto<Solicitudes> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			Solicitudes solicitudes = new Solicitudes(resultSet.getInt("IDSOLICITUD"), resultSet.getClob("SOLICITUD"),
					resultSet.getDate("FECHAREGISTRO"),
					new MensajesSolicitud(resultSet.getInt("MensajesSolicitudIDMENSAJE"),
							resultSet.getString("MensajesSolicitudTIPOMENSAJE"),
							resultSet.getClob("MensajesSolicitudMENSAJE"),
							resultSet.getDate("MensajesSolicitudFECHAREGISTRO")));

			JerarquiaDto<Solicitudes> jerarquia = new JerarquiaDto<Solicitudes>();
			jerarquia.setModel(solicitudes);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return Solicitudes
	 */
	public Solicitudes add(Solicitudes solicitudes) {

		int idSolicitud = this.jdbcTemplate.queryForInt(SecuenciasSQL.getSelectNext("AB61.S_AB6118T00"));
		String query = "INSERT INTO SOLICITUDES (ID_SOLICITUD, ID_MENSAJE, SOLICITUD, FECHA_REGISTRO, FOLDER_ID, FECHA_ENTRADA, COD_PROCEDURE_ID) VALUES (?,?,?,?,?, TO_DATE(?,'DD/MM/YYYY hh24:mi:ss'),?)";
		Object getMensajesSolicitudIdMensajeAux = null;
		if (solicitudes.getMensajesSolicitud() != null && solicitudes.getMensajesSolicitud().getIdMensaje() != null) {
			getMensajesSolicitudIdMensajeAux = solicitudes.getMensajesSolicitud().getIdMensaje();
		}
		this.jdbcTemplate.update(query, idSolicitud, getMensajesSolicitudIdMensajeAux,
				ClobUtils.clobToString(solicitudes.getSolicitud()), solicitudes.getFechaRegistro(),
				solicitudes.getFolderId(), solicitudes.getFechaEntradaStr(), solicitudes.getCodProcedureId());
		solicitudes.setIdSolicitud(idSolicitud);
		return solicitudes;
	}

	/**
	 * Updates a single row in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return Solicitudes
	 */
	public Solicitudes update(Solicitudes solicitudes) {
		String query = "UPDATE SOLICITUDES SET ID_MENSAJE=?, SOLICITUD=?, FECHA_REGISTRO=? WHERE ID_SOLICITUD=?";
		Object getMensajesSolicitudIdMensajeAux = null;
		if (solicitudes.getMensajesSolicitud() != null && solicitudes.getMensajesSolicitud().getIdMensaje() != null) {
			getMensajesSolicitudIdMensajeAux = solicitudes.getMensajesSolicitud().getIdMensaje();
		}
		this.jdbcTemplate.update(query, getMensajesSolicitudIdMensajeAux, solicitudes.getSolicitud(),
				solicitudes.getFechaRegistro(), solicitudes.getIdSolicitud());
		return solicitudes;
	}

	/**
	 * Finds a single row in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return Solicitudes
	 */
	@Transactional(readOnly = true)
	public Solicitudes find(Solicitudes solicitudes) {
		String query = "SELECT t1.ID_SOLICITUD IDSOLICITUD, t1.SOLICITUD SOLICITUD, t1.FECHA_REGISTRO FECHAREGISTRO, t2.ID_MENSAJE MENSAJESSOLICITUDIDMENSAJE, t2.TIPOMENSAJE MENSAJESSOLICITUDTIPOMENSAJE, t2.MENSAJE MENSAJESSOLICITUDMENSAJE, t2.FECHA_REGISTRO MENSAJESSOLICITUDFECHAREGISTRO FROM SOLICITUDES t1 , MENSAJES_SOLICITUD t2  WHERE t1.ID_SOLICITUD = ?   AND t1.ID_MENSAJE= t2.ID_MENSAJE(+)";

		List<Solicitudes> solicitudesList = this.jdbcTemplate.query(query, this.rwMap, solicitudes.getIdSolicitud());
		return (Solicitudes) DataAccessUtils.uniqueResult(solicitudesList);
	}

	/**
	 * Removes a single row in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return
	 */
	public void remove(Solicitudes solicitudes) {
		String query = "DELETE FROM SOLICITUDES WHERE ID_SOLICITUD=?";
		this.jdbcTemplate.update(query, solicitudes.getIdSolicitud());
	}

	/**
	 * Finds a list of rows in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Solicitudes>
	 */
	@Transactional(readOnly = true)
	public List<Solicitudes> findAll(Solicitudes solicitudes, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_SOLICITUD IDSOLICITUD,t1.SOLICITUD SOLICITUD,t1.FECHA_REGISTRO FECHAREGISTRO,t2.ID_MENSAJE MENSAJESSOLICITUDIDMENSAJE,t2.TIPOMENSAJE MENSAJESSOLICITUDTIPOMENSAJE,t2.MENSAJE MENSAJESSOLICITUDMENSAJE,t2.FECHA_REGISTRO MENSAJESSOLICITUDFECHAREGISTRO ");
		query.append("FROM SOLICITUDES t1 ,MENSAJES_SOLICITUD t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(solicitudes);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_MENSAJE= t2.ID_MENSAJE(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, SolicitudesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Solicitudes>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the Solicitudes table using like.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Solicitudes>
	 */
	@Transactional(readOnly = true)
	public List<Solicitudes> findAllLike(Solicitudes solicitudes, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_SOLICITUD IDSOLICITUD,t1.SOLICITUD SOLICITUD,t1.FECHA_REGISTRO FECHAREGISTRO,t2.ID_MENSAJE MENSAJESSOLICITUDIDMENSAJE,t2.TIPOMENSAJE MENSAJESSOLICITUDTIPOMENSAJE,t2.MENSAJE MENSAJESSOLICITUDMENSAJE,t2.FECHA_REGISTRO MENSAJESSOLICITUDFECHAREGISTRO ");
		query.append("FROM SOLICITUDES t1 ,MENSAJES_SOLICITUD t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(solicitudes, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_MENSAJE= t2.ID_MENSAJE(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, SolicitudesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<Solicitudes>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the Solicitudes table.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(Solicitudes solicitudes) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM SOLICITUDES t1 , MENSAJES_SOLICITUD t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(solicitudes);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_MENSAJE= t2.ID_MENSAJE(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the Solicitudes table using like.
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(Solicitudes solicitudes, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM SOLICITUDES t1 ,MENSAJES_SOLICITUD t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(solicitudes, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_MENSAJE= t2.ID_MENSAJE(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of Solicitudes selected for rup_table
	 *
	 * @param solicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Solicitudes>>
	 */
	@Override
	public List<TableRowDto<Solicitudes>> reorderSelection(Solicitudes solicitudes, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_SOLICITUD IDSOLICITUD,t1.SOLICITUD SOLICITUD,t1.FECHA_REGISTRO FECHAREGISTRO,t2.ID_MENSAJE MENSAJESSOLICITUDIDMENSAJE,t2.TIPOMENSAJE MENSAJESSOLICITUDTIPOMENSAJE,t2.MENSAJE MENSAJESSOLICITUDMENSAJE,t2.FECHA_REGISTRO MENSAJESSOLICITUDFECHAREGISTRO ");
		// FROM
		query.append("FROM SOLICITUDES t1 ,MENSAJES_SOLICITUD t2 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(solicitudes, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_MENSAJE= t2.ID_MENSAJE(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto, Solicitudes.class,
				filterParamList, "ID_SOLICITUD");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Solicitudes>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            Solicitudes
	 * @param searchParams
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Solicitudes>>
	 */
	@Override
	public List<TableRowDto<Solicitudes>> search(Solicitudes filterParams, Solicitudes searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_SOLICITUD IDSOLICITUD,t1.SOLICITUD SOLICITUD,t1.FECHA_REGISTRO FECHAREGISTRO,t2.ID_MENSAJE MENSAJESSOLICITUDIDMENSAJE,t2.TIPOMENSAJE MENSAJESSOLICITUDTIPOMENSAJE,t2.MENSAJE MENSAJESSOLICITUDMENSAJE,t2.FECHA_REGISTRO MENSAJESSOLICITUDFECHAREGISTRO ");
		// FROM
		query.append("FROM SOLICITUDES t1 ,MENSAJES_SOLICITUD t2 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_MENSAJE= t2.ID_MENSAJE(+) ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, Solicitudes.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_SOLICITUD");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<Solicitudes>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filtersolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(Solicitudes filtersolicitudes, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_SOLICITUD IDSOLICITUD,t1.SOLICITUD SOLICITUD,t1.FECHA_REGISTRO FECHAREGISTRO,t2.ID_MENSAJE MENSAJESSOLICITUDIDMENSAJE,t2.TIPOMENSAJE MENSAJESSOLICITUDTIPOMENSAJE,t2.MENSAJE MENSAJESSOLICITUDMENSAJE,t2.FECHA_REGISTRO MENSAJESSOLICITUDFECHAREGISTRO ");
		// FROM
		query.append("FROM SOLICITUDES t1 ,MENSAJES_SOLICITUD t2 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filtersolicitudes, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, Solicitudes.class,
				query, params, "ID_SOLICITUD");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical Solicitudes table.
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<Solicitudes>>
	 */

	@Override
	public Solicitudes findSolicitud(Integer idMensaje) {
		String query = "SELECT t1.ID_SOLICITUD IDSOLICITUD, t1.SOLICITUD SOLICITUD, t1.FECHA_REGISTRO FECHAREGISTRO FROM SOLICITUDES t1 WHERE t1.ID_MENSAJE = ?";

		List<Solicitudes> solicitudesList = this.jdbcTemplate.query(query, this.rwMapFind, idMensaje);
		return (Solicitudes) DataAccessUtils.uniqueResult(solicitudesList);
	}

	@Override
	public List<JerarquiaDto<Solicitudes>> findAllLikeJerarquia(Solicitudes filterSolicitudes,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_SOLICITUD IDSOLICITUD,t1.SOLICITUD SOLICITUD,t1.FECHA_REGISTRO FECHAREGISTRO,t2.ID_MENSAJE MENSAJESSOLICITUDIDMENSAJE,t2.TIPOMENSAJE MENSAJESSOLICITUDTIPOMENSAJE,t2.MENSAJE MENSAJESSOLICITUDMENSAJE,t2.FECHA_REGISTRO MENSAJESSOLICITUDFECHAREGISTRO ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM SOLICITUDES t1
		// ,MENSAJES_SOLICITUD t2 ");
		List<String> from = new ArrayList<String>();
		from.add("SOLICITUDES");
		from.add("MENSAJES_SOLICITUD");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterSolicitudes, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_SOLICITUD", "ID_PADRE",
				"NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					SolicitudesDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM SOLICITUDES t1
		// ,MENSAJES_SOLICITUD t2 ");
		List<String> from = new ArrayList<String>();
		from.add("SOLICITUDES");
		from.add("MENSAJES_SOLICITUD");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterSolicitudes, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_SOLICITUD",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterSolicitudes
	 *            Solicitudes
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<Solicitudes>>
	 */
	@Override
	public List<TableRowDto<Solicitudes>> findAllChild(Solicitudes filterSolicitudes,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM SOLICITUDES t1
		// ,MENSAJES_SOLICITUD t2 ");
		List<String> from = new ArrayList<String>();
		from.add("SOLICITUDES");
		from.add("MENSAJES_SOLICITUD");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterSolicitudes, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_SOLICITUD",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<Solicitudes>(this.rwMapPK, "ID_SOLICITUD"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Solicitudes entity
	 *
	 * @param solicitudes
	 *            Solicitudes Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap(Solicitudes solicitudes) {

		StringBuilder where = new StringBuilder(SolicitudesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (solicitudes != null && solicitudes.getIdSolicitud() != null) {
			where.append(" AND t1.ID_SOLICITUD = ?");
			params.add(solicitudes.getIdSolicitud());
		}
		if (solicitudes != null && solicitudes.getMensajesSolicitud() != null
				&& solicitudes.getMensajesSolicitud().getIdMensaje() != null) {
			where.append(" AND t2.ID_MENSAJE = ?");
			params.add(solicitudes.getMensajesSolicitud().getIdMensaje());
		}
		if (solicitudes != null && solicitudes.getMensajesSolicitud() != null
				&& solicitudes.getMensajesSolicitud().getTipomensaje() != null) {
			where.append(" AND t2.TIPOMENSAJE = ?");
			params.add(solicitudes.getMensajesSolicitud().getTipomensaje());
		}
		if (solicitudes != null && solicitudes.getMensajesSolicitud() != null
				&& solicitudes.getMensajesSolicitud().getFechaRegistro() != null) {
			where.append(" AND t2.FECHA_REGISTRO = ?");
			params.add(solicitudes.getMensajesSolicitud().getFechaRegistro());
		}
		if (solicitudes != null && solicitudes.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(solicitudes.getFechaRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the Solicitudes entity
	 *
	 * @param solicitudes
	 *            Solicitudes Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap(Solicitudes solicitudes, Boolean startsWith) {

		StringBuilder where = new StringBuilder(SolicitudesDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (solicitudes != null && solicitudes.getIdSolicitud() != null) {
			where.append(" AND UPPER(t1.ID_SOLICITUD) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(solicitudes.getIdSolicitud() + "%");
			} else {
				params.add("%" + solicitudes.getIdSolicitud() + "%");
			}
			where.append(" AND t1.ID_SOLICITUD IS NOT NULL");
		}
		if (solicitudes != null && solicitudes.getMensajesSolicitud() != null
				&& solicitudes.getMensajesSolicitud().getIdMensaje() != null) {
			where.append(" AND t2.ID_MENSAJE = ?");
			params.add(solicitudes.getMensajesSolicitud().getIdMensaje());
		}
		if (solicitudes != null && solicitudes.getMensajesSolicitud() != null
				&& solicitudes.getMensajesSolicitud().getTipomensaje() != null) {
			where.append(" AND UPPER(t2.TIPOMENSAJE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(solicitudes.getMensajesSolicitud().getTipomensaje().toUpperCase() + "%");
			} else {
				params.add("%" + solicitudes.getMensajesSolicitud().getTipomensaje().toUpperCase() + "%");
			}
			where.append(" AND t2.TIPOMENSAJE IS NOT NULL");
		}
		if (solicitudes != null && solicitudes.getMensajesSolicitud() != null
				&& solicitudes.getMensajesSolicitud().getFechaRegistro() != null) {
			where.append(" AND t2.FECHA_REGISTRO = ?");
			params.add(solicitudes.getMensajesSolicitud().getFechaRegistro());
		}
		if (solicitudes != null && solicitudes.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(solicitudes.getFechaRegistro());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;

}
