package com.ejie.ab61.dao.terceros;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.sql.DataSource;

import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.comun.utilidades.SecuenciasSQL;
import com.ejie.ab61.model.expedientes.RelExpTer;
import com.ejie.ab61.model.terceros.DatosTerceros;
import com.ejie.ab61.model.terceros.Sexo;
import com.ejie.ab61.model.terceros.TiposTercero;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * DatosTercerosDaoImpl  
 * 
 *  
 */

@Repository
@Transactional
public class DatosTercerosDaoImpl implements DatosTercerosDao {

	public static final String[] ORDER_BY_WHITE_LIST = new String[] { "IDTERCERO", "DOCUMENTO", "IDTIPODOCUMENTO",
			"NOMBRE", "APELLIDO1", "APELLIDO2", "IDSEXO", "TELEFONO", "EMAIL", "IDIOMA", "TIPOPARTICIPANTE",
			"FECHAREGISTRO", "USUARIOREGISTRO", "FECHAULTMODIFICACION", "USUARIOULTMODIFICACION",
			"DATOSTERCEROSIDTERCERO", "DATOSTERCEROSDOCUMENTO", "DATOSTERCEROSIDTIPODOCUMENTO", "DATOSTERCEROSNOMBRE",
			"DATOSTERCEROSAPELLIDO1", "DATOSTERCEROSAPELLIDO2", "DATOSTERCEROSIDSEXO", "DATOSTERCEROSTELEFONO",
			"DATOSTERCEROSEMAIL", "DATOSTERCEROSIDIOMA", "DATOSTERCEROSTIPOPARTICIPANTE", "DATOSTERCEROSFECHAREGISTRO",
			"DATOSTERCEROSUSUARIOREGISTRO", "DATOSTERCEROSFULTMOD", "DATOSTERCEROSUULTMOD",
			"DATOSTERCEROSIDREPRESENTANTE" };

	private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<DatosTerceros> rwMap = new RowMapper<DatosTerceros>() {
		public DatosTerceros mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new DatosTerceros(resultSet.getInt("IDTERCERO"), resultSet.getString("DOCUMENTO"),
					resultSet.getLong("IDTIPODOCUMENTO"), resultSet.getString("NOMBRE"),
					resultSet.getString("APELLIDO1"), resultSet.getString("APELLIDO2"), resultSet.getLong("IDSEXO"),
					resultSet.getLong("TELEFONO"), resultSet.getString("EMAIL"), resultSet.getInt("IDIOMA"),
					resultSet.getInt("TIPOPARTICIPANTE"), resultSet.getDate("FECHAREGISTRO"),
					resultSet.getString("USUARIOREGISTRO"), resultSet.getDate("FECHAULTMODIFICACION"),
					resultSet.getString("USUARIOULTMODIFICACION"));
		}
	};

	private RowMapper<DatosTerceros> rwMapTercero = new RowMapper<DatosTerceros>() {
		public DatosTerceros mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			DatosTerceros datosTerceros = new DatosTerceros();
			datosTerceros.setIdTercero(resultSet.getInt("IDTERCERO"));
			datosTerceros.setDocumento(resultSet.getString("DOCUMENTO"));
			datosTerceros.setIdTipoDocumento(resultSet.getLong("IDTIPODOCUMENTO"));
			datosTerceros.setNombre(resultSet.getString("NOMBRE"));
			datosTerceros.setApellido1(resultSet.getString("APELLIDO1"));
			datosTerceros.setApellido2(resultSet.getString("APELLIDO2"));
			datosTerceros.setIdSexo(resultSet.getLong("IDSEXO"));
			datosTerceros.setTelefono(resultSet.getLong("TELEFONO"));
			datosTerceros.setEmail(resultSet.getString("EMAIL"));
			datosTerceros.setTipoParticipante(resultSet.getInt("TIPOPARTICIPANTE"));
			Sexo sexo = new Sexo();
			sexo.setDescripcionCastellano(resultSet.getString("DESCRIPCIONCASTELLANO"));
			sexo.setDescripcionEuskera(resultSet.getString("DESCRIPCIONEUSKERA"));

			RelExpTer relExpTer = new RelExpTer();
			relExpTer.setParticipantId(resultSet.getString("PARTICIPANTID"));
			TiposTercero tiposTercero = new TiposTercero();
			tiposTercero.setIdTipoTercero(resultSet.getLong("IDTIPOTERCERO"));
			relExpTer.setTiposTercero(tiposTercero);
			List<RelExpTer> lista = new ArrayList<RelExpTer>();
			lista.add(relExpTer);
			datosTerceros.setRelExpTers(lista);
			datosTerceros.setSexoIdioma(sexo);
			return datosTerceros;

		}
	};

	private RowMapper<DatosTerceros> rwMapPK = new RowMapper<DatosTerceros>() {
		public DatosTerceros mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new DatosTerceros(resultSet.getInt("IDTERCERO"));
		}
	};

	/**
	 * Rowmapper para JerarquÃ­a
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	private RowMapper<JerarquiaDto<DatosTerceros>> rwMapJerarquia = new RowMapper<JerarquiaDto<DatosTerceros>>() {
		public JerarquiaDto<DatosTerceros> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			DatosTerceros datosterceros = new DatosTerceros(resultSet.getInt("IDTERCERO"),
					resultSet.getString("DOCUMENTO"), resultSet.getLong("IDTIPODOCUMENTO"),
					resultSet.getString("NOMBRE"), resultSet.getString("APELLIDO1"), resultSet.getString("APELLIDO2"),
					resultSet.getLong("IDSEXO"), resultSet.getLong("TELEFONO"), resultSet.getString("EMAIL"),
					resultSet.getInt("IDIOMA"), resultSet.getInt("TIPOPARTICIPANTE"),
					resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"),
					resultSet.getDate("FECHAULTMODIFICACION"), resultSet.getString("USUARIOULTMODIFICACION"));

			JerarquiaDto<DatosTerceros> jerarquia = new JerarquiaDto<DatosTerceros>();
			jerarquia.setModel(datosterceros);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */
	@Resource
	public void setDataSource(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	}

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Method use to set the datasource.
	 *
	 * @param dataSource
	 *            DataSource
	 * @return
	 */

	/*
	 * OPERACIONES CRUD
	 */

	/**
	 * Inserts a single row in the DatosTerceros table.
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	public DatosTerceros add(DatosTerceros datosterceros) {
		int secuencia = this.jdbcTemplate.queryForInt(SecuenciasSQL.getSelectNext("AB61.S_AB6135T00"));
		String query = "INSERT INTO DATOS_TERCEROS (ID_TERCERO, DOCUMENTO, ID_TIPO_DOCUMENTO, NOMBRE, APELLIDO1, APELLIDO2, ID_SEXO, TELEFONO, EMAIL, IDIOMA, TIPO_PARTICIPANTE, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)";

		this.jdbcTemplate.update(query, secuencia, datosterceros.getDocumento(), datosterceros.getIdTipoDocumento(),
				datosterceros.getNombre(), datosterceros.getApellido1(), datosterceros.getApellido2(),
				datosterceros.getIdSexo(), datosterceros.getTelefono(), datosterceros.getEmail(),
				datosterceros.getIdioma(), datosterceros.getTipoParticipante(), new Date(),
				datosterceros.getUsuarioRegistro());
		return datosterceros;
	}

	/**
	 * Finds el siguiente al ultimo id de la tabla in the DatosTerceros table.
	 *
	 * @return datosTercerosiD
	 */
	@Transactional(readOnly = true)
	public DatosTerceros findUltimoId() {
		String query = "SELECT ID_TERCERO AS IDTERCERO FROM DATOS_TERCEROS WHERE ID_TERCERO = (SELECT MAX(ID_TERCERO) FROM DATOS_TERCEROS)";
		List<DatosTerceros> datosTercerosiD = this.jdbcTemplate.query(query, this.rwMapPK);
		return (DatosTerceros) DataAccessUtils.uniqueResult(datosTercerosiD);

	}

	/**
	 * Insert los terceros que viene de searchFolder
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	public DatosTerceros addTerceroSearchFolder(DatosTerceros datosterceros) {
		String query = "INSERT INTO DATOS_TERCEROS (ID_TERCERO,  DOCUMENTO, ID_TIPO_DOCUMENTO,IDIOMA, NOMBRE, APELLIDO1, APELLIDO2, TIPO_PARTICIPANTE, FECHA_REGISTRO, USUARIO_REGISTRO ) VALUES (AB61.S_AB6135T00.NEXTVAL,?,?,?,?,?,?,?,?,?)";

		this.jdbcTemplate.update(query, datosterceros.getDocumento(), datosterceros.getIdTipoDocumento(),
				datosterceros.getIdioma(), datosterceros.getNombre(), datosterceros.getApellido1(),
				datosterceros.getApellido2(), datosterceros.getTipoParticipante(), datosterceros.getFechaRegistro(),
				datosterceros.getUsuarioRegistro());
		return datosterceros;
	}

	/**
	 * Upadte los terceros que viene de searchFolder
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	public DatosTerceros updateTerceroSearchfolder(DatosTerceros datosterceros) {
		String query = "UPDATE AB61.DATOS_TERCEROS SET  DOCUMENTO=?, ID_TIPO_DOCUMENTO=?, IDIOMA = ? ,  NOMBRE=?, APELLIDO1=?, APELLIDO2=?,  TIPO_PARTICIPANTE=?,  FECHA_ULTMODIFICACION=?,  USUARIO_ULTMODIFICACION=?   WHERE ID_TERCERO=?";

		this.jdbcTemplate.update(query, datosterceros.getDocumento(), datosterceros.getIdTipoDocumento(),
				datosterceros.getIdioma(), datosterceros.getNombre(), datosterceros.getApellido1(),
				datosterceros.getApellido2(), datosterceros.getTipoParticipante(),
				datosterceros.getFechaUltmodificacion(), datosterceros.getUsuarioUltmodificacion(),
				datosterceros.getIdTercero());
		return datosterceros;
	}

	/**
	 * Updates a single row in the DatosTerceros table.
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	public DatosTerceros update(DatosTerceros datosterceros) {
		String query = "UPDATE DATOS_TERCEROS SET  DOCUMENTO=?, ID_TIPO_DOCUMENTO=? , IDIOMA=? , NOMBRE=?, APELLIDO1=?, APELLIDO2=?, ID_SEXO=?, TELEFONO=?, EMAIL=?,  FECHA_ULTMODIFICACION=?, USUARIO_ULTMODIFICACION=? WHERE ID_TERCERO=?";

		this.jdbcTemplate.update(query, datosterceros.getDocumento(), datosterceros.getIdTipoDocumento(),
				datosterceros.getIdioma(), datosterceros.getNombre(), datosterceros.getApellido1(),
				datosterceros.getApellido2(), datosterceros.getIdSexo(), datosterceros.getTelefono(),
				datosterceros.getEmail(), datosterceros.getFechaUltmodificacion(),
				datosterceros.getUsuarioUltmodificacion(), datosterceros.getIdTercero());
		return datosterceros;
	}

	/**
	 * Finds a single row in the DatosTerceros table.
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	@Transactional(readOnly = true)
	public DatosTerceros findTercero(Integer idTercero, String partipantId) {
		String query = "SELECT  t1.ID_TERCERO IDTERCERO, t1.DOCUMENTO DOCUMENTO, t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO, t1.NOMBRE NOMBRE, t1.APELLIDO1 APELLIDO1, t1.APELLIDO2 APELLIDO2, t1.ID_SEXO IDSEXO, t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE, t2.PARTICIPANT_ID PARTICIPANTID, t2.ID_TIPO_TERCERO IDTIPOTERCERO, t3.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t3.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA FROM DATOS_TERCEROS t1, REL_EXP_TER t2, SEXO t3 WHERE t1.ID_TERCERO  = ? AND t2.PARTICIPANT_ID = ? AND T2.ID_TERCERO(+) = t1.ID_TERCERO  AND t3.ID_SEXO(+) = t1.ID_SEXO";
		List<DatosTerceros> datostercerosList = this.jdbcTemplate.query(query, this.rwMapTercero, idTercero,
				partipantId);
		return (DatosTerceros) DataAccessUtils.uniqueResult(datostercerosList);
	}

	/**
	 * Encuentra los datos de tercero tipo : persona de contacto o responsable
	 * legal
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	@Transactional(readOnly = true)
	public DatosTerceros findPersonacontactoResponsableLegal(Integer idTercero, Long tipo, Integer idExpediente) {
		String query = "SELECT t1.ID_TERCERO IDTERCERO, t1.DOCUMENTO DOCUMENTO, t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO, t1.NOMBRE NOMBRE, t1.APELLIDO1 APELLIDO1, t1.APELLIDO2 APELLIDO2, t1.ID_SEXO IDSEXO, t1.TELEFONO TELEFONO , t1.EMAIL EMAIL,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE, t2.PARTICIPANT_ID PARTICIPANTID, t2.ID_TIPO_TERCERO IDTIPOTERCERO, t3.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t3.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA FROM  DATOS_TERCEROS t1, REL_EXP_TER t2, SEXO t3 WHERE t1.ID_TERCERO   = ? AND t2.ID_EXPEDIENTE = ? AND t2.ID_TIPO_TERCERO = ? AND T2.ID_TERCERO(+)  = t1.ID_TERCERO AND t3.ID_SEXO(+) = t1.ID_SEXO";
		List<DatosTerceros> datostercerosList = this.jdbcTemplate.query(query, this.rwMapTercero, idTercero,
				idExpediente, tipo);
		return (DatosTerceros) DataAccessUtils.uniqueResult(datostercerosList);
	}

	/**
	 * Remove a single row in the DATOS_TERCEROS dado un idTercero
	 *
	 * @param idTercero
	 *            Integer
	 * @return
	 */
	public void remove(Integer idTercero) {
		String query = "DELETE FROM DATOS_TERCEROS WHERE ID_TERCERO=?";
		this.jdbcTemplate.update(query, idTercero);
	}

	/**
	 * Finds a single row in the DatosTercedos table.
	 *
	 * @param datosTerceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	@Transactional(readOnly = true)
	public DatosTerceros find(DatosTerceros datosTerceros) {
		String query = "SELECT t1.ID_TERCERO IDTERCERO,t1.DOCUMENTO DOCUMENTO,t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO,t1.NOMBRE NOMBRE,t1.APELLIDO1 APELLIDO1 ,t1.APELLIDO2 APELLIDO2 ,t1.ID_SEXO IDSEXO,t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.IDIOMA IDIOMA,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.ID_REPRESENTANTE IDREPRESENTANTE,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM DATOS_TERCEROS t1 WHERE t1.ID_TERCERO=?";

		List<DatosTerceros> datosTercerosList = this.jdbcTemplate.query(query, this.rwMap,
				datosTerceros.getIdTercero());
		return (DatosTerceros) DataAccessUtils.uniqueResult(datosTercerosList);
	}

	@Override
	public List<DatosTerceros> findTitularExpediente(Integer idExpediente) {
		String query = "SELECT t1.ID_TERCERO IDTERCERO,t1.DOCUMENTO DOCUMENTO,t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO,t1.NOMBRE NOMBRE,t1.APELLIDO1 APELLIDO1,t1.APELLIDO2 APELLIDO2,t1.ID_SEXO IDSEXO,t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.IDIOMA IDIOMA,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION FROM DATOS_TERCEROS t1, REL_EXP_TER t2, EXPEDIENTES t3 WHERE t1.ID_TERCERO = t2.ID_TERCERO AND T2.ID_EXPEDIENTE = T3.ID_EXPEDIENTE AND t1.TIPO_PARTICIPANTE = ? AND t3.ID_EXPEDIENTE = ? ";

		return (List<DatosTerceros>) this.jdbcTemplate.query(query.toString(), this.rwMap, Constantes.MAGIC_NUMBER_0,
				idExpediente);
	}

	/**
	 * Finds a list of rows in the DatosTerceros table.
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<DatosTerceros>
	 */
	@Transactional(readOnly = true)
	public List<DatosTerceros> findAll(DatosTerceros datosterceros, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_TERCERO IDTERCERO,t1.DOCUMENTO DOCUMENTO,t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO,t1.NOMBRE NOMBRE,t1.APELLIDO1 APELLIDO1,t1.APELLIDO2 APELLIDO2,t1.ID_SEXO IDSEXO,t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.IDIOMA IDIOMA,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		query.append("FROM DATOS_TERCEROS t1 ,DATOS_TERCEROS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(datosterceros);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_REPRESENTANTE= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DatosTercerosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<DatosTerceros>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the DatosTerceros table using like.
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<DatosTerceros>
	 */
	@Transactional(readOnly = true)
	public List<DatosTerceros> findAllLike(DatosTerceros datosterceros, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		StringBuilder query = new StringBuilder(
				"SELECT  t1.ID_TERCERO IDTERCERO,t1.DOCUMENTO DOCUMENTO,t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO,t1.NOMBRE NOMBRE,t1.APELLIDO1 APELLIDO1,t1.APELLIDO2 APELLIDO2,t1.ID_SEXO IDSEXO,t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.IDIOMA IDIOMA,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		query.append("FROM DATOS_TERCEROS t1 ,DATOS_TERCEROS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(datosterceros, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_REPRESENTANTE= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, DatosTercerosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<DatosTerceros>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Counts rows in the DatosTerceros table.
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllCount(DatosTerceros datosterceros) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DATOS_TERCEROS t1 , DATOS_TERCEROS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(datosterceros);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_REPRESENTANTE= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the DatosTerceros table using like.
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @param startsWith
	 *            Boolean
	 * @return Long
	 */
	@Transactional(readOnly = true)
	public Long findAllLikeCount(DatosTerceros datosterceros, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM DATOS_TERCEROS t1 ,DATOS_TERCEROS t2 ");

		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(datosterceros, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_REPRESENTANTE= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of DatosTerceros selected for rup_table
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<DatosTerceros>>
	 */
	@Override
	public List<TableRowDto<DatosTerceros>> reorderSelection(DatosTerceros datosterceros,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_TERCERO IDTERCERO,t1.DOCUMENTO DOCUMENTO,t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO,t1.NOMBRE NOMBRE,t1.APELLIDO1 APELLIDO1,t1.APELLIDO2 APELLIDO2,t1.ID_SEXO IDSEXO,t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.IDIOMA IDIOMA,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		// FROM
		query.append("FROM DATOS_TERCEROS t1 ,DATOS_TERCEROS t2 ");
		// FILTRADO
		// Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(datosterceros, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_REPRESENTANTE= t2.ID_TERCERO(+) ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// ParÃ¡metros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenaciÃ³n
		StringBuilder sbReorderSelectionSQL = JQGridManager.getReorderQuery(query, jqGridRequestDto,
				DatosTerceros.class, filterParamList, "ID_TERCERO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<DatosTerceros>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
	 *
	 * @param filterParams
	 *            DatosTerceros
	 * @param searchParams
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<DatosTerceros>>
	 */
	@Override
	public List<TableRowDto<DatosTerceros>> search(DatosTerceros filterParams, DatosTerceros searchParams,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_TERCERO IDTERCERO,t1.DOCUMENTO DOCUMENTO,t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO,t1.NOMBRE NOMBRE,t1.APELLIDO1 APELLIDO1,t1.APELLIDO2 APELLIDO2,t1.ID_SEXO IDSEXO,t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.IDIOMA IDIOMA,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		// FROM
		query.append("FROM DATOS_TERCEROS t1 ,DATOS_TERCEROS t2 ");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 AND t1.ID_REPRESENTANTE= t2.ID_TERCERO(+) ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, DatosTerceros.class,
				filterParamList, searchSQL, searchParamList, from_alias, "ID_TERCERO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(),
				new RowNumResultSetExtractor<DatosTerceros>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
	 *
	 * @param filterdatosterceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	@Override
	public void removeMultiple(DatosTerceros filterdatosterceros, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_TERCERO IDTERCERO,t1.DOCUMENTO DOCUMENTO,t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO,t1.NOMBRE NOMBRE,t1.APELLIDO1 APELLIDO1,t1.APELLIDO2 APELLIDO2,t1.ID_SEXO IDSEXO,t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.IDIOMA IDIOMA,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		// FROM
		query.append("FROM DATOS_TERCEROS t1 ,DATOS_TERCEROS t2 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filterdatosterceros, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, DatosTerceros.class,
				query, params, "ID_TERCERO");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical DatosTerceros table.
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<JerarquiaDto<DatosTerceros>>
	 */
	@Override
	public List<JerarquiaDto<DatosTerceros>> findAllLikeJerarquia(DatosTerceros filterDatosTerceros,
			JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder(
				"SELECT t1.ID_TERCERO IDTERCERO,t1.DOCUMENTO DOCUMENTO,t1.ID_TIPO_DOCUMENTO IDTIPODOCUMENTO,t1.NOMBRE NOMBRE,t1.APELLIDO1 APELLIDO1,t1.APELLIDO2 APELLIDO2,t1.ID_SEXO IDSEXO,t1.TELEFONO TELEFONO,t1.EMAIL EMAIL,t1.IDIOMA IDIOMA,t1.TIPO_PARTICIPANTE TIPOPARTICIPANTE,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO,t1.FECHA_ULTMODIFICACION FECHAULTMODIFICACION,t1.USUARIO_ULTMODIFICACION USUARIOULTMODIFICACION,t2.ID_TERCERO DATOSTERCEROSIDTERCERO,t2.DOCUMENTO DATOSTERCEROSDOCUMENTO,t2.ID_TIPO_DOCUMENTO DATOSTERCEROSIDTIPODOCUMENTO,t2.NOMBRE DATOSTERCEROSNOMBRE,t2.APELLIDO1 DATOSTERCEROSAPELLIDO1,t2.APELLIDO2 DATOSTERCEROSAPELLIDO2,t2.ID_SEXO DATOSTERCEROSIDSEXO,t2.TELEFONO DATOSTERCEROSTELEFONO,t2.EMAIL DATOSTERCEROSEMAIL,t2.IDIOMA DATOSTERCEROSIDIOMA,t2.TIPO_PARTICIPANTE DATOSTERCEROSTIPOPARTICIPANTE,t2.FECHA_REGISTRO DATOSTERCEROSFECHAREGISTRO,t2.USUARIO_REGISTRO DATOSTERCEROSUSUARIOREGISTRO,t2.FECHA_ULTMODIFICACION DATOSTERCEROSFULTMOD,t2.USUARIO_ULTMODIFICACION DATOSTERCEROSUULTMOD,t2.ID_REPRESENTANTE DATOSTERCEROSIDREPRESENTANTE ");
		// FROM
		// StringBuilder from = new StringBuilder("FROM DATOS_TERCEROS t1
		// ,DATOS_TERCEROS t2 ");
		List<String> from = new ArrayList<String>();
		from.add("DATOS_TERCEROS");
		from.add("DATOS_TERCEROS");
		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquÃ­a
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDatosTerceros, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_TERCERO", "ID_PADRE", "NOMBRE",
				from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query,
		// mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins,
		// businessFilters, businessParams);

		// PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query,
					DatosTercerosDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return Long
	 */
	@Override
	public Long findAllLikeCountJerarquia(DatosTerceros filterDatosTerceros, JQGridRequestDto jqGridRequestDto) {
		// FROM
		// StringBuilder from = new StringBuilder("FROM DATOS_TERCEROS t1
		// ,DATOS_TERCEROS t2 ");
		List<String> from = new ArrayList<String>();
		from.add("DATOS_TERCEROS");
		from.add("DATOS_TERCEROS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquÃ­a
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDatosTerceros, false);

		// JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_TERCERO",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TableRowDto<DatosTerceros>>
	 */
	@Override
	public List<TableRowDto<DatosTerceros>> findAllChild(DatosTerceros filterDatosTerceros,
			JQGridRequestDto jqGridRequestDto) {

		// FROM
		// StringBuilder from = new StringBuilder("FROM DATOS_TERCEROS t1
		// ,DATOS_TERCEROS t2 ");
		List<String> from = new ArrayList<String>();
		from.add("DATOS_TERCEROS");
		from.add("DATOS_TERCEROS");

		// TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");
		from_alias.add(" t2 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquÃ­a
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		// FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterDatosTerceros, false);

		// MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada
		// por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_TERCERO",
				"ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y
		 * condiciones de negocio
		 */
		// StringBuilder query =
		// JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere,
		// "ID", "ID_PADRE", from, from_alias, joins, businessFilters,
		// businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(),
				new RowNumResultSetExtractor<DatosTerceros>(this.rwMapPK, "ID_TERCERO"), params.toArray());
	}

	/*
	 * MÃ‰TODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the DatosTerceros entity
	 *
	 * @param datosterceros
	 *            DatosTerceros Bean with the criteria values to filter by.
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - GeneraciÃ³n de cÃ³digo de UDA
	private Map<String, ?> getWhereMap(DatosTerceros datosterceros) {

		StringBuilder where = new StringBuilder(DatosTercerosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (datosterceros != null && datosterceros.getIdTercero() != null) {
			where.append(" AND t1.ID_TERCERO = ?");
			params.add(datosterceros.getIdTercero());
		}

		if (datosterceros != null && datosterceros.getDocumento() != null) {
			where.append(" AND t1.DOCUMENTO = ?");
			params.add(datosterceros.getDocumento());
		}
		if (datosterceros != null && datosterceros.getIdTipoDocumento() != null) {
			where.append(" AND t1.ID_TIPO_DOCUMENTO = ?");
			params.add(datosterceros.getIdTipoDocumento());
		}
		if (datosterceros != null && datosterceros.getNombre() != null) {
			where.append(" AND t1.NOMBRE = ?");
			params.add(datosterceros.getNombre());
		}
		if (datosterceros != null && datosterceros.getApellido1() != null) {
			where.append(" AND t1.APELLIDO1 = ?");
			params.add(datosterceros.getApellido1());
		}
		if (datosterceros != null && datosterceros.getApellido2() != null) {
			where.append(" AND t1.APELLIDO2 = ?");
			params.add(datosterceros.getApellido2());
		}
		if (datosterceros != null && datosterceros.getIdSexo() != null) {
			where.append(" AND t1.ID_SEXO = ?");
			params.add(datosterceros.getIdSexo());
		}
		if (datosterceros != null && datosterceros.getTelefono() != null) {
			where.append(" AND t1.TELEFONO = ?");
			params.add(datosterceros.getTelefono());
		}
		if (datosterceros != null && datosterceros.getEmail() != null) {
			where.append(" AND t1.EMAIL = ?");
			params.add(datosterceros.getEmail());
		}
		if (datosterceros != null && datosterceros.getIdioma() != null) {
			where.append(" AND t1.IDIOMA = ?");
			params.add(datosterceros.getIdioma());
		}
		if (datosterceros != null && datosterceros.getTipoParticipante() != null) {
			where.append(" AND t1.TIPO_PARTICIPANTE = ?");
			params.add(datosterceros.getTipoParticipante());
		}
		if (datosterceros != null && datosterceros.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(datosterceros.getFechaRegistro());
		}
		if (datosterceros != null && datosterceros.getUsuarioRegistro() != null) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(datosterceros.getUsuarioRegistro());
		}
		if (datosterceros != null && datosterceros.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(datosterceros.getFechaUltmodificacion());
		}
		if (datosterceros != null && datosterceros.getUsuarioUltmodificacion() != null) {
			where.append(" AND t1.USUARIO_ULTMODIFICACION = ?");
			params.add(datosterceros.getUsuarioUltmodificacion());
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - GeneraciÃ³n de cÃ³digo de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the DatosTerceros entity
	 *
	 * @param datosterceros
	 *            DatosTerceros Bean with the criteria values to filter by.
	 * @param startsWith
	 *            Boolean
	 * @return Map created with two keys key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the
	 *         condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - GeneraciÃ³n de cÃ³digo de UDA
	private Map<String, Object> getWhereLikeMap(DatosTerceros datosterceros, Boolean startsWith) {

		StringBuilder where = new StringBuilder(DatosTercerosDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (datosterceros != null && datosterceros.getIdTercero() != null) {
			where.append(" AND t1.ID_TERCERO = ?");
			params.add(datosterceros.getIdTercero());
		}

		if (datosterceros != null && datosterceros.getDocumento() != null) {
			where.append(" AND UPPER(t1.DOCUMENTO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(datosterceros.getDocumento().toUpperCase() + "%");
			} else {
				params.add("%" + datosterceros.getDocumento().toUpperCase() + "%");
			}
			where.append(" AND t1.DOCUMENTO IS NOT NULL");
		}
		if (datosterceros != null && datosterceros.getIdTipoDocumento() != null) {
			where.append(" AND t1.ID_TIPO_DOCUMENTO = ?");
			params.add(datosterceros.getIdTipoDocumento());
		}
		if (datosterceros != null && datosterceros.getNombre() != null) {
			where.append(" AND UPPER(t1.NOMBRE) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(datosterceros.getNombre().toUpperCase() + "%");
			} else {
				params.add("%" + datosterceros.getNombre().toUpperCase() + "%");
			}
			where.append(" AND t1.NOMBRE IS NOT NULL");
		}
		if (datosterceros != null && datosterceros.getApellido1() != null) {
			where.append(" AND UPPER(t1.APELLIDO1) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(datosterceros.getApellido1().toUpperCase() + "%");
			} else {
				params.add("%" + datosterceros.getApellido1().toUpperCase() + "%");
			}
			where.append(" AND t1.APELLIDO1 IS NOT NULL");
		}
		if (datosterceros != null && datosterceros.getApellido2() != null) {
			where.append(" AND UPPER(t1.APELLIDO2) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(datosterceros.getApellido2().toUpperCase() + "%");
			} else {
				params.add("%" + datosterceros.getApellido2().toUpperCase() + "%");
			}
			where.append(" AND t1.APELLIDO2 IS NOT NULL");
		}
		if (datosterceros != null && datosterceros.getIdSexo() != null) {
			where.append(" AND t1.ID_SEXO = ?");
			params.add(datosterceros.getIdSexo());
		}
		if (datosterceros != null && datosterceros.getTelefono() != null) {
			where.append(" AND t1.TELEFONO = ?");
			params.add(datosterceros.getTelefono());
		}
		if (datosterceros != null && datosterceros.getEmail() != null) {
			where.append(" AND UPPER(t1.EMAIL) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(datosterceros.getEmail().toUpperCase() + "%");
			} else {
				params.add("%" + datosterceros.getEmail().toUpperCase() + "%");
			}
			where.append(" AND t1.EMAIL IS NOT NULL");
		}
		if (datosterceros != null && datosterceros.getIdioma() != null) {
			where.append(" AND t1.IDIOMA = ?");
			params.add(datosterceros.getIdioma());
		}
		if (datosterceros != null && datosterceros.getTipoParticipante() != null) {
			where.append(" AND t1.TIPO_PARTICIPANTE = ?");
			params.add(datosterceros.getTipoParticipante());
		}
		if (datosterceros != null && datosterceros.getFechaRegistro() != null) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(datosterceros.getFechaRegistro());
		}
		if (datosterceros != null && datosterceros.getUsuarioRegistro() != null) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(datosterceros.getUsuarioRegistro().toUpperCase() + "%");
			} else {
				params.add("%" + datosterceros.getUsuarioRegistro().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
		}
		if (datosterceros != null && datosterceros.getFechaUltmodificacion() != null) {
			where.append(" AND t1.FECHA_ULTMODIFICACION = ?");
			params.add(datosterceros.getFechaUltmodificacion());
		}
		if (datosterceros != null && datosterceros.getUsuarioUltmodificacion() != null) {
			where.append(" AND UPPER(t1.USUARIO_ULTMODIFICACION) like ? ESCAPE  '\\'");
			if (startsWith) {
				params.add(datosterceros.getUsuarioUltmodificacion().toUpperCase() + "%");
			} else {
				params.add("%" + datosterceros.getUsuarioUltmodificacion().toUpperCase() + "%");
			}
			where.append(" AND t1.USUARIO_ULTMODIFICACION IS NOT NULL");
		}

		Map<String, Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - GeneraciÃ³n de cÃ³digo de UDA

	/**
	 * StringBuilder initilization value
	 */
	public static final int STRING_BUILDER_INIT = 4096;
}
