package com.ejie.ab61.dao.terceros;

import com.ejie.ab61.model.terceros.TiposCanalAviso;
import com.ejie.x38.dao.RowNumResultSetExtractor;
import com.ejie.x38.dto.JQGridManager;
import com.ejie.x38.dto.JQGridManagerJerarquia;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import javax.sql.DataSource;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

/**
 * TiposCanalAvisoDaoImpl  
 *  
 */

@Repository
@Transactional
public class TiposCanalAvisoDaoImpl implements TiposCanalAvisoDao {


    public static final String[] ORDER_BY_WHITE_LIST = new String[] {"IDTIPOCANALAVISO", "DESCRIPCIONCASTELLANO", "DESCRIPCIONEUSKERA", "FECHAREGISTRO", "USUARIOREGISTRO"};

    private JdbcTemplate jdbcTemplate;

	/*
	 * ROW_MAPPERS
	 */
	private RowMapper<TiposCanalAviso> rwMap = new RowMapper<TiposCanalAviso>() {
		public TiposCanalAviso mapRow(ResultSet resultSet, int rowNum) throws SQLException {
           return new TiposCanalAviso(
               resultSet.getLong("IDTIPOCANALAVISO"), resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO")
           ); } } ;

	private RowMapper<TiposCanalAviso> rwMapPK = new RowMapper<TiposCanalAviso>() {
		public TiposCanalAviso mapRow(ResultSet resultSet, int rowNum) throws SQLException {
			return new TiposCanalAviso(
				resultSet.getLong("IDTIPOCANALAVISO")
           ); } } ;

	/**
     * Rowmapper para Jerarquía
     *
     * @param dataSource DataSource
     * @return
     */
	private RowMapper<JerarquiaDto< TiposCanalAviso>> rwMapJerarquia = new RowMapper<JerarquiaDto<TiposCanalAviso>>() {
		public JerarquiaDto<TiposCanalAviso> mapRow(ResultSet resultSet, int rowNum) throws SQLException {

			TiposCanalAviso tiposcanalaviso = new TiposCanalAviso(resultSet.getLong("IDTIPOCANALAVISO"), resultSet.getString("DESCRIPCIONCASTELLANO"), resultSet.getString("DESCRIPCIONEUSKERA"), resultSet.getDate("FECHAREGISTRO"), resultSet.getString("USUARIOREGISTRO"));

			JerarquiaDto<TiposCanalAviso> jerarquia = new JerarquiaDto<TiposCanalAviso>();
			jerarquia.setModel(tiposcanalaviso);
			jerarquia.setLevel(resultSet.getBigDecimal("LEVEL").intValue());
			jerarquia.setParentNodes(resultSet.getString("PARENTNODES"));
			jerarquia.setIsLeaf(Boolean.parseBoolean(resultSet.getString("ISLEAF")));
			jerarquia.setFilter(Boolean.parseBoolean(resultSet.getString("FILTER")));
			return jerarquia;
		}
	};


	/**
     * Method use to set the datasource.
     *
     * @param dataSource DataSource
     * @return
     */
    @Resource
    public void setDataSource(DataSource dataSource) {
    	this.jdbcTemplate = new JdbcTemplate(dataSource);
    }

	/*
	 * OPERACIONES CRUD
	 */

    /**
     * Inserts a single row in the TiposCanalAviso table.
     *
     * @param tiposcanalaviso TiposCanalAviso
     * @return TiposCanalAviso
     */
	public TiposCanalAviso add(TiposCanalAviso tiposcanalaviso) {
    	String query = "INSERT INTO TIPOS_CANAL_AVISO (ID_TIPO_CANAL_AVISO, DESCRIPCION_CASTELLANO, DESCRIPCION_EUSKERA, FECHA_REGISTRO, USUARIO_REGISTRO) VALUES (?,?,?,?,?)";
		this.jdbcTemplate.update(query, tiposcanalaviso.getIdTipoCanalAviso(), tiposcanalaviso.getDescripcionCastellano(), tiposcanalaviso.getDescripcionEuskera(), tiposcanalaviso.getFechaRegistro(), tiposcanalaviso.getUsuarioRegistro());
		return tiposcanalaviso;
	}

    /**
     * Updates a single row in the TiposCanalAviso table.
     *
     * @param tiposcanalaviso TiposCanalAviso
     * @return TiposCanalAviso
     */
    public TiposCanalAviso update(TiposCanalAviso tiposcanalaviso) {
		String query = "UPDATE TIPOS_CANAL_AVISO SET DESCRIPCION_CASTELLANO=?, DESCRIPCION_EUSKERA=?, FECHA_REGISTRO=?, USUARIO_REGISTRO=? WHERE ID_TIPO_CANAL_AVISO=?";
		this.jdbcTemplate.update(query, tiposcanalaviso.getDescripcionCastellano(), tiposcanalaviso.getDescripcionEuskera(), tiposcanalaviso.getFechaRegistro(), tiposcanalaviso.getUsuarioRegistro(), tiposcanalaviso.getIdTipoCanalAviso());
		return tiposcanalaviso;
	}

    /**
     * Finds a single row in the TiposCanalAviso table.
     *
     * @param tiposcanalaviso TiposCanalAviso
     * @return TiposCanalAviso
     */
    @Transactional (readOnly = true)
    public TiposCanalAviso find(TiposCanalAviso tiposcanalaviso) {
		String query = "SELECT t1.ID_TIPO_CANAL_AVISO IDTIPOCANALAVISO, t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO, t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA, t1.FECHA_REGISTRO FECHAREGISTRO, t1.USUARIO_REGISTRO USUARIOREGISTRO FROM TIPOS_CANAL_AVISO t1  WHERE t1.ID_TIPO_CANAL_AVISO = ?  ";

		List<TiposCanalAviso> tiposcanalavisoList = this.jdbcTemplate.query(query, this.rwMap, tiposcanalaviso.getIdTipoCanalAviso());
		return (TiposCanalAviso) DataAccessUtils.uniqueResult(tiposcanalavisoList);
    }

    /**
     * Removes a single row in the TiposCanalAviso table.
     *
     * @param tiposcanalaviso TiposCanalAviso
     * @return
     */
    public void remove(TiposCanalAviso tiposcanalaviso) {
		String query = "DELETE FROM TIPOS_CANAL_AVISO WHERE ID_TIPO_CANAL_AVISO=?";
		this.jdbcTemplate.update(query, tiposcanalaviso.getIdTipoCanalAviso());
    }

   /**
    * Finds a list of rows in the TiposCanalAviso table.
    *
    * @param tiposcanalaviso TiposCanalAviso
    * @param jqGridRequestDto JQGridRequestDto
    * @return List<TiposCanalAviso>
    */
	@Transactional (readOnly = true)
    public List<TiposCanalAviso> findAll(TiposCanalAviso tiposcanalaviso, JQGridRequestDto jqGridRequestDto) {
		StringBuilder query = new StringBuilder("SELECT  t1.ID_TIPO_CANAL_AVISO IDTIPOCANALAVISO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		query.append("FROM TIPOS_CANAL_AVISO t1 ");

		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(tiposcanalaviso); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, TiposCanalAvisoDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<TiposCanalAviso>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/**
	 * Finds rows in the TiposCanalAviso table using like.
     *
     * @param tiposcanalaviso TiposCanalAviso
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     * @return List<TiposCanalAviso>
     */
	@Transactional (readOnly = true)
    public List<TiposCanalAviso> findAllLike(TiposCanalAviso tiposcanalaviso, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT  t1.ID_TIPO_CANAL_AVISO IDTIPOCANALAVISO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
        query.append("FROM TIPOS_CANAL_AVISO t1 ");

		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(tiposcanalaviso,startsWith); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		if (jqGridRequestDto != null) {
			query = JQGridManager.getPaginationQuery(jqGridRequestDto, query, TiposCanalAvisoDaoImpl.ORDER_BY_WHITE_LIST);
		}

		return (List<TiposCanalAviso>) this.jdbcTemplate.query(query.toString(), this.rwMap, params.toArray());
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

    /**
     * Counts rows in the TiposCanalAviso table.
     *
     * @param tiposcanalaviso TiposCanalAviso
     * @return Long
     */
    @Transactional (readOnly = true)
    public Long findAllCount(TiposCanalAviso tiposcanalaviso) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM TIPOS_CANAL_AVISO t1 ");

		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereMap(tiposcanalaviso); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Counts rows in the TiposCanalAviso table using like.
     *
     * @param tiposcanalaviso TiposCanalAviso
     * @param startsWith Boolean
     * @return Long
     */
	@Transactional (readOnly = true)
    public Long findAllLikeCount(TiposCanalAviso tiposcanalaviso, Boolean startsWith) {
		StringBuilder query = new StringBuilder("SELECT COUNT(1) FROM TIPOS_CANAL_AVISO t1 ");

		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(tiposcanalaviso,startsWith); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Reorder the data list of TiposCanalAviso selected for rup_table
     *
     * @param tiposcanalaviso TiposCanalAviso
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     * @return List<TableRowDto<TiposCanalAviso>>
     */
	@Override
	public List<TableRowDto<TiposCanalAviso>> reorderSelection(TiposCanalAviso tiposcanalaviso, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder("SELECT t1.ID_TIPO_CANAL_AVISO IDTIPOCANALAVISO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
        query.append("FROM TIPOS_CANAL_AVISO t1 ");
		// FILTRADO
		//Where clause & Params
		Map<String, ?> mapaWhere = this.getWhereLikeMap(tiposcanalaviso,startsWith); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		// Parámetros de filtrado
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhere.get("params");

		// SQL para la reordenación
		StringBuilder sbReorderSelectionSQL =  JQGridManager.getReorderQuery(query, jqGridRequestDto, TiposCanalAviso.class, filterParamList, "ID_TIPO_CANAL_AVISO" );

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(), new RowNumResultSetExtractor<TiposCanalAviso>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Search method for rup_table
     *
     * @param filterParams TiposCanalAviso
     * @param searchParams TiposCanalAviso
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     * @return List<TableRowDto<TiposCanalAviso>>
     */
	@Override
	public List<TableRowDto<TiposCanalAviso>> search(TiposCanalAviso filterParams, TiposCanalAviso searchParams, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		StringBuilder query = new StringBuilder("SELECT t1.ID_TIPO_CANAL_AVISO IDTIPOCANALAVISO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
        query.append("FROM TIPOS_CANAL_AVISO t1 ");
        //TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// FILTRADO
		Map<String, ?> mapaWhereFilter = this.getWhereLikeMap(filterParams, startsWith); 
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhereFilter.get("query"));
		query.append(where);
		// Filter params
		@SuppressWarnings("unchecked")
		List<Object> filterParamList = (List<Object>) mapaWhereFilter.get("params");

		// BUSQUEDA
		Map<String, Object> mapaWhereSearch = this.getWhereLikeMap(searchParams, startsWith);
		String searchSQL = ((StringBuilder) mapaWhereSearch.get("query")).toString();
		// Search params
		@SuppressWarnings("unchecked")
		List<Object> searchParamList = (List<Object>) mapaWhereSearch.get("params");

		// SQL
		StringBuilder sbReorderSelectionSQL = JQGridManager.getSearchQuery(query, jqGridRequestDto, TiposCanalAviso.class, filterParamList, searchSQL, searchParamList, from_alias, "ID_TIPO_CANAL_AVISO");

		return this.jdbcTemplate.query(sbReorderSelectionSQL.toString(), new RowNumResultSetExtractor<TiposCanalAviso>(this.rwMapPK, jqGridRequestDto), filterParamList.toArray());
	}

	/**
	 * Remove multiple method for rup_table
     *
     * @param filtertiposcanalaviso TiposCanalAviso
     * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean
     */
	@Override
	public void removeMultiple(TiposCanalAviso filtertiposcanalaviso, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		// SELECT
		/** TODO: select por clave */
		StringBuilder query = new StringBuilder("SELECT t1.ID_TIPO_CANAL_AVISO IDTIPOCANALAVISO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
        query.append("FROM TIPOS_CANAL_AVISO t1 ");

		// WHERE
		Map<String, Object> mapaWhere = this.getWhereLikeMap(filtertiposcanalaviso, startsWith);
		StringBuilder where = new StringBuilder(" WHERE 1=1 ");
		where.append(mapaWhere.get("query"));
		query.append(where);

		@SuppressWarnings("unchecked")
		List<Object> params = (List<Object>) mapaWhere.get("params");

		StringBuilder sbRemoveMultipleSQL = JQGridManager.getRemoveMultipleQuery(jqGridRequestDto, TiposCanalAviso.class, query, params, "ID_TIPO_CANAL_AVISO");
		this.jdbcTemplate.update(sbRemoveMultipleSQL.toString(), params.toArray());
	}

	/**
	 * Filter in the hierarchical TiposCanalAviso table.
     *
     * @param filterTiposCanalAviso TiposCanalAviso
     * @param jqGridRequestDto JQGridRequestDto
     * @return List<JerarquiaDto<TiposCanalAviso>>
     */
	@Override
	public List<JerarquiaDto<TiposCanalAviso>> findAllLikeJerarquia(TiposCanalAviso filterTiposCanalAviso, JQGridRequestDto jqGridRequestDto) {
		// SELECT
		StringBuilder query = new StringBuilder("SELECT t1.ID_TIPO_CANAL_AVISO IDTIPOCANALAVISO,t1.DESCRIPCION_CASTELLANO DESCRIPCIONCASTELLANO,t1.DESCRIPCION_EUSKERA DESCRIPCIONEUSKERA,t1.FECHA_REGISTRO FECHAREGISTRO,t1.USUARIO_REGISTRO USUARIOREGISTRO ");
		// FROM
        //StringBuilder from = new StringBuilder("FROM TIPOS_CANAL_AVISO t1 ");
        List<String> from = new ArrayList<String>();
        from.add("TIPOS_CANAL_AVISO");
        //TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		//FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterTiposCanalAviso, false);

		//JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada por defecto (ID_PADRE)
		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID_TIPO_CANAL_AVISO", "ID_PADRE", "NOMBRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y condiciones de negocio
		 */
//		query = JQGridManagerJerarquia.getQuery(jqGridRequestDto, query, mapaWhere, "ID", "ID_PADRE", "NOMBRE", from, from_alias, joins, businessFilters, businessParams);

		//PAGINACION
		if (jqGridRequestDto != null) {
			query = JQGridManagerJerarquia.getPaginationQuery(jqGridRequestDto, query, TiposCanalAvisoDaoImpl.ORDER_BY_WHITE_LIST);
		}

		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.query(query.toString(), this.rwMapJerarquia, params.toArray());
	}

	/**
	 * Count rows in hierarchy
     *
     * @param filterTiposCanalAviso TiposCanalAviso
     * @param jqGridRequestDto JQGridRequestDto
     * @return Long
     */
	@Override
	public Long findAllLikeCountJerarquia(TiposCanalAviso filterTiposCanalAviso, JQGridRequestDto jqGridRequestDto) {
		// FROM
        //StringBuilder from = new StringBuilder("FROM TIPOS_CANAL_AVISO t1 ");
        List<String> from = new ArrayList<String>();
        from.add("TIPOS_CANAL_AVISO");

        //TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		//FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterTiposCanalAviso, false);

		//JERARQUIA
		// Especificar la columna padre correspondiente en lugar de la generada por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID_TIPO_CANAL_AVISO", "ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y condiciones de negocio
		 */
//		StringBuilder query = JQGridManagerJerarquia.getQueryCount(jqGridRequestDto, mapaWhere, "ID", "ID_PADRE", from, from_alias, joins, businessFilters, businessParams);


		List<?> params = (List<?>) mapaWhere.get("params");
		return this.jdbcTemplate.queryForLong(query.toString(), params.toArray());
	}

	/**
	 * Find dependent hierarchical rows
     *
     * @param filterTiposCanalAviso TiposCanalAviso
     * @param jqGridRequestDto JQGridRequestDto
     * @return List<TableRowDto<TiposCanalAviso>>
     */
	@Override
	public List<TableRowDto<TiposCanalAviso>> findAllChild(TiposCanalAviso filterTiposCanalAviso, JQGridRequestDto jqGridRequestDto) {

		// FROM
        //StringBuilder from = new StringBuilder("FROM TIPOS_CANAL_AVISO t1 ");
        List<String> from = new ArrayList<String>();
        from.add("TIPOS_CANAL_AVISO");

        //TABLAS_ALIAS
		List<String> from_alias = new ArrayList<String>();
		from_alias.add(" t1 ");

		// JOINS TABLAS
		/*
		 * Ejemplo de como indicar joins entre las tablas de la query
		 */
		/*
		 * StringBuilder joins = new StringBuilder("");
		 * joins.append("AND T1.ID=T2.ID_T1");
		 */

		// CONDICIONES (negocio)
		/*
		 * Ejemplo de como incluir condiciones de negocio en la consulta de la
		 * jerarquía
		 */
		/*
		 * StringBuilder businessFilters = new StringBuilder(); List<Object>
		 * businessParams = new ArrayList<Object>();
		 * businessFilters.append("   AND t1.EJIE = ?  ");
		 * businessParams.add("1");
		 */

		//FILTRO
		Map<String, ?> mapaWhere = this.getWhereLikeMap(filterTiposCanalAviso, false);

		//MULTISELECCION
		// Especificar la columna padre correspondiente en lugar de la generada por defecto (ID_PADRE)
		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID_TIPO_CANAL_AVISO", "ID_PADRE", from, from_alias);

		/*
		 * Ejemplo de llamada en caso de indicar joins entre tablas y condiciones de negocio
		 */
//		StringBuilder query = JQGridManagerJerarquia.getQueryChildren(jqGridRequestDto, mapaWhere, "ID", "ID_PADRE", from, from_alias, joins, businessFilters, businessParams);

		List<?> params = (List<?>) mapaWhere.get("params");

		return this.jdbcTemplate.query(query.toString(), new RowNumResultSetExtractor<TiposCanalAviso>(this.rwMapPK, "ID_TIPO_CANAL_AVISO"), params.toArray());
	}

	/*
	 * MÉTODOS PRIVADOS
	 */

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the TiposCanalAviso entity
	 *
	 * @param tiposcanalaviso TiposCanalAviso
	 *            Bean with the criteria values to filter by.
	 * @return Map created with two keys
	 *         key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, ?> getWhereMap (TiposCanalAviso tiposcanalaviso){

		StringBuilder where = new StringBuilder(TiposCanalAvisoDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (tiposcanalaviso  != null  && tiposcanalaviso.getIdTipoCanalAviso() != null ) {
			where.append(" AND t1.ID_TIPO_CANAL_AVISO = ?");
			params.add(tiposcanalaviso.getIdTipoCanalAviso());
		}
		if (tiposcanalaviso  != null  && tiposcanalaviso.getDescripcionCastellano() != null ) {
			where.append(" AND t1.DESCRIPCION_CASTELLANO = ?");
			params.add(tiposcanalaviso.getDescripcionCastellano());
		}
		if (tiposcanalaviso  != null  && tiposcanalaviso.getDescripcionEuskera() != null ) {
			where.append(" AND t1.DESCRIPCION_EUSKERA = ?");
			params.add(tiposcanalaviso.getDescripcionEuskera());
		}
		if (tiposcanalaviso  != null  && tiposcanalaviso.getFechaRegistro() != null ) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(tiposcanalaviso.getFechaRegistro());
		}
		if (tiposcanalaviso  != null  && tiposcanalaviso.getUsuarioRegistro() != null ) {
			where.append(" AND t1.USUARIO_REGISTRO = ?");
			params.add(tiposcanalaviso.getUsuarioRegistro());
		}

		Map<String,Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * Returns a map with the needed value to create the conditions to filter by
	 * the TiposCanalAviso entity
	 *
	 * @param tiposcanalaviso TiposCanalAviso
	 *            Bean with the criteria values to filter by.
     * @param startsWith Boolean
	 * @return Map created with two keys
	 *         key query stores the sql query syntax
	 *         key params stores the parameter values to be used in the condition sentence.
	 */
	// CHECKSTYLE:OFF CyclomaticComplexity - Generación de código de UDA
	private Map<String, Object> getWhereLikeMap (TiposCanalAviso tiposcanalaviso, Boolean startsWith){

		StringBuilder where = new StringBuilder(TiposCanalAvisoDaoImpl.STRING_BUILDER_INIT);
		List<Object> params = new ArrayList<Object>();

		if (tiposcanalaviso  != null  && tiposcanalaviso.getIdTipoCanalAviso() != null ) {
			where.append(" AND t1.ID_TIPO_CANAL_AVISO = ?");
			params.add(tiposcanalaviso.getIdTipoCanalAviso());
	     }
		if (tiposcanalaviso  != null  && tiposcanalaviso.getDescripcionCastellano() != null ) {
			where.append(" AND UPPER(t1.DESCRIPCION_CASTELLANO) like ? ESCAPE  '\\'");
			if (startsWith){
				params.add(tiposcanalaviso.getDescripcionCastellano().toUpperCase() +"%");
			}else{
				params.add("%"+tiposcanalaviso.getDescripcionCastellano().toUpperCase() +"%");
			}
			where.append(" AND t1.DESCRIPCION_CASTELLANO IS NOT NULL");
	     }
		if (tiposcanalaviso  != null  && tiposcanalaviso.getDescripcionEuskera() != null ) {
			where.append(" AND UPPER(t1.DESCRIPCION_EUSKERA) like ? ESCAPE  '\\'");
			if (startsWith){
				params.add(tiposcanalaviso.getDescripcionEuskera().toUpperCase() +"%");
			}else{
				params.add("%"+tiposcanalaviso.getDescripcionEuskera().toUpperCase() +"%");
			}
			where.append(" AND t1.DESCRIPCION_EUSKERA IS NOT NULL");
	     }
		if (tiposcanalaviso  != null  && tiposcanalaviso.getFechaRegistro() != null ) {
			where.append(" AND t1.FECHA_REGISTRO = ?");
			params.add(tiposcanalaviso.getFechaRegistro());
	     }
		if (tiposcanalaviso  != null  && tiposcanalaviso.getUsuarioRegistro() != null ) {
			where.append(" AND UPPER(t1.USUARIO_REGISTRO) like ? ESCAPE  '\\'");
			if (startsWith){
				params.add(tiposcanalaviso.getUsuarioRegistro().toUpperCase() +"%");
			}else{
				params.add("%"+tiposcanalaviso.getUsuarioRegistro().toUpperCase() +"%");
			}
			where.append(" AND t1.USUARIO_REGISTRO IS NOT NULL");
	     }

		Map<String,Object> mapWhere = new HashMap<String, Object>();
		mapWhere.put("query", where);
		mapWhere.put("params", params);

		return mapWhere;
	}
	// CHECKSTYLE:ON CyclomaticComplexity - Generación de código de UDA

	/**
	 * StringBuilder initilization value
	 */
	 public static final int STRING_BUILDER_INIT = 4096;}

