package com.ejie.ab61.service.configuracionEventos;

import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

import com.ejie.ab61.model.configuracionEventos.ConfigEventos;

/**
 * ConfigEventosService  
 * 
 *  
 */

public interface ConfigEventosService {

	/**
	 * Inserts a single row in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos add(ConfigEventos configEventos);

	/**
	 * Updates a single row in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos update(ConfigEventos configEventos);

	/**
	 * Finds a single row in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos find(ConfigEventos configEventos);

	/**
	 * Encuentra la configuracion de un evento determinado
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos findConfiguracion(ConfigEventos configeventos);

	/**
	 * Encuentra la configuracion de un evento determinado por tipo evento,
	 * platea_type y platea_subtype e instancias., Para aportaciones de
	 * documentacion en N-Instancias
	 *
	 * @param configeventos
	 *            ConfigEventos
	 * @return ConfigEventos
	 */
	ConfigEventos findConfiguracionPorInstancia(ConfigEventos configeventos);

	/**
	 * Deletes a single row in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 */
	void remove(ConfigEventos configEventos);

	/**
	 * Finds a list of rows in the ConfigEventos table.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<ConfigEventos>
	 */
	List<ConfigEventos> findAll(ConfigEventos configEventos, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the ConfigEventos table using like.
	 *
	 * @param configEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<ConfigEventos>
	 */
	List<ConfigEventos> findAllLike(ConfigEventos configEventos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Filter method in the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<ConfigEventos>
	 */
	JQGridResponseDto<ConfigEventos> filter(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches rows in the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param searchConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<ConfigEventos>>
	 */
	List<TableRowDto<ConfigEventos>> search(ConfigEventos filterConfigEventos, ConfigEventos searchConfigEventos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Reorder the selection made in ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	Object reorderSelection(ConfigEventos filterConfigEventos, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the ConfigEventos table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<ConfigEventos>>
	 */
	JQGridResponseDto<JerarquiaDto<ConfigEventos>> jerarquia(ConfigEventos filterConfigEventos,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Finder method for siblings rows in the hierarchical the ConfigEventos
	 * table.
	 *
	 * @param filterConfigEventos
	 *            ConfigEventos
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<ConfigEventos>>
	 */
	JQGridResponseDto<JerarquiaDto<ConfigEventos>> jerarquiaChildren(ConfigEventos filterConfigEventos,
			JQGridRequestDto jqGridRequestDto);

}
