package com.ejie.ab61.service.plantillas;

import java.util.List;

import com.ejie.ab61.model.plantillas.TipoCampo;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * TipoCampoService  
 * 
 *  
 */

public interface TipoCampoService {

	/**
	 * Inserts a single row in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	TipoCampo add(TipoCampo tipoCampo);

	/**
	 * Updates a single row in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	TipoCampo update(TipoCampo tipoCampo);

	/**
	 * Finds a single row in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @return TipoCampo
	 */
	TipoCampo find(TipoCampo tipoCampo);

	/**
	 * Deletes a single row in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 */
	void remove(TipoCampo tipoCampo);

	/**
	 * Finds a list of rows in the TipoCampo table.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<TipoCampo>
	 */
	List<TipoCampo> findAll(TipoCampo tipoCampo, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the TipoCampo table using like.
	 *
	 * @param tipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TipoCampo>
	 */
	List<TipoCampo> findAllLike(TipoCampo tipoCampo, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	void removeMultiple(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Filter method in the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<TipoCampo>
	 */
	JQGridResponseDto<TipoCampo> filter(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Searches rows in the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param searchTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<TipoCampo>>
	 */
	List<TableRowDto<TipoCampo>> search(TipoCampo filterTipoCampo, TipoCampo searchTipoCampo,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/**
	 * Reorder the selection made in TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	Object reorderSelection(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<TipoCampo>>
	 */
	JQGridResponseDto<JerarquiaDto<TipoCampo>> jerarquia(TipoCampo filterTipoCampo, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith);

	/**
	 * Finder method for siblings rows in the hierarchical the TipoCampo table.
	 *
	 * @param filterTipoCampo
	 *            TipoCampo
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<TipoCampo>>
	 */
	JQGridResponseDto<JerarquiaDto<TipoCampo>> jerarquiaChildren(TipoCampo filterTipoCampo,
			JQGridRequestDto jqGridRequestDto);

}
