package com.ejie.ab61.service.procedimientos;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.dao.procedimientos.AplicacionesDao;
import com.ejie.ab61.model.procedimientos.Aplicaciones;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * AplicacionesServiceImpl  
 * 
 *  
 */

@Service(value = "aplicacionesService")
public class AplicacionesServiceImpl implements AplicacionesService {

	@Autowired
	private AplicacionesDao aplicacionesDao;

	/**
	 * Inserts a single row in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @return Aplicaciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Aplicaciones add(Aplicaciones aplicaciones) {
		return this.aplicacionesDao.add(aplicaciones);
	}

	/**
	 * Updates a single row in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @return Aplicaciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Aplicaciones update(Aplicaciones aplicaciones) {
		return this.aplicacionesDao.update(aplicaciones);
	}

	/**
	 * Finds a single row in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @return Aplicaciones
	 */
	public Aplicaciones find(Aplicaciones aplicaciones) {
		return (Aplicaciones) this.aplicacionesDao.find(aplicaciones);
	}

	/**
	 * Deletes a single row in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(Aplicaciones aplicaciones) {
		this.aplicacionesDao.remove(aplicaciones);
	}

	/**
	 * Finds a list of rows in the Aplicaciones table.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Aplicaciones>
	 */
	public List<Aplicaciones> findAll(Aplicaciones aplicaciones, JQGridRequestDto jqGridRequestDto) {
		return (List<Aplicaciones>) this.aplicacionesDao.findAll(aplicaciones, jqGridRequestDto);
	}

	/**
	 * Finds rows in the Aplicaciones table using like.
	 *
	 * @param aplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Aplicaciones>
	 */
	public List<Aplicaciones> findAllLike(Aplicaciones aplicaciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return (List<Aplicaciones>) this.aplicacionesDao.findAllLike(aplicaciones, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the Aplicaciones table.
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	public void removeMultiple(Aplicaciones filterAplicaciones, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		this.aplicacionesDao.removeMultiple(filterAplicaciones, jqGridRequestDto, startsWith);
	}

	/**
	 * Filter method in the Aplicaciones table.
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<Aplicaciones>
	 */
	public JQGridResponseDto<Aplicaciones> filter(Aplicaciones filterAplicaciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		List<Aplicaciones> listaAplicaciones = this.aplicacionesDao.findAllLike(filterAplicaciones, jqGridRequestDto,
				false);
		Long recordNum = this.aplicacionesDao
				.findAllLikeCount(filterAplicaciones != null ? filterAplicaciones : new Aplicaciones(), false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<Aplicaciones>> reorderSelection = this.aplicacionesDao.reorderSelection(filterAplicaciones,
					jqGridRequestDto, startsWith);
			return new JQGridResponseDto<Aplicaciones>(jqGridRequestDto, recordNum, listaAplicaciones,
					reorderSelection);
		}
		return new JQGridResponseDto<Aplicaciones>(jqGridRequestDto, recordNum, listaAplicaciones);
	}

	/**
	 * Searches rows in the Aplicaciones table.
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param searchAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Aplicaciones>>
	 */
	public List<TableRowDto<Aplicaciones>> search(Aplicaciones filterAplicaciones, Aplicaciones searchAplicaciones,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.aplicacionesDao.search(filterAplicaciones, searchAplicaciones, jqGridRequestDto, startsWith);
	}

	/**
	 * Reorder the selection made in Aplicaciones table.
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	public Object reorderSelection(Aplicaciones filterAplicaciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return this.aplicacionesDao.reorderSelection(filterAplicaciones, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the Aplicaciones table.
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<Aplicaciones>>
	 */
	public JQGridResponseDto<JerarquiaDto<Aplicaciones>> jerarquia(Aplicaciones filterAplicaciones,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<JerarquiaDto<Aplicaciones>> listaAplicaciones = this.aplicacionesDao
				.findAllLikeJerarquia(filterAplicaciones, jqGridRequestDto);
		Long recordNum = this.aplicacionesDao.findAllLikeCountJerarquia(filterAplicaciones, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<Aplicaciones>>(jqGridRequestDto, recordNum, listaAplicaciones);
	}

	/**
	 * Finder method for siblings rows in the hierarchical the Aplicaciones
	 * table.
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<Aplicaciones>>
	 */
	public JQGridResponseDto<JerarquiaDto<Aplicaciones>> jerarquiaChildren(Aplicaciones filterAplicaciones,
			JQGridRequestDto jqGridRequestDto) {
		JQGridResponseDto<JerarquiaDto<Aplicaciones>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<Aplicaciones>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN,
				this.aplicacionesDao.findAllChild(filterAplicaciones, jqGridRequestDto));
		return jqGridResponseDto;
	}

}
