package com.ejie.ab61.service.solicitudes;

import com.ejie.ab61.dao.solicitudes.ConfiguracionDao;
import com.ejie.ab61.model.procedimientos.Procedimientos;
import com.ejie.ab61.model.solicitudes.Configuracion;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * ConfiguracionServiceImpl  
 * 
 *  
 */

@Service(value = "configuracionService")
public class ConfiguracionServiceImpl implements ConfiguracionService {

	@Autowired
	private ConfiguracionDao configuracionDao;

	/**
	 * Inserts a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Configuracion add(Configuracion configuracion) {
		return this.configuracionDao.add(configuracion);
	}

	/**
	 * Updates a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	@Transactional(rollbackFor = Throwable.class)
	public Configuracion update(Configuracion configuracion) {
		return this.configuracionDao.update(configuracion);
	}

	/**
	 * Finds a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @return Configuracion
	 */
	public Configuracion find(Configuracion configuracion) {
		return (Configuracion) this.configuracionDao.find(configuracion);
	}

	/**
	 * Deletes a single row in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(Configuracion configuracion) {
		this.configuracionDao.remove(configuracion);
	}

	/**
	 * Finds a list of rows in the Configuracion table.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<Configuracion>
	 */
	public List<Configuracion> findAll(Configuracion configuracion, JQGridRequestDto jqGridRequestDto) {
		return (List<Configuracion>) this.configuracionDao.findAll(configuracion, jqGridRequestDto);
	}

	/**
	 * Finds rows in the Configuracion table using like.
	 *
	 * @param configuracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<Configuracion>
	 */
	public List<Configuracion> findAllLike(Configuracion configuracion, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return (List<Configuracion>) this.configuracionDao.findAllLike(configuracion, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the Configuracion table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	public void removeMultiple(Configuracion filterConfiguracion, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		this.configuracionDao.removeMultiple(filterConfiguracion, jqGridRequestDto, startsWith);
	}

	/**
	 * Filter method in the Configuracion table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<Configuracion>
	 */
	public JQGridResponseDto<Configuracion> filter(Configuracion filterConfiguracion, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		List<Configuracion> listaConfiguracion = this.configuracionDao.findAllLike(filterConfiguracion,
				jqGridRequestDto, false);
		Long recordNum = this.configuracionDao
				.findAllLikeCount(filterConfiguracion != null ? filterConfiguracion : new Configuracion(), false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<Configuracion>> reorderSelection = this.configuracionDao
					.reorderSelection(filterConfiguracion, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<Configuracion>(jqGridRequestDto, recordNum, listaConfiguracion,
					reorderSelection);
		}
		return new JQGridResponseDto<Configuracion>(jqGridRequestDto, recordNum, listaConfiguracion);
	}

	/**
	 * Searches rows in the Configuracion table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param searchConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<Configuracion>>
	 */
	public List<TableRowDto<Configuracion>> search(Configuracion filterConfiguracion, Configuracion searchConfiguracion,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.configuracionDao.search(filterConfiguracion, searchConfiguracion, jqGridRequestDto, startsWith);
	}

	/**
	 * Reorder the selection made in Configuracion table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	public Object reorderSelection(Configuracion filterConfiguracion, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return this.configuracionDao.reorderSelection(filterConfiguracion, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the Configuracion table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<Configuracion>>
	 */
	public JQGridResponseDto<JerarquiaDto<Configuracion>> jerarquia(Configuracion filterConfiguracion,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<JerarquiaDto<Configuracion>> listaConfiguracion = this.configuracionDao
				.findAllLikeJerarquia(filterConfiguracion, jqGridRequestDto);
		Long recordNum = this.configuracionDao.findAllLikeCountJerarquia(filterConfiguracion, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<Configuracion>>(jqGridRequestDto, recordNum, listaConfiguracion);
	}

	/**
	 * Finder method for siblings rows in the hierarchical the Configuracion
	 * table.
	 *
	 * @param filterConfiguracion
	 *            Configuracion
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<Configuracion>>
	 */
	public JQGridResponseDto<JerarquiaDto<Configuracion>> jerarquiaChildren(Configuracion filterConfiguracion,
			JQGridRequestDto jqGridRequestDto) {
		JQGridResponseDto<JerarquiaDto<Configuracion>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<Configuracion>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN,
				this.configuracionDao.findAllChild(filterConfiguracion, jqGridRequestDto));
		return jqGridResponseDto;
	}

	@Override
	public Integer comprobarConfigEventosRepetido(String codProcedureId, Integer idConfigEventos) {
		return this.configuracionDao.comprobarConfigEventosRepetido(codProcedureId, idConfigEventos);
	}

	@Override
	public void removeByCodProcedureId(String codProcedureId) {
		this.configuracionDao.removeByCodProcedureId(codProcedureId);
	}

	@Override
	@Transactional(rollbackFor = Throwable.class)
	public List<Configuracion> findByCodProcedureId(String codProcedureId) {
		return this.configuracionDao.findByCodProcedureId(codProcedureId);
	}

}
