package com.ejie.ab61.service.solicitudes;


import com.ejie.ab61.model.solicitudes.Solicitudes;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;

/**
 * SolicitudesService  
 *  
 */

public interface SolicitudesService {

	/**
	 * Inserts a single row in the Solicitudes table.
	 *
	 * @param solicitudes Solicitudes
	 * @return Solicitudes
	 */
    Solicitudes add(Solicitudes solicitudes);

	/**
	 * Updates a single row in the Solicitudes table.
	 *
	 * @param solicitudes Solicitudes
	 * @return Solicitudes
	 */
	Solicitudes update(Solicitudes solicitudes);

	/**
	 * Finds a single row in the Solicitudes table.
	 *
	 * @param solicitudes Solicitudes
	 * @return Solicitudes
	 */
	Solicitudes find(Solicitudes solicitudes);

	/**
	 * Deletes a single row in the Solicitudes table.
	 *
	 * @param solicitudes Solicitudes
	 */
	void remove(Solicitudes solicitudes);

	/**
	 * Finds a list of rows in the Solicitudes table.
	 *
	 * @param solicitudes Solicitudes
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return List<Solicitudes>
	 */
	List<Solicitudes> findAll(Solicitudes solicitudes, JQGridRequestDto jqGridRequestDto);

	/**
	 * Finds rows in the Solicitudes table using like.
	 *
	 * @param solicitudes Solicitudes
	 * @param jqGridRequestDto JQGridRequestDto
     * @param startsWith Boolean	 
	 * @return List<Solicitudes>
	 */
	List<Solicitudes> findAllLike(Solicitudes solicitudes, JQGridRequestDto jqGridRequestDto, Boolean startsWith);

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the Solicitudes table.
	 *
	 * @param filterSolicitudes Solicitudes
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 */	
	void removeMultiple(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto,  Boolean startsWith);
   
    
	/**
	 * Filter method in the Solicitudes table.
	 *
	 * @param filterSolicitudes Solicitudes
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<Solicitudes>
	 */	
	JQGridResponseDto<Solicitudes> filter(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto,  Boolean startsWith);
    
    /**
	 * Searches rows in the Solicitudes table.
	 *
	 * @param filterSolicitudes Solicitudes
	 * @param searchSolicitudes Solicitudes
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return List<TableRowDto<Solicitudes>>
	 */	
    List<TableRowDto<Solicitudes>> search(Solicitudes filterSolicitudes, Solicitudes searchSolicitudes, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
    
    /**
	 * Reorder the selection made in Solicitudes table.
	 *
	 * @param filterSolicitudes Solicitudes
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return Object
	 */	
    Object reorderSelection(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
    
    /*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */
	 
	/**
	 * Finder method in the hierarchical the Solicitudes table.
	 *
	 * @param filterSolicitudes Solicitudes
	 * @param jqGridRequestDto JQGridRequestDto
	 * @param startsWith Boolean
	 * @return JQGridResponseDto<JerarquiaDto<Solicitudes>>
	 */	
	JQGridResponseDto<JerarquiaDto<Solicitudes>> jerarquia(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto, Boolean startsWith);
	
	/**
	 * Finder method for siblings rows in the hierarchical the Solicitudes table.
	 *
	 * @param filterSolicitudes Solicitudes
	 * @param jqGridRequestDto JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<Solicitudes>>
	 */	
	JQGridResponseDto<JerarquiaDto<Solicitudes>> jerarquiaChildren(Solicitudes filterSolicitudes, JQGridRequestDto jqGridRequestDto);
    
}


