package com.ejie.ab61.service.terceros;

import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.dao.terceros.DatosDireccionesDao;
import com.ejie.ab61.model.expedientes.Expedientes;
import com.ejie.ab61.model.terceros.DatosDirecciones;

/**
 * DatosDireccionesServiceImpl  
 * 
 *  
 */

@Service(value = "datosDireccionesService")
public class DatosDireccionesServiceImpl implements DatosDireccionesService {

	@Autowired
	private DatosDireccionesDao datosDireccionesDao;

	/**
	 * Inserts a single row in the DatosDirecciones table.
	 *
	 * @param datosDirecciones
	 *            DatosDirecciones
	 * @return DatosDirecciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public DatosDirecciones add(DatosDirecciones datosDirecciones) {
		return this.datosDireccionesDao.add(datosDirecciones);
	}

	/**
	 * Updates a single row in the DatosDirecciones table.
	 *
	 * @param datosDirecciones
	 *            DatosDirecciones
	 * @return DatosDirecciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public DatosDirecciones update(DatosDirecciones datosDirecciones) {
		return this.datosDireccionesDao.update(datosDirecciones);
	}

	/**
	 * Finds a single row in the DatosDirecciones table.
	 *
	 * @param datosDirecciones
	 *            DatosDirecciones
	 * @return DatosDirecciones
	 */
	public DatosDirecciones find(DatosDirecciones datosDirecciones) {
		return (DatosDirecciones) this.datosDireccionesDao.find(datosDirecciones);
	}

	/**
	 * Deletes a single row in the DatosDirecciones table.
	 *
	 * @param datosDirecciones
	 *            DatosDirecciones
	 */
	@Transactional(rollbackFor = Throwable.class)
	public void remove(DatosDirecciones datosDirecciones) {
		this.datosDireccionesDao.remove(datosDirecciones);
	}

	/**
	 * Finds a list of rows in the DatosDirecciones table.
	 *
	 * @param datosDirecciones
	 *            DatosDirecciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<DatosDirecciones>
	 */
	public List<DatosDirecciones> findAll(DatosDirecciones datosDirecciones, JQGridRequestDto jqGridRequestDto) {
		return (List<DatosDirecciones>) this.datosDireccionesDao.findAll(datosDirecciones, jqGridRequestDto);
	}

	/**
	 * Finds rows in the DatosDirecciones table using like.
	 *
	 * @param datosDirecciones
	 *            DatosDirecciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<DatosDirecciones>
	 */
	public List<DatosDirecciones> findAllLike(DatosDirecciones datosDirecciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return (List<DatosDirecciones>) this.datosDireccionesDao.findAllLike(datosDirecciones, jqGridRequestDto,
				startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the DatosDirecciones table.
	 *
	 * @param filterDatosDirecciones
	 *            DatosDirecciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	public void removeMultiple(DatosDirecciones filterDatosDirecciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		this.datosDireccionesDao.removeMultiple(filterDatosDirecciones, jqGridRequestDto, startsWith);
	}

	/**
	 * Filter method in the DatosDirecciones table.
	 *
	 * @param filterDatosDirecciones
	 *            DatosDirecciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<DatosDirecciones>
	 */
	public JQGridResponseDto<DatosDirecciones> filter(DatosDirecciones filterDatosDirecciones,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<DatosDirecciones> listaDatosDirecciones = this.datosDireccionesDao.findAllLike(filterDatosDirecciones,
				jqGridRequestDto, false);
		Long recordNum = this.datosDireccionesDao.findAllLikeCount(
				filterDatosDirecciones != null ? filterDatosDirecciones : new DatosDirecciones(), false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<DatosDirecciones>> reorderSelection = this.datosDireccionesDao
					.reorderSelection(filterDatosDirecciones, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<DatosDirecciones>(jqGridRequestDto, recordNum, listaDatosDirecciones,
					reorderSelection);
		}
		return new JQGridResponseDto<DatosDirecciones>(jqGridRequestDto, recordNum, listaDatosDirecciones);
	}

	/**
	 * Searches rows in the DatosDirecciones table.
	 *
	 * @param filterDatosDirecciones
	 *            DatosDirecciones
	 * @param searchDatosDirecciones
	 *            DatosDirecciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<DatosDirecciones>>
	 */
	public List<TableRowDto<DatosDirecciones>> search(DatosDirecciones filterDatosDirecciones,
			DatosDirecciones searchDatosDirecciones, JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.datosDireccionesDao.search(filterDatosDirecciones, searchDatosDirecciones, jqGridRequestDto,
				startsWith);
	}

	/**
	 * Reorder the selection made in DatosDirecciones table.
	 *
	 * @param filterDatosDirecciones
	 *            DatosDirecciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	public Object reorderSelection(DatosDirecciones filterDatosDirecciones, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return this.datosDireccionesDao.reorderSelection(filterDatosDirecciones, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the DatosDirecciones table.
	 *
	 * @param filterDatosDirecciones
	 *            DatosDirecciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<DatosDirecciones>>
	 */
	public JQGridResponseDto<JerarquiaDto<DatosDirecciones>> jerarquia(DatosDirecciones filterDatosDirecciones,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<JerarquiaDto<DatosDirecciones>> listaDatosDirecciones = this.datosDireccionesDao
				.findAllLikeJerarquia(filterDatosDirecciones, jqGridRequestDto);
		Long recordNum = this.datosDireccionesDao.findAllLikeCountJerarquia(filterDatosDirecciones, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<DatosDirecciones>>(jqGridRequestDto, recordNum,
				listaDatosDirecciones);
	}

	/**
	 * Finder method for siblings rows in the hierarchical the DatosDirecciones
	 * table.
	 *
	 * @param filterDatosDirecciones
	 *            DatosDirecciones
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<DatosDirecciones>>
	 */
	public JQGridResponseDto<JerarquiaDto<DatosDirecciones>> jerarquiaChildren(DatosDirecciones filterDatosDirecciones,
			JQGridRequestDto jqGridRequestDto) {
		JQGridResponseDto<JerarquiaDto<DatosDirecciones>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<DatosDirecciones>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN,
				this.datosDireccionesDao.findAllChild(filterDatosDirecciones, jqGridRequestDto));
		return jqGridResponseDto;
	}

	@Override
	public DatosDirecciones findDireccionByExpediente(Expedientes expediente) {
		return this.datosDireccionesDao.findDireccionByExpediente(expediente);
	}

}
