package com.ejie.ab61.service.terceros;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.ejie.ab61.dao.expedientes.RelExpTerDao;
import com.ejie.ab61.dao.terceros.DatosTercerosDao;
import com.ejie.ab61.model.expedientes.RelExpTer;
import com.ejie.ab61.model.terceros.DatosTerceros;
import com.ejie.ab61.model.terceros.DatosTerceros;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * DatosTercerosServiceImpl  
 * 
 *  
 */

@Service(value = "datosTercerosService")
public class DatosTercerosServiceImpl implements DatosTercerosService {

	@Autowired
	private DatosTercerosDao datosTercerosDao;
	@Autowired
	private RelExpTerDao relExpTerDao;

	/**
	 * Inserts a single row in the DatosTerceros table.
	 *
	 * @param datosTerceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	@Transactional(rollbackFor = Throwable.class)
	public DatosTerceros add(DatosTerceros datosTerceros) {
		return this.datosTercerosDao.add(datosTerceros);
	}

	/**
	 * Finds el ultimo id de la tabla in the DatosTerceros table.
	 *
	 * 
	 * @return datosTerceros Datosterceros
	 */
	public DatosTerceros findUltimoId() {
		return this.datosTercerosDao.findUltimoId();
	}

	/**
	 * Updates a single row in the DatosTerceros table.
	 *
	 * @param datosTerceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */
	@Transactional(rollbackFor = Throwable.class)
	public DatosTerceros update(DatosTerceros datosTerceros) {
		if (0 == datosTerceros.getIdSexo()) {
			datosTerceros.setIdSexo(null);
		}
		return this.datosTercerosDao.update(datosTerceros);
	}

	/**
	 * Finds a single row in the DatosTerceros table.
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */

	public DatosTerceros findTercero(Integer idTercero, String partipantId) {
		return this.datosTercerosDao.findTercero(idTercero, partipantId);
	}

	/**
	 * Encuentra los datos de tercero tipo : persona de contacto o responsable
	 * legal
	 *
	 * @param datosterceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */

	public DatosTerceros findPersonacontactoResponsableLegal(Integer idTercero, Long tipo, Integer idExpediente) {
		return this.datosTercerosDao.findPersonacontactoResponsableLegal(idTercero, tipo, idExpediente);
	}

	/**
	 * Deletes a single row in the DatosTercero.
	 *
	 * @param datosTerceros
	 *            DatosTerceros
	 * @param idExpediente
	 *            Integer
	 **/
	public void removeIdTercero(DatosTerceros datosTerceros) {

		this.relExpTerDao.removeidTercero(datosTerceros.getIdTercero());
		this.datosTercerosDao.remove(datosTerceros.getIdTercero());

	}

	/**
	 * Finds a single row in the DatosDirecciones table.
	 *
	 * @param datosTerceros
	 *            DatosTerceros
	 * @return DatosTerceros
	 */

	public DatosTerceros find(DatosTerceros datosTerceros) {
		return (DatosTerceros) this.datosTercerosDao.find(datosTerceros);
	}

	/**
	 * Finds a list of rows in the DatosTerceros table.
	 *
	 * @param datosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return List<DatosTerceros>
	 */
	public List<DatosTerceros> findAll(DatosTerceros datosTerceros, JQGridRequestDto jqGridRequestDto) {
		return (List<DatosTerceros>) this.datosTercerosDao.findAll(datosTerceros, jqGridRequestDto);
	}

	/**
	 * Finds rows in the DatosTerceros table using like.
	 *
	 * @param datosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<DatosTerceros>
	 */
	public List<DatosTerceros> findAllLike(DatosTerceros datosTerceros, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return (List<DatosTerceros>) this.datosTercerosDao.findAllLike(datosTerceros, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE
	 */

	/**
	 * Removes rows from the DatosTerceros table.
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 */
	public void removeMultiple(DatosTerceros filterDatosTerceros, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		this.datosTercerosDao.removeMultiple(filterDatosTerceros, jqGridRequestDto, startsWith);
	}

	/**
	 * Filter method in the DatosTerceros table.
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<DatosTerceros>
	 */
	public JQGridResponseDto<DatosTerceros> filter(DatosTerceros filterDatosTerceros, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		List<DatosTerceros> listaDatosTerceros = this.datosTercerosDao.findAllLike(filterDatosTerceros,
				jqGridRequestDto, false);
		Long recordNum = this.datosTercerosDao
				.findAllLikeCount(filterDatosTerceros != null ? filterDatosTerceros : new DatosTerceros(), false);
		if (jqGridRequestDto.getMultiselection().getSelectedIds() != null) {
			List<TableRowDto<DatosTerceros>> reorderSelection = this.datosTercerosDao
					.reorderSelection(filterDatosTerceros, jqGridRequestDto, startsWith);
			return new JQGridResponseDto<DatosTerceros>(jqGridRequestDto, recordNum, listaDatosTerceros,
					reorderSelection);
		}
		return new JQGridResponseDto<DatosTerceros>(jqGridRequestDto, recordNum, listaDatosTerceros);
	}

	/**
	 * Searches rows in the DatosTerceros table.
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param searchDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return List<TableRowDto<DatosTerceros>>
	 */
	public List<TableRowDto<DatosTerceros>> search(DatosTerceros filterDatosTerceros, DatosTerceros searchDatosTerceros,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		return this.datosTercerosDao.search(filterDatosTerceros, searchDatosTerceros, jqGridRequestDto, startsWith);
	}

	/**
	 * Reorder the selection made in DatosTerceros table.
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return Object
	 */
	public Object reorderSelection(DatosTerceros filterDatosTerceros, JQGridRequestDto jqGridRequestDto,
			Boolean startsWith) {
		return this.datosTercerosDao.reorderSelection(filterDatosTerceros, jqGridRequestDto, startsWith);
	}

	/*
	 * OPERACIONES RUP_TABLE JERARQUIA
	 */

	/**
	 * Finder method in the hierarchical the DatosTerceros table.
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @param startsWith
	 *            Boolean
	 * @return JQGridResponseDto<JerarquiaDto<DatosTerceros>>
	 */
	public JQGridResponseDto<JerarquiaDto<DatosTerceros>> jerarquia(DatosTerceros filterDatosTerceros,
			JQGridRequestDto jqGridRequestDto, Boolean startsWith) {
		List<JerarquiaDto<DatosTerceros>> listaDatosTerceros = this.datosTercerosDao
				.findAllLikeJerarquia(filterDatosTerceros, jqGridRequestDto);
		Long recordNum = this.datosTercerosDao.findAllLikeCountJerarquia(filterDatosTerceros, jqGridRequestDto);
		return new JQGridResponseDto<JerarquiaDto<DatosTerceros>>(jqGridRequestDto, recordNum, listaDatosTerceros);
	}

	/**
	 * Finder method for siblings rows in the hierarchical the DatosTerceros
	 * table.
	 *
	 * @param filterDatosTerceros
	 *            DatosTerceros
	 * @param jqGridRequestDto
	 *            JQGridRequestDto
	 * @return JQGridResponseDto<JerarquiaDto<DatosTerceros>>
	 */
	public JQGridResponseDto<JerarquiaDto<DatosTerceros>> jerarquiaChildren(DatosTerceros filterDatosTerceros,
			JQGridRequestDto jqGridRequestDto) {
		JQGridResponseDto<JerarquiaDto<DatosTerceros>> jqGridResponseDto = new JQGridResponseDto<JerarquiaDto<DatosTerceros>>();
		jqGridResponseDto.addAdditionalParam(JQGridResponseDto.CHILDREN,
				this.datosTercerosDao.findAllChild(filterDatosTerceros, jqGridRequestDto));
		return jqGridResponseDto;
	}

	@Override
	public DatosTerceros addTerceroConRelacion(DatosTerceros datosTerceros, RelExpTer relExpTer) {
		this.datosTercerosDao.add(datosTerceros);

		DatosTerceros datosTercerosAUxId = datosTercerosDao.findUltimoId();
		relExpTer.setDatosTerceros(datosTercerosAUxId);

		this.relExpTerDao.add(relExpTer);
		return datosTerceros;
	}

	@Override
	public List<DatosTerceros> findTitularExpediente(Integer idExpediente) {
		return this.datosTercerosDao.findTitularExpediente(idExpediente);
	}

}
