package com.ejie.ab61.control;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.ejie.ab61.model.expedientes.RelExpPla;
import com.ejie.ab61.service.expedientes.RelExpPlaService;

/**
 * RelExpPlaController  
 * 
 *  
 */

@Controller
@RequestMapping(value = "/relexppla")
@SessionAttributes("usuario")
public class RelExpPlaController {

	private static final Logger logger = LoggerFactory.getLogger(RelExpPlaController.class);

	@Autowired
	private RelExpPlaService relExpPlaService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterRelExpPla
	 *            RelExpPla Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<RelExpPla> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<RelExpPla> getAll(@ModelAttribute RelExpPla filterRelExpPla) {
		RelExpPlaController.logger.info("[GET - find_ALL] : Obtener RelExpPla por filtro");
		return this.relExpPlaService.findAll(filterRelExpPla, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param relExpPla
	 *            RelExpPla Bean que contiene la informacion a modificar.
	 * @return RelExpPla Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody RelExpPla edit(@RequestBody RelExpPla relExpPla) {
		RelExpPla relExpPlaAux = this.relExpPlaService.update(relExpPla);
		RelExpPlaController.logger.info("[PUT] : RelExpPla actualizado correctamente");
		return relExpPlaAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param relExpPla
	 *            RelExpPla Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return RelExpPla Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody RelExpPla add(@RequestBody RelExpPla relExpPla) {
		RelExpPla relExpPlaAux = this.relExpPlaService.add(relExpPla);
		RelExpPlaController.logger.info("[POST] : RelExpPla insertado correctamente");
		return relExpPlaAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idRelExpPla
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return RelExpPla Bean eliminado.
	 */
	@RequestMapping(value = "/{idRelExpPla}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody RelExpPla remove(@PathVariable Integer idRelExpPla) {
		RelExpPla relExpPla = new RelExpPla();
		relExpPla.setIdRelExpPla(idRelExpPla);
		this.relExpPlaService.remove(relExpPla);
		RelExpPlaController.logger.info("[DELETE] : RelExpPla borrado correctamente");
		return relExpPla;
	}

}
