package com.ejie.ab61.control.expedientes;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.model.RelExpInsSinExp;
import com.ejie.ab61.model.VTInsSinExpediente;
import com.ejie.ab61.model.expedientes.Expedientes;
import com.ejie.ab61.model.terceros.DatosTerceros;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.RelExpInsSinExpService;
import com.ejie.ab61.service.VTInsSinExpedienteService;
import com.ejie.ab61.service.expedientes.ExpedientesService;
import com.ejie.ab61.service.terceros.DatosDireccionesService;
import com.ejie.ab61.service.terceros.DatosTercerosService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * ExpedientesController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/expedientesRelacionados")

public class ExpedientesRelacionadosController {

	private static final Logger logger = LoggerFactory.getLogger(ExpedientesRelacionadosController.class);

	@Autowired
	private ExpedientesService expedientesService;

	@Autowired
	private DatosTercerosService datosTercerosService;

	@Autowired
	private DatosDireccionesService datosDireccionesService;

	@Autowired
	private VTInsSinExpedienteService vTInsSinExpedienteService;

	@Autowired
	private RelExpInsSinExpService relExpInsSinExpService;

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint/{idExpediente}", method = RequestMethod.GET)
	public String getFormEdit(Model model, @PathVariable Integer idExpediente) {
		ExpedientesRelacionadosController.logger.info("[GET - View] : expedientesRelacionados " + idExpediente);

		Authentication aut = SecurityContextHolder.getContext().getAuthentication();
		ExpedientesRelacionadosController.logger.info("autotenticaciÃ³n:getAuthentication" + aut);

		/** Dejamos el usuario logueado en session */
		Usuario usuario = new Usuario();
		String username = "";
		if (null != aut) {
			username = (String) aut.getPrincipal();
		}

		usuario.setNombreUsuario(username);
		model.addAttribute("usuario", usuario);

		Expedientes expediente = new Expedientes();
		expediente.setIdExpediente(idExpediente);
		expediente = this.expedientesService.find(expediente);
		expediente.setDatosDirecciones(this.datosDireccionesService.findDireccionByExpediente(expediente));
		String nombreExpediente = expediente.getFolderNumber() + " - " + expediente.getTitulo();

		VTInsSinExpediente vTInsSinExpediente = new VTInsSinExpediente();
		if (expediente.getDatosDirecciones() != null) {
			vTInsSinExpediente.setIdProvincia(expediente.getDatosDirecciones().getIdProvincia());
			vTInsSinExpediente.setIdMunicipio(expediente.getDatosDirecciones().getIdMunicipio());
			vTInsSinExpediente.setIdLocalidad(expediente.getDatosDirecciones().getIdLocalidad());
		}

		model.addAttribute("vTInsSinExpediente", vTInsSinExpediente);
		model.addAttribute("nombreExpediente", nombreExpediente);
		model.addAttribute("expedientes", expediente);
		model.addAttribute("origen", Constantes.origen.EXPEDIENTES_RELACIONADOS);

		return "expedientesRelacionados";
	}

	@RequestMapping(value = "/tablaExpedientesARelacionar/{idExpediente}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Expedientes> filterExpedientesARelacionar(
			@RequestJsonBody(param = "filter") Expedientes filterExpedientes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto, @PathVariable Integer idExpediente) {
		ExpedientesRelacionadosController.logger
				.info("[POST - filter] : Obtener expedientes a relacionar (Por ubicación)");

		List<DatosTerceros> listaTitulares = this.datosTercerosService.findTitularExpediente(idExpediente);
		String documentosTitular = "";

		if (!listaTitulares.isEmpty()) {

			for (DatosTerceros titular : listaTitulares) {
				if (titular.getDocumento() != null) {
					documentosTitular += "\'" + titular.getDocumento() + "\', ";
				}
			}

			if (documentosTitular.length() > 0) {
				documentosTitular = documentosTitular.substring(0, documentosTitular.length() - 2);
			}
		}

		return this.expedientesService.filterExpedientesARelacionar(filterExpedientes, idExpediente, jqGridRequestDto,
				documentosTitular, false);
	}

	@RequestMapping(value = "/tablaInspeccionesARelacionar/{idExpediente}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<VTInsSinExpediente> filterInspeccionesARelacionar(
			@RequestJsonBody(param = "filter") VTInsSinExpediente filterVTInsSinExpediente,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto, @PathVariable Integer idExpediente) {
		ExpedientesRelacionadosController.logger
				.info("[POST - filter] : Obtener inspecciones a relacionar (Por ubicación)");

		List<DatosTerceros> listaTitulares = this.datosTercerosService.findTitularExpediente(idExpediente);
		String documentosTitular = "";

		if (!listaTitulares.isEmpty()) {
			for (DatosTerceros titular : listaTitulares) {
				if (titular.getDocumento() != null) {
					documentosTitular += "\'" + titular.getDocumento() + "\', ";
				}
			}

			if (documentosTitular.length() > 0) {
				documentosTitular = documentosTitular.substring(0, documentosTitular.length() - 2);
			}
		}

		return this.vTInsSinExpedienteService.filterInspeccionesARelacionar(filterVTInsSinExpediente, idExpediente,
				jqGridRequestDto, documentosTitular, false);
	}

	@RequestMapping(value = "/tablaExpedientesRelacionados/{idExpediente}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Expedientes> filterExpedientesRelacionados(
			@RequestJsonBody(param = "filter") Expedientes filterExpedientes,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto, @PathVariable Integer idExpediente) {
		ExpedientesRelacionadosController.logger
				.info("[POST - filter] : Obtener expedientes relacionados por ubicación (Relación Expediente)");

		return this.expedientesService.filterExpedientesRelacionadosUbicacionRelacionExpediente(
				filterExpedientes.getRelacionExpediente(), idExpediente, jqGridRequestDto);
	}

	@RequestMapping(value = "/tablaInspeccionesRelacionadas/{idExpediente}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<RelExpInsSinExp> filterInspeccionesRelacionadas(
			@RequestJsonBody(param = "filter") RelExpInsSinExp filterRelExpInsSinExp,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto, @PathVariable Integer idExpediente) {
		ExpedientesRelacionadosController.logger
				.info("[POST - filter] : Obtener inspecciones relacionadas por ubicación");

		return this.relExpInsSinExpService.filterInspeccionesRelacionadas(idExpediente, jqGridRequestDto);
	}

	@RequestMapping(value = "/relacionarExpediente/{idExpedienteARelacionar}/{idExpedienteOrigenAyudas}/{relacionExpediente}", method = RequestMethod.POST)
	public @ResponseBody Expedientes relacionarExpediente(
			@RequestBody(required = false) List<Expedientes> listaExpedientesRelacionados,
			@PathVariable Integer idExpedienteARelacionar, @PathVariable Integer idExpedienteOrigenAyudas,
			@PathVariable Integer relacionExpediente, @ModelAttribute("usuario") Usuario usuario) {

		return this.expedientesService.relacionarExpediente(listaExpedientesRelacionados, idExpedienteARelacionar,
				idExpedienteOrigenAyudas, relacionExpediente, usuario);
	}

	@RequestMapping(value = "/relacionarInspeccion/{idExpediente}", method = RequestMethod.POST)
	public @ResponseBody RelExpInsSinExp relacionarInspeccion(@RequestBody VTInsSinExpediente vTInsSinExpediente,
			@PathVariable Integer idExpediente, @ModelAttribute("usuario") Usuario usuario) {

		RelExpInsSinExp relExpInsSinExp = new RelExpInsSinExp();
		relExpInsSinExp.setIdExpediente(idExpediente);
		relExpInsSinExp.setIdInspeccion(vTInsSinExpediente.getIdInspeccion());

		String identificadorInspeccion = vTInsSinExpediente.getIdentificadorInspeccion();
		if (!"".equals(identificadorInspeccion) && null != identificadorInspeccion) {
			identificadorInspeccion = identificadorInspeccion.substring(identificadorInspeccion.indexOf('>') + 1);
			identificadorInspeccion = identificadorInspeccion.substring(0, identificadorInspeccion.indexOf('<'));
		}

		relExpInsSinExp.setIdentificadorInspeccion(identificadorInspeccion);
		relExpInsSinExp.setTitulo(vTInsSinExpediente.getTitulo());
		relExpInsSinExp.setIdTercero(vTInsSinExpediente.getIdTercero());
		relExpInsSinExp.setIdDireccion(vTInsSinExpediente.getIdDireccion());
		relExpInsSinExp.setUsuarioRegistro(usuario.getNombreUsuario());

		return this.relExpInsSinExpService.add(relExpInsSinExp);
	}

	@RequestMapping(value = "/eliminarRelacionExpediente/{idExpediente}/{expedienteHistorico}", method = RequestMethod.POST)
	public @ResponseBody Expedientes eliminarRelacion(@PathVariable Integer idExpediente,
			@PathVariable boolean expedienteHistorico, @ModelAttribute("usuario") Usuario usuario) {

		Expedientes expediente = new Expedientes();
		expediente.setIdExpediente(idExpediente);
		expediente.setUsuarioUltmodificacion(usuario.getNombreUsuario());

		if (expedienteHistorico) {
			expediente = this.expedientesService.eliminarRelacionHistorico(expediente);
		} else {
			expediente = this.expedientesService.eliminarRelacion(expediente);
		}

		return expediente;
	}

	@RequestMapping(value = "eliminarRelacionInspeccion/{idRelExpInsSinExp}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody RelExpInsSinExp eliminarRelacionInspeccion(@PathVariable Integer idRelExpInsSinExp) {
		RelExpInsSinExp relExpInsSinExp = new RelExpInsSinExp();
		relExpInsSinExp.setIdRelExpInsSinExp(idRelExpInsSinExp);
		this.relExpInsSinExpService.remove(relExpInsSinExp);
		ExpedientesRelacionadosController.logger.info("[DELETE] : RelExpInsSinExp borrado correctamente");

		return relExpInsSinExp;
	}

}
