package com.ejie.ab61.control.listados;

import java.io.IOException;

import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.multipart.MultipartFile;

import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.model.listados.Listados;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.listados.ListadosService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;

/**
 * 
 *  
 *
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/documentosListado")

public class DocumentosListadoController {

	private static final Logger logger = LoggerFactory.getLogger(DocumentosListadoController.class);

	@Autowired
	private ListadosService listadosService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	// /**
	// * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	// * indicado.
	// *
	// * @param idListado
	// * Integer
	// * @return Listados Objeto correspondiente al identificador indicado.
	// */
	// @RequestMapping(value = "/{idListado}", method = RequestMethod.GET)
	// public @ResponseBody Listados get(@PathVariable Integer idListado) {
	// Listados listados = new Listados();
	// listados.setIdListado(idListado);
	// listados = this.listadosService.find(listados);
	// DocumentosListadoController.logger.info("[GET - findBy_PK] : Obtener
	// Listados por PK");
	// return listados;
	// }

	@RequestMapping(value = "/modificar/{idListado}", method = RequestMethod.GET)
	public String getFormEdit(Model model, @PathVariable Integer idListado) throws IOException {
		DocumentosListadoController.logger.info("[GET - View] : busquedasParametros");
		Listados listados = new Listados();
		listados.setIdListado(idListado);
		listados = this.listadosService.find(listados);
		model.addAttribute("idListado", idListado);
		model.addAttribute("listados", listados);
		model.addAttribute("origen", Constantes.origen.DOCUMENTOS_LISTADOS);
		return "busquedasParametros";
	}

	@RequestMapping(value = "/anyadir", method = RequestMethod.GET)
	public String getFormAdd(Model model) {
		DocumentosListadoController.logger.info("[GET - View] : busquedasParametros");
		model.addAttribute("nombreListado", "");
		model.addAttribute("idListado", 0);
		model.addAttribute("listados", new Listados());
		return "busquedasParametros";
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param listados
	 *            Listados Bean que contiene la informacion a modificar.
	 * @return Listados Bean resultante de la modificacion.
	 * @throws IOException
	 */
	@RequestMapping(value = "/modif", method = RequestMethod.POST)
	public @ResponseBody Listados edit(Model model, @ModelAttribute Listados listados,
			@RequestParam(value = "file", required = false) MultipartFile file,
			@ModelAttribute(value = "usuario") Usuario usuario) throws IOException {
		Listados listadosAux = new Listados();
		boolean updateSinFichero = false;
		if (file.getSize() > 0) {
			listados.setFicheroArray(file.getBytes());
			listados.setNombreFichero(file.getOriginalFilename());
			listadosAux.setNombreFichero(file.getOriginalFilename());
		} else {
			updateSinFichero = true;
		}
		if (0 == listados.getIdListado()) {
			listados.setUsuarioRegistro(usuario.getNombreUsuario());
			listadosAux = this.listadosService.add(listados);
		} else {
			listados.setUsuarioUltmodificacion(usuario.getNombreUsuario());
			if (updateSinFichero) {
				listadosAux = this.listadosService.updateSinFichero(listados);
			} else {
				listadosAux = this.listadosService.update(listados);
			}
		}
		model.addAttribute("idListado", listadosAux.getIdListado());
		DocumentosListadoController.logger.info("[PUT] : Listados actualizado correctamente");
		return listadosAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idListado
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Listados Bean eliminado.
	 */
	@RequestMapping(value = "/{idListado}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Listados remove(@PathVariable Integer idListado) {
		Listados listados = new Listados();
		listados.setIdListado(idListado);
		this.listadosService.remove(listados);
		DocumentosListadoController.logger.info("[DELETE] : Listados borrado correctamente");
		return listados;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model, Usuario usuario) {
		DocumentosListadoController.logger.info("[GET - View] : listados");
		model.addAttribute("listados", new Listados());
		Authentication aut = SecurityContextHolder.getContext().getAuthentication();
		/** Dejamos el usuario logueado en session */
		String username = "";
		if (null != aut) {
			username = (String) aut.getPrincipal();
		}
		usuario.setNombreUsuario(username);
		model.addAttribute("usuario", usuario);
		model.addAttribute("origen", Constantes.origen.DOCUMENTOS_LISTADOS);
		return "documentosListado";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterListados
	 *            Listados Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Listados> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Listados> filter(@RequestJsonBody(param = "filter") Listados filterListados,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DocumentosListadoController.logger.info("[POST - filter] : Obtener Listadoss");
		return this.listadosService.filter(filterListados, jqGridRequestDto, false);
	}

	@RequestMapping(value = "/refrescarPantalla", method = RequestMethod.GET)
	public String refrescar(Model model, Usuario usuario) {
		DocumentosListadoController.logger.info("[GET - View] : refrescarPantalla");
		model.addAttribute("listados", new Listados());
		Authentication aut = SecurityContextHolder.getContext().getAuthentication();
		/** Dejamos el usuario logueado en session */
		String username = "";
		if (null != aut) {
			username = (String) aut.getPrincipal();
		}
		usuario.setNombreUsuario(username);
		model.addAttribute("usuario", usuario);
		model.addAttribute("origen", Constantes.origen.DOCUMENTOS_LISTADOS);
		return "documentosListado";
	}

	@RequestMapping(value = "/borrarFichero/{idListado}", method = RequestMethod.POST)
	public @ResponseBody Listados borrarFichero(@PathVariable Integer idListado,
			@ModelAttribute(value = "usuario") Usuario usuario) {
		Listados listados = new Listados(idListado);
		listados.setUsuarioUltmodificacion(usuario.getNombreUsuario());
		Listados listadosAux = this.listadosService.removeFicheros(listados);
		DocumentosListadoController.logger.info("[POST] : Listados insertado correctamente");
		return listadosAux;
	}

	@RequestMapping(value = "/descargarFichero/{idListado}", method = RequestMethod.GET)
	public void descargarFichero(@PathVariable Integer idListado, HttpServletResponse response) throws IOException {
		DocumentosListadoController.logger.info("[POST - descargarFichero] : Inicio descargarFichero");
		this.listadosService.descargarFichero(idListado, response);
		DocumentosListadoController.logger.info("[POST - descargarFichero] : Fin descargarFichero");
	}
}
