package com.ejie.ab61.control.plantillas;

import java.sql.SQLException;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;

import com.ejie.ab61.comun.CargarPropiedades;
import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.comun.SeguridadAplicacion;
import com.ejie.ab61.model.plantillas.Plantillas;
import com.ejie.ab61.model.plantillas.RelPlaCam;
import com.ejie.ab61.model.procedimientos.Procedimientos;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.plantillas.PlantillasService;
import com.ejie.ab61.service.plantillas.RelPlaCamService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;

/**
 * MantenimientoPlantillasController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/mantenimientoPlantillas")

public class MantenimientoPlantillasController {

	private static final Logger logger = LoggerFactory.getLogger(MantenimientoPlantillasController.class);

	@Autowired
	private PlantillasService plantillasService;

	@Autowired
	private RelPlaCamService relPlaCamService;

	public static final String TEMP_PATH = CargarPropiedades.getConfigAplicacion("TEMP.DIR");

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idPlantilla
	 *            Integer
	 * @return Plantillas Objeto correspondiente al identificador indicado.
	 * @throws SQLException
	 */
	@RequestMapping(value = "/{idPlantilla}", method = RequestMethod.GET)
	public @ResponseBody Plantillas get(@PathVariable Integer idPlantilla, Model model) throws SQLException {
		Plantillas plantillas = new Plantillas();
		plantillas.setIdPlantilla(idPlantilla);
		plantillas = this.plantillasService.find(plantillas);
		RelPlaCam camposPlantilla = new RelPlaCam();
		camposPlantilla.setIdDocumento(idPlantilla);
		List<RelPlaCam> campos = this.relPlaCamService.findAll(camposPlantilla, null);
		String camposDesignados = "";
		if (campos != null && !campos.isEmpty()) {
			for (int i = 0; i < campos.size(); i++) {
				camposDesignados += String.valueOf(campos.get(i).getIdCampo()) + ",";
			}
		}
		plantillas.setCampos(camposDesignados);
		if (plantillas.getDocumento().getArchivo() != null && plantillas.getDocumento().getArchivo().length() > 0) {
			plantillas.setTienePlantilla(1);
		} else {
			plantillas.setTienePlantilla(0);
		}

		MantenimientoPlantillasController.logger.info("[GET - findBy_PK] : Obtener Plantillas por PK");

		return plantillas;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterPlantillas
	 *            Plantillas Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Plantillas> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Plantillas> getAll(@ModelAttribute Plantillas filterPlantillas) {
		MantenimientoPlantillasController.logger.info("[GET - find_ALL] : Obtener Plantillas por filtro");
		return this.plantillasService.findAll(filterPlantillas, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param plantillas
	 *            Plantillas Bean que contiene la informacion a modificar.
	 * @return Plantillas Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody Plantillas edit(@RequestBody Plantillas plantillas,
			@ModelAttribute(value = "usuario") Usuario usuario) {

		plantillas.setFechaUltmodificacion(new Date());
		plantillas.setUsuarioUltmodificacion(usuario.getNombreUsuario());

		Plantillas plantillasAux = this.plantillasService.update(plantillas);
		MantenimientoPlantillasController.logger.info("[PUT] : Plantillas actualizado correctamente");
		return plantillasAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param plantillas
	 *            Plantillas Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return Plantillas Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Plantillas add(@RequestBody Plantillas plantillas,
			@ModelAttribute(value = "usuario") Usuario usuario) {

		plantillas.setFechaRegistro(new Date());
		plantillas.setUsuarioRegistro(usuario.getNombreUsuario());

		Plantillas plantillasAux = this.plantillasService.add(plantillas);
		MantenimientoPlantillasController.logger.info("[POST] : Plantillas insertado correctamente");
		return plantillasAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idPlantilla
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Plantillas Bean eliminado.
	 */
	@RequestMapping(value = "/{idPlantilla}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Plantillas remove(@PathVariable Integer idPlantilla) {
		Plantillas plantillas = new Plantillas();
		plantillas.setIdPlantilla(idPlantilla);
		this.plantillasService.remove(plantillas);
		MantenimientoPlantillasController.logger.info("[DELETE] : Plantillas borrado correctamente");
		return plantillas;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model, Usuario usuario) {
		MantenimientoPlantillasController.logger.info("[GET - View] : plantillas");
		model.addAttribute("plantillas", new Plantillas());
		model.addAttribute("origen", Constantes.origen.GESTION_PLANTILLAS);

		Authentication aut = SecurityContextHolder.getContext().getAuthentication();
		/** Dejamos el usuario logueado en session */
		String username = "";
		if (null != aut) {
			username = (String) aut.getPrincipal();
		}
		usuario.setNombreUsuario(username);
		model.addAttribute("usuario", usuario);

		Vector<String> rolesUsuario = SeguridadAplicacion
				.getRolesUsuario(SecurityContextHolder.getContext().getAuthentication());

		if (rolesUsuario.contains(SeguridadAplicacion.ROLE_AB61_IN_0008)) {

			model.addAttribute("roleAb61In0008", true);

			if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_TURISMO)) {
				model.addAttribute("roleTecnicoTurismo", true);
			} else {
				model.addAttribute("roleTecnicoTurismo", false);
			}

			if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_COMERCIO)) {
				model.addAttribute("roleTecnicoComercio", true);
			} else {
				model.addAttribute("roleTecnicoComercio", false);
			}

			if (rolesUsuario.contains(SeguridadAplicacion.ROLE_TECNICO_CONSUMO)) {
				model.addAttribute("roleTecnicoConsumo", true);
			} else {
				model.addAttribute("roleTecnicoConsumo", false);
			}

		} else {
			model.addAttribute("roleAb61In0008", false);
		}

		return "mantenimientoPlantillas";
	}

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint/{idProcedimiento}/{idExpediente}", method = RequestMethod.GET)
	public String getFormEditPlantilla(@PathVariable Integer idProcedimiento, @PathVariable String idExpediente,
			Model model) {
		MantenimientoPlantillasController.logger.info("[GET - View] : plantillas");
		model.addAttribute("idProcedimiento", idProcedimiento);
		model.addAttribute("idExpediente", idExpediente);
		model.addAttribute("plantilla", new Plantillas());
		return "plantillas";
	}

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/inicio/buscador/{codAplic}", method = RequestMethod.GET)
	public String getFormEditPlantilla(@PathVariable Integer codAplic, Model model) {
		MantenimientoPlantillasController.logger.info("[GET - View] : plantillas");
		return "plantillas";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterPlantillas
	 *            Plantillas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Plantillas> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Plantillas> filter(
			@RequestJsonBody(param = "filter") Plantillas filterPlantillas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MantenimientoPlantillasController.logger.info("[POST - filter] : Obtener Plantillass");
		return this.plantillasService.filterMantenimientoPlantillas(filterPlantillas, jqGridRequestDto, false);
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterPlantillas
	 *            Plantillas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Plantillas> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/plantillasPorDireccion/{direccion}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Plantillas> plantillasPorDireccion(
			@RequestJsonBody(param = "filter") Plantillas filterPlantillas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto, @PathVariable Integer direccion) {
		MantenimientoPlantillasController.logger.info("[POST - filter] : Obtener Plantillass");
		return this.plantillasService.filterMantenimientoPlantillasPorDireccion(filterPlantillas, jqGridRequestDto,
				false, direccion);
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 *
	 * @param filterPlantillas
	 *            Plantillas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Plantillas> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/tabla/{idProcedimiento}/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Plantillas> filter2(
			@RequestJsonBody(param = "filter") Plantillas filterPlantillas, @PathVariable Integer idProcedimiento,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MantenimientoPlantillasController.logger.info("[POST - filter] : Obteneer Plantillass");
		filterPlantillas.setProcedimientos(new Procedimientos(idProcedimiento));
		return this.plantillasService.filter(filterPlantillas, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterPlantillas
	 *            Plantillas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") Plantillas filterPlantillas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MantenimientoPlantillasController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Plantillass");
		this.plantillasService.removeMultiple(filterPlantillas, jqGridRequestDto, false);
		MantenimientoPlantillasController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterPlantillas
	 *            Plantillas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Plantillas>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Plantillas>> jerarquia(
			@RequestJsonBody(param = "filter") Plantillas filterPlantillas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MantenimientoPlantillasController.logger.info("[POST - jerarquia] : Obtener Plantillass jerarquia");
		return this.plantillasService.jerarquia(filterPlantillas, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterPlantillas
	 *            Plantillas Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Plantillas>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Plantillas>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") Plantillas filterPlantillas,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MantenimientoPlantillasController.logger.info("[POST - jerarquia] : Obtener Plantillass jerarquia - Hijos");
		return this.plantillasService.jerarquiaChildren(filterPlantillas, jqGridRequestDto);
	}

}
