package com.ejie.ab61.control.plantillas;

import com.ejie.ab61.model.plantillas.RelPlaCam;
import com.ejie.ab61.service.plantillas.RelPlaCamService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * RelPlaCamController  
 * 
 *  
 */

@Controller
@RequestMapping(value = "/relplacam")

public class RelPlaCamController {

	private static final Logger logger = LoggerFactory.getLogger(RelPlaCamController.class);

	@Autowired
	private RelPlaCamService relPlaCamService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idRelPlaCam
	 *            String
	 * @return RelPlaCam Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idRelPlaCam}", method = RequestMethod.GET)
	public @ResponseBody RelPlaCam get(@PathVariable String idRelPlaCam) {
		RelPlaCam relPlaCam = new RelPlaCam();
		// relPlaCam.setIdRelPlaCam(idRelPlaCam);
		relPlaCam = this.relPlaCamService.find(relPlaCam);
		RelPlaCamController.logger.info("[GET - findBy_PK] : Obtener RelPlaCam por PK");
		return relPlaCam;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterRelPlaCam
	 *            RelPlaCam Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<RelPlaCam> Lista de objetos correspondientes a la busqueda
	 *         realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<RelPlaCam> getAll(@ModelAttribute RelPlaCam filterRelPlaCam) {
		RelPlaCamController.logger.info("[GET - find_ALL] : Obtener RelPlaCam por filtro");
		return this.relPlaCamService.findAll(filterRelPlaCam, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param relPlaCam
	 *            RelPlaCam Bean que contiene la informacion a modificar.
	 * @return RelPlaCam Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody RelPlaCam edit(@RequestBody RelPlaCam relPlaCam) {
		RelPlaCam relPlaCamAux = this.relPlaCamService.update(relPlaCam);
		RelPlaCamController.logger.info("[PUT] : RelPlaCam actualizado correctamente");
		return relPlaCamAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param relPlaCam
	 *            RelPlaCam Bean que contiene la informacion con la que se va a
	 *            crear el nuevo registro.
	 * @return RelPlaCam Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody RelPlaCam add(@RequestBody RelPlaCam relPlaCam) {
		RelPlaCam relPlaCamAux = this.relPlaCamService.add(relPlaCam);
		RelPlaCamController.logger.info("[POST] : RelPlaCam insertado correctamente");
		return relPlaCamAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idRelPlaCam
	 *            String Identificador del objeto que se desea eliminar.
	 * @return RelPlaCam Bean eliminado.
	 */
	@RequestMapping(value = "/{idRelPlaCam}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody RelPlaCam remove(@PathVariable String idRelPlaCam) {
		RelPlaCam relPlaCam = new RelPlaCam();
		// relPlaCam.setIdRelPlaCam(idRelPlaCam);
		this.relPlaCamService.remove(relPlaCam);
		RelPlaCamController.logger.info("[DELETE] : RelPlaCam borrado correctamente");
		return relPlaCam;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		RelPlaCamController.logger.info("[GET - View] : relplacam");
		return "relplacam";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterRelPlaCam
	 *            RelPlaCam Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<RelPlaCam> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<RelPlaCam> filter(
			@RequestJsonBody(param = "filter") RelPlaCam filterRelPlaCam,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		RelPlaCamController.logger.info("[POST - filter] : Obtener RelPlaCams");
		return this.relPlaCamService.filter(filterRelPlaCam, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterRelPlaCam
	 *            RelPlaCam Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchRelPlaCam
	 *            RelPlaCam Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<RelPlaCam> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<RelPlaCam>> search(
			@RequestJsonBody(param = "filter") RelPlaCam filterRelPlaCam,
			@RequestJsonBody(param = "search") RelPlaCam searchRelPlaCam,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		RelPlaCamController.logger.info("[POST - search] : Buscar RelPlaCams");
		return this.relPlaCamService.search(filterRelPlaCam, searchRelPlaCam, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterRelPlaCam
	 *            RelPlaCam Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") RelPlaCam filterRelPlaCam,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		RelPlaCamController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples RelPlaCams");
		this.relPlaCamService.removeMultiple(filterRelPlaCam, jqGridRequestDto, false);
		RelPlaCamController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterRelPlaCam
	 *            RelPlaCam Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<RelPlaCam>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<RelPlaCam>> jerarquia(
			@RequestJsonBody(param = "filter") RelPlaCam filterRelPlaCam,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		RelPlaCamController.logger.info("[POST - jerarquia] : Obtener RelPlaCams jerarquia");
		return this.relPlaCamService.jerarquia(filterRelPlaCam, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterRelPlaCam
	 *            RelPlaCam Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<RelPlaCam>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<RelPlaCam>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") RelPlaCam filterRelPlaCam,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		RelPlaCamController.logger.info("[POST - jerarquia] : Obtener RelPlaCams jerarquia - Hijos");
		return this.relPlaCamService.jerarquiaChildren(filterRelPlaCam, jqGridRequestDto);
	}

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idConvocatoria
	 *            Integer
	 * @return Convocatorias Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "obtenerNumDocumentosCampo/{idCampo}", method = RequestMethod.GET)
	public @ResponseBody Long obtenerNumDocumentosCampo(@PathVariable Integer idCampo) {
		Long temp = this.relPlaCamService.obtenerNumDocumentosCampo(idCampo);

		return temp;
	}
}
