package com.ejie.ab61.control.plantillas;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.servlet.ModelAndView;

import com.ejie.ab61.comun.Constantes;
import com.ejie.ab61.model.inicio.InitForm;
import com.ejie.ab61.model.plantillas.RelTipPlaPro;
import com.ejie.ab61.model.plantillas.TiposPlantilla;
import com.ejie.ab61.model.usuario.Usuario;
import com.ejie.ab61.service.plantillas.RelTipPlaProService;
import com.ejie.ab61.service.plantillas.TiposPlantillaService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import com.ejie.x38.reports.ReportData;

/**
 * TiposPlantillaController  
 * 
 *  
 */

@Controller
@SessionAttributes("usuario")
@RequestMapping(value = "/tiposplantilla")

public class TiposPlantillaController {

	private static final Logger logger = LoggerFactory.getLogger(TiposPlantillaController.class);

	@Autowired
	private TiposPlantillaService tiposPlantillaService;

	@Autowired
	private RelTipPlaProService relTipPlaProService;

	@Resource
	private ReloadableResourceBundleMessageSource appMessageSource;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idTipoPlantilla
	 *            Long
	 * @return TiposPlantilla Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idTipoPlantilla}", method = RequestMethod.GET)
	public @ResponseBody TiposPlantilla get(@PathVariable Long idTipoPlantilla) {
		TiposPlantilla tiposPlantilla = new TiposPlantilla();
		tiposPlantilla.setIdTipoPlantilla(idTipoPlantilla);
		tiposPlantilla = this.tiposPlantillaService.find(tiposPlantilla);
		TiposPlantillaController.logger.info("[GET - findBy_PK] : Obtener TiposPlantilla por PK");
		return tiposPlantilla;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterTiposPlantilla
	 *            TiposPlantilla Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<TiposPlantilla> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(value = "/verValorCombo", method = RequestMethod.GET)
	public @ResponseBody List<TiposPlantilla> init() {
		TiposPlantillaController.logger.info("[[GET - getValorCombo] : Obtener valores del combo ");
		List<TiposPlantilla> lista = this.tiposPlantillaService.findTipos();
		return lista;
	}

	/**
	 * Devuelve una lista de tiposplantilla cuyo tipo sea tipo de plantilla.
	 *
	 * @param filterTiposPlantilla
	 *            TiposPlantilla Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<TiposPlantilla> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(value = "/tipoDePlantilla", method = RequestMethod.GET)
	public @ResponseBody List<TiposPlantilla> tipoDePlantilla(@ModelAttribute TiposPlantilla filterTiposPlantilla) {
		TiposPlantillaController.logger
				.info("[GET - find_ALL] : Obtener TiposPlantilla cuyo tipo sea tipo de plantilla");
		filterTiposPlantilla.setTiposDocumento(Constantes.TIPO_DOCUMENTO_DE_PLANTILLA);
		return this.tiposPlantillaService.findAll(filterTiposPlantilla, null);
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterTiposPlantilla
	 *            TiposPlantilla Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<TiposPlantilla> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<TiposPlantilla> getAll(@ModelAttribute TiposPlantilla filterTiposPlantilla) {
		TiposPlantillaController.logger.info("[GET - find_ALL] : Obtener TiposPlantilla por filtro");
		return this.tiposPlantillaService.findAll(filterTiposPlantilla, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param tiposPlantilla
	 *            TiposPlantilla Bean que contiene la informacion a modificar.
	 * @return TiposPlantilla Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody TiposPlantilla edit(@RequestBody TiposPlantilla tiposPlantilla,
			@ModelAttribute(value = "usuario") Usuario usuario) {

		tiposPlantilla.setFechaUltmodificacion(new Date());
		tiposPlantilla.setUsuarioUltmodificacion(usuario.getNombreUsuario());
		TiposPlantilla tiposPlantillaAux = this.tiposPlantillaService.update(tiposPlantilla);

		List<RelTipPlaPro> listaRelTipPlaPro = this.relTipPlaProService.findByTipoPlantilla(tiposPlantilla);
		if (!listaRelTipPlaPro.isEmpty()) {
			this.relTipPlaProService.removeByTipoPlantilla(tiposPlantilla);
		}

		if (tiposPlantilla.getIdsProcedimientos() != null && !"".equals(tiposPlantilla.getIdsProcedimientos())) {
			String[] idsProcedimientos = tiposPlantilla.getIdsProcedimientos().split(",");
			for (String idProcedimiento : idsProcedimientos) {
				RelTipPlaPro relTipPlaPro = new RelTipPlaPro();
				relTipPlaPro.setIdTipoPlantilla(tiposPlantillaAux.getIdTipoPlantilla().intValue());
				relTipPlaPro.setIdProcedimiento(Integer.parseInt(idProcedimiento));
				relTipPlaPro.setUsuarioRegistro(usuario.getNombreUsuario());
				this.relTipPlaProService.add(relTipPlaPro);
			}
		}

		TiposPlantillaController.logger.info("[PUT] : TiposPlantilla actualizado correctamente");
		return tiposPlantillaAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param tiposPlantilla
	 *            TiposPlantilla Bean que contiene la informacion con la que se
	 *            va a crear el nuevo registro.
	 * @return TiposPlantilla Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody TiposPlantilla add(@RequestBody TiposPlantilla tiposPlantilla,
			@ModelAttribute(value = "usuario") Usuario usuario) {

		tiposPlantilla.setFechaRegistro(new Date());
		tiposPlantilla.setUsuarioRegistro(usuario.getNombreUsuario());

		TiposPlantilla tiposPlantillaAux = this.tiposPlantillaService.add(tiposPlantilla);

		if (tiposPlantilla.getIdsProcedimientos() != null && !"".equals(tiposPlantilla.getIdsProcedimientos())) {
			String[] idsProcedimientos = tiposPlantilla.getIdsProcedimientos().split(",");
			for (String idProcedimiento : idsProcedimientos) {
				RelTipPlaPro relTipPlaPro = new RelTipPlaPro();
				relTipPlaPro.setIdTipoPlantilla(tiposPlantillaAux.getIdTipoPlantilla().intValue());
				relTipPlaPro.setIdProcedimiento(Integer.parseInt(idProcedimiento));
				relTipPlaPro.setUsuarioRegistro(usuario.getNombreUsuario());
				this.relTipPlaProService.add(relTipPlaPro);
			}
		}

		TiposPlantillaController.logger.info("[POST] : TiposPlantilla insertado correctamente");
		return tiposPlantillaAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idTipoPlantilla
	 *            Long Identificador del objeto que se desea eliminar.
	 * @return TiposPlantilla Bean eliminado.
	 */
	@RequestMapping(value = "/{idTipoPlantilla}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody TiposPlantilla remove(@PathVariable Long idTipoPlantilla) {
		TiposPlantilla tiposPlantilla = new TiposPlantilla();
		tiposPlantilla.setIdTipoPlantilla(idTipoPlantilla);
		this.relTipPlaProService.removeByTipoPlantilla(tiposPlantilla);
		this.tiposPlantillaService.remove(tiposPlantilla);
		TiposPlantillaController.logger.info("[DELETE] : TiposPlantilla borrado correctamente");
		return tiposPlantilla;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		TiposPlantillaController.logger.info("[GET - View] : tiposplantilla");
		model.addAttribute("tiposPlantilla", new TiposPlantilla());
		model.addAttribute("origen", Constantes.origen.TIPOS_DE_DOCUMENTO);
		return "tiposplantilla";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterTiposPlantilla
	 *            TiposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<TiposPlantilla> Dto que contiene el resultado
	 *         del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<TiposPlantilla> filter(
			@RequestJsonBody(param = "filter") TiposPlantilla filterTiposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposPlantillaController.logger.info("[POST - filter] : Obtener TiposPlantillas");
		return this.tiposPlantillaService.filter(filterTiposPlantilla, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterTiposPlantilla
	 *            TiposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchTiposPlantilla
	 *            TiposPlantilla Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<TiposPlantilla> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<TiposPlantilla>> search(
			@RequestJsonBody(param = "filter") TiposPlantilla filterTiposPlantilla,
			@RequestJsonBody(param = "search") TiposPlantilla searchTiposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposPlantillaController.logger.info("[POST - search] : Buscar TiposPlantillas");
		return this.tiposPlantillaService.search(filterTiposPlantilla, searchTiposPlantilla, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterTiposPlantilla
	 *            TiposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param = "filter") TiposPlantilla filterTiposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposPlantillaController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples TiposPlantillas");
		this.tiposPlantillaService.removeMultiple(filterTiposPlantilla, jqGridRequestDto, false);
		TiposPlantillaController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterTiposPlantilla
	 *            TiposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<TiposPlantilla>> Dto que contiene
	 *         el resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<TiposPlantilla>> jerarquia(
			@RequestJsonBody(param = "filter") TiposPlantilla filterTiposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposPlantillaController.logger.info("[POST - jerarquia] : Obtener TiposPlantillas jerarquia");
		return this.tiposPlantillaService.jerarquia(filterTiposPlantilla, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterTiposPlantilla
	 *            TiposPlantilla Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<TiposPlantilla>> Dto que contiene
	 *         el resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<TiposPlantilla>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") TiposPlantilla filterTiposPlantilla,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposPlantillaController.logger.info("[POST - jerarquia] : Obtener TiposPlantillas jerarquia - Hijos");
		return this.tiposPlantillaService.jerarquiaChildren(filterTiposPlantilla, jqGridRequestDto);
	}

	@RequestMapping(value = { "/xlsReport", "/xlsxReport" }, method = RequestMethod.POST)
	protected ModelAndView getExcelPOI(@ModelAttribute TiposPlantilla filterCamposPlantilla,
			@ModelAttribute JQGridRequestDto jqGridRequestDto, ModelMap modelMap,
			@ModelAttribute(value = "initForm") InitForm initForm,
			@RequestParam(value = "columns", required = false) String columns, HttpServletRequest request) {

		Locale locale = LocaleContextHolder.getLocale();
		if (locale.getLanguage().equals("es")) {
			columns = "[['descripcionCastellano','" + appMessageSource.getMessage("Descripción", null, locale) + "']"
					+ ",['documentroleNameCas','"
					+ appMessageSource.getMessage("Document Role Name (Castellano)", null, locale) + "']"
					+ ",['documentroleNameEus','"
					+ appMessageSource.getMessage("Document Role Name (Euskera)", null, locale) + "']]";
		} else {
			columns = "[['descripcionEuskera','" + appMessageSource.getMessage("Deskribapena", null, locale) + "']"
					+ ",['documentroleNameCas','"
					+ appMessageSource.getMessage("Document Role Name (Gaztelera)", null, locale) + "']"
					+ ",['documentroleNameEus','"
					+ appMessageSource.getMessage("Document Role Name (Euskera)", null, locale) + "']]";
		}
		// Acceso a BD para recuperar datos
		List<TiposPlantilla> listactSubPage = this.tiposPlantillaService.findAllLike(filterCamposPlantilla, null, null);
		jqGridRequestDto.setPage(null);
		jqGridRequestDto.setRows(null);

		// Nombre fichero
		if (locale.getLanguage().equals("es")) {
			modelMap.put("fileName", appMessageSource.getMessage("Tipos plantilla", null, locale));
		} else {
			modelMap.put("fileName", appMessageSource.getMessage("Txantiloi motak", null, locale));
		}

		// Datos
		List<Object> reportData = new ArrayList<Object>();
		// Hoja 1
		ReportData<TiposPlantilla> excelDataAll = new ReportData<TiposPlantilla>();
		// nombre hoja
		if (locale.getLanguage().equals("es")) {
			excelDataAll.setSheetName(appMessageSource.getMessage("Tipos plantilla", null, locale));
		} else {
			excelDataAll.setSheetName(appMessageSource.getMessage("Txantiloi motak", null, locale));
		}
		// cabeceras hoja
		excelDataAll.setHeaderNames(ReportData.parseColumns(columns));
		// datos hoja
		excelDataAll.setModelData(listactSubPage);
		reportData.add(excelDataAll);
		modelMap.put("reportData", reportData);
		// GeneraciÃ³n del XLS o XLSX
		if (request.getServletPath().indexOf("xlsReport") != -1) {
			return new ModelAndView("xlsReport", modelMap);
		} else {
			return new ModelAndView("xlsxReport", modelMap);
		}
	}

}
