package com.ejie.ab61.control.procedimientos;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.ejie.ab61.model.procedimientos.Aplicaciones;
import com.ejie.ab61.service.procedimientos.AplicacionesService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;

/**
 * AplicacionesController  
 * 
 *  
 */

@Controller
@RequestMapping(value = "/aplicaciones")

public class AplicacionesController {

	private static final Logger logger = LoggerFactory.getLogger(AplicacionesController.class);

	@Autowired
	private AplicacionesService aplicacionesService;

	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */

	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador
	 * indicado.
	 * 
	 * @param idAplicacion
	 *            Integer
	 * @return Aplicaciones Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idAplicacion}", method = RequestMethod.GET)
	public @ResponseBody Aplicaciones get(@PathVariable Integer idAplicacion) {
		Aplicaciones aplicaciones = new Aplicaciones();
		aplicaciones.setIdAplicacion(idAplicacion);
		aplicaciones = this.aplicacionesService.find(aplicaciones);
		AplicacionesController.logger.info("[GET - findBy_PK] : Obtener Aplicaciones por PK");
		return aplicaciones;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterAplicaciones
	 *            Aplicaciones Objeto que contiene los parametros de filtrado
	 *            utilizados en la busqueda.
	 * @return List<Aplicaciones> Lista de objetos correspondientes a la
	 *         busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Aplicaciones> getAll(@ModelAttribute Aplicaciones filterAplicaciones) {
		AplicacionesController.logger.info("[GET - find_ALL] : Obtener Aplicaciones por filtro");
		return this.aplicacionesService.findAll(filterAplicaciones, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param aplicaciones
	 *            Aplicaciones Bean que contiene la informacion a modificar.
	 * @return Aplicaciones Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
	public @ResponseBody Aplicaciones edit(@RequestBody Aplicaciones aplicaciones) {
		Aplicaciones aplicacionesAux = this.aplicacionesService.update(aplicaciones);
		AplicacionesController.logger.info("[PUT] : Aplicaciones actualizado correctamente");
		return aplicacionesAux;
	}

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param aplicaciones
	 *            Aplicaciones Bean que contiene la informacion con la que se va
	 *            a crear el nuevo registro.
	 * @return Aplicaciones Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Aplicaciones add(@RequestBody Aplicaciones aplicaciones) {
		Aplicaciones aplicacionesAux = this.aplicacionesService.add(aplicaciones);
		AplicacionesController.logger.info("[POST] : Aplicaciones insertado correctamente");
		return aplicacionesAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idAplicacion
	 *            Integer Identificador del objeto que se desea eliminar.
	 * @return Aplicaciones Bean eliminado.
	 */
	@RequestMapping(value = "/{idAplicacion}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody Aplicaciones remove(@PathVariable Integer idAplicacion) {
		Aplicaciones aplicaciones = new Aplicaciones();
		aplicaciones.setIdAplicacion(idAplicacion);
		this.aplicacionesService.remove(aplicaciones);
		AplicacionesController.logger.info("[DELETE] : Aplicaciones borrado correctamente");
		return aplicaciones;
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	/**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model
	 *            Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		AplicacionesController.logger.info("[GET - View] : aplicaciones");
		return "aplicaciones";
	}

	/**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterAplicaciones
	 *            Aplicaciones Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Aplicaciones> Dto que contiene el resultado del
	 *         filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Aplicaciones> filter(
			@RequestJsonBody(param = "filter") Aplicaciones filterAplicaciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AplicacionesController.logger.info("[POST - filter] : Obtener Aplicacioness");
		return this.aplicacionesService.filter(filterAplicaciones, jqGridRequestDto, false);
	}

	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterAplicaciones
	 *            Aplicaciones Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param searchAplicaciones
	 *            Aplicaciones Bean que contiene los parametros de busqueda a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Aplicaciones> Dto que contiene el resultado de la
	 *         busqueda realizada por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Aplicaciones>> search(
			@RequestJsonBody(param = "filter") Aplicaciones filterAplicaciones,
			@RequestJsonBody(param = "search") Aplicaciones searchAplicaciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AplicacionesController.logger.info("[POST - search] : Buscar Aplicacioness");
		return this.aplicacionesService.search(filterAplicaciones, searchAplicaciones, jqGridRequestDto, false);
	}

	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterAplicaciones
	 *            Aplicaciones Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String> Lista de los identificadores de los registros
	 *         eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(@RequestJsonBody(param = "filter") Aplicaciones filterAplicaciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AplicacionesController.logger
				.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Aplicacioness");
		this.aplicacionesService.removeMultiple(filterAplicaciones, jqGridRequestDto, false);
		AplicacionesController.logger.info("All entities correctly deleted!");

		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}

	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */

	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterAplicaciones
	 *            Aplicaciones Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Aplicaciones>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Aplicaciones>> jerarquia(
			@RequestJsonBody(param = "filter") Aplicaciones filterAplicaciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AplicacionesController.logger.info("[POST - jerarquia] : Obtener Aplicacioness jerarquia");
		return this.aplicacionesService.jerarquia(filterAplicaciones, jqGridRequestDto, false);
	}

	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion
	 * jerarquica.
	 * 
	 * @param filterAplicaciones
	 *            Aplicaciones Bean que contiene los parametros de filtrado a
	 *            emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Aplicaciones>> Dto que contiene el
	 *         resultado del filtrado realizado por el componente RUP_TABLE.
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Aplicaciones>> jerarquiaChildren(
			@RequestJsonBody(param = "filter") Aplicaciones filterAplicaciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		AplicacionesController.logger.info("[POST - jerarquia] : Obtener Aplicacioness jerarquia - Hijos");
		return this.aplicacionesService.jerarquiaChildren(filterAplicaciones, jqGridRequestDto);
	}

}
