package com.ejie.ab61.control.procedimientos;

import com.ejie.ab61.model.procedimientos.Departamentos;
import com.ejie.ab61.service.procedimientos.DepartamentosService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * DepartamentosController  
 *  
 */
 
@Controller
@RequestMapping (value = "/departamentos")

public class DepartamentosController  {

	private static final Logger logger = LoggerFactory.getLogger(DepartamentosController.class);

	@Autowired
	private DepartamentosService departamentosService;
	
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */
	
	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador indicado.
	 * 
	 * @param idDepartamento Integer
	 * @return Departamentos 
	 *            Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idDepartamento}", method = RequestMethod.GET)
	public @ResponseBody Departamentos get(@PathVariable Integer idDepartamento) {
        Departamentos departamentos = new Departamentos();
		departamentos.setIdDepartamento(idDepartamento);
        departamentos = this.departamentosService.find(departamentos);
        DepartamentosController.logger.info("[GET - findBy_PK] : Obtener Departamentos por PK");
        return departamentos;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterDepartamentos Departamentos
	 *            Objeto que contiene los parametros de filtrado utilizados en
	 *            la busqueda.
	 * @return List<Departamentos> 
	 *            Lista de objetos correspondientes a la busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Departamentos> getAll(@ModelAttribute Departamentos filterDepartamentos) {
		DepartamentosController.logger.info("[GET - find_ALL] : Obtener Departamentos por filtro");
	    return this.departamentosService.findAll(filterDepartamentos, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param departamentos Departamentos 
	 *            Bean que contiene la informacion a modificar.
	 * @return Departamentos 
	 *            Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
    public @ResponseBody Departamentos edit(@RequestBody Departamentos departamentos) {		
        Departamentos departamentosAux = this.departamentosService.update(departamentos);
		DepartamentosController.logger.info("[PUT] : Departamentos actualizado correctamente");
        return departamentosAux;
    }

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param departamentos Departamentos 
	 *            Bean que contiene la informacion con la que se va a crear el
	 *            nuevo registro.
	 * @return Departamentos
	 *            Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Departamentos add(@RequestBody Departamentos departamentos) {		
        Departamentos departamentosAux = this.departamentosService.add(departamentos);
        DepartamentosController.logger.info("[POST] : Departamentos insertado correctamente");
    	return departamentosAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idDepartamento Integer
	 *            Identificador del objeto que se desea eliminar.
	 * @return Departamentos
	 *            Bean eliminado.
	 */
	@RequestMapping(value = "/{idDepartamento}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
    public @ResponseBody Departamentos remove(@PathVariable Integer idDepartamento) {
        Departamentos departamentos = new Departamentos();
        departamentos.setIdDepartamento(idDepartamento);
        this.departamentosService.remove(departamentos);
       	DepartamentosController.logger.info("[DELETE] : Departamentos borrado correctamente");
       	return departamentos;
    }
    
	
	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	 /**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		DepartamentosController.logger.info("[GET - View] : departamentos");
		return "departamentos";
	}
	 
	 /**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterDepartamentos Departamentos
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Departamentos>
	 *            Dto que contiene el resultado del filtrado realizado por el 
	 *            componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Departamentos> filter(
			@RequestJsonBody(param="filter") Departamentos filterDepartamentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DepartamentosController.logger.info("[POST - filter] : Obtener Departamentoss");
		return this.departamentosService.filter(filterDepartamentos, jqGridRequestDto, false);
	}
	 
	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterDepartamentos Departamentos
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param searchDepartamentos Departamentos
	 *            Bean que contiene los parametros de busqueda a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Departamentos> 
	 *            Dto que contiene el resultado de la busqueda realizada por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Departamentos>> search(
			@RequestJsonBody(param="filter") Departamentos filterDepartamentos,
			@RequestJsonBody(param="search") Departamentos searchDepartamentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DepartamentosController.logger.info("[POST - search] : Buscar Departamentoss");
		return this.departamentosService.search(filterDepartamentos, searchDepartamentos, jqGridRequestDto, false);
	}
	
	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterDepartamentos Departamentos
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String>
	 *            Lista de los identificadores de los registros eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param="filter") Departamentos filterDepartamentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DepartamentosController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Departamentoss");
		this.departamentosService.removeMultiple(filterDepartamentos, jqGridRequestDto, false);
		DepartamentosController.logger.info("All entities correctly deleted!");
		
		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}
	
	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */
	
	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterDepartamentos Departamentos
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Departamentos>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Departamentos>> jerarquia(
			@RequestJsonBody(param="filter") Departamentos filterDepartamentos,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DepartamentosController.logger.info("[POST - jerarquia] : Obtener Departamentoss jerarquia");
		return this.departamentosService.jerarquia(filterDepartamentos, jqGridRequestDto, false);
	}
	
	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion jerarquica.
	 * 
	 * @param filterDepartamentos Departamentos
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Departamentos>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Departamentos>> jerarquiaChildren(
			@RequestJsonBody(param="filter") Departamentos  filterDepartamentos ,
			@RequestJsonBody JQGridRequestDto  jqGridRequestDto) {
		DepartamentosController.logger.info("[POST - jerarquia] : Obtener Departamentoss jerarquia - Hijos");
		return this.departamentosService.jerarquiaChildren(filterDepartamentos, jqGridRequestDto);
	}
	
}	
