package com.ejie.ab61.control.procedimientos;


import com.ejie.ab61.model.procedimientos.Direcciones;
import com.ejie.ab61.service.procedimientos.DireccionesService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * DireccionesController  
 *  
 */
 
@Controller
@RequestMapping (value = "/direcciones")

public class DireccionesController  {

	private static final Logger logger = LoggerFactory.getLogger(DireccionesController.class);

	@Autowired
	private DireccionesService direccionesService;
	
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */
	
	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador indicado.
	 * 
	 * @param idDireccion Integer
	 * @return Direcciones 
	 *            Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idDireccion}", method = RequestMethod.GET)
	public @ResponseBody Direcciones get(@PathVariable Integer idDireccion) {
        Direcciones direcciones = new Direcciones();
		direcciones.setIdDireccion(idDireccion);
        direcciones = this.direccionesService.find(direcciones);
        DireccionesController.logger.info("[GET - findBy_PK] : Obtener Direcciones por PK");
        return direcciones;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterDirecciones Direcciones
	 *            Objeto que contiene los parametros de filtrado utilizados en
	 *            la busqueda.
	 * @return List<Direcciones> 
	 *            Lista de objetos correspondientes a la busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<Direcciones> getAll(@ModelAttribute Direcciones filterDirecciones) {
		DireccionesController.logger.info("[GET - find_ALL] : Obtener Direcciones por filtro");
	    return this.direccionesService.findAll(filterDirecciones, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param direcciones Direcciones 
	 *            Bean que contiene la informacion a modificar.
	 * @return Direcciones 
	 *            Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
    public @ResponseBody Direcciones edit(@RequestBody Direcciones direcciones) {		
        Direcciones direccionesAux = this.direccionesService.update(direcciones);
		DireccionesController.logger.info("[PUT] : Direcciones actualizado correctamente");
        return direccionesAux;
    }

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param direcciones Direcciones 
	 *            Bean que contiene la informacion con la que se va a crear el
	 *            nuevo registro.
	 * @return Direcciones
	 *            Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody Direcciones add(@RequestBody Direcciones direcciones) {		
        Direcciones direccionesAux = this.direccionesService.add(direcciones);
        DireccionesController.logger.info("[POST] : Direcciones insertado correctamente");
    	return direccionesAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idDireccion Integer
	 *            Identificador del objeto que se desea eliminar.
	 * @return Direcciones
	 *            Bean eliminado.
	 */
	@RequestMapping(value = "/{idDireccion}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
    public @ResponseBody Direcciones remove(@PathVariable Integer idDireccion) {
        Direcciones direcciones = new Direcciones();
        direcciones.setIdDireccion(idDireccion);
        this.direccionesService.remove(direcciones);
       	DireccionesController.logger.info("[DELETE] : Direcciones borrado correctamente");
       	return direcciones;
    }
    
	
	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	 /**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		DireccionesController.logger.info("[GET - View] : direcciones");
		return "direcciones";
	}
	 
	 /**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterDirecciones Direcciones
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<Direcciones>
	 *            Dto que contiene el resultado del filtrado realizado por el 
	 *            componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<Direcciones> filter(
			@RequestJsonBody(param="filter") Direcciones filterDirecciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DireccionesController.logger.info("[POST - filter] : Obtener Direccioness");
		return this.direccionesService.filter(filterDirecciones, jqGridRequestDto, false);
	}
	 
	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterDirecciones Direcciones
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param searchDirecciones Direcciones
	 *            Bean que contiene los parametros de busqueda a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<Direcciones> 
	 *            Dto que contiene el resultado de la busqueda realizada por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<Direcciones>> search(
			@RequestJsonBody(param="filter") Direcciones filterDirecciones,
			@RequestJsonBody(param="search") Direcciones searchDirecciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DireccionesController.logger.info("[POST - search] : Buscar Direccioness");
		return this.direccionesService.search(filterDirecciones, searchDirecciones, jqGridRequestDto, false);
	}
	
	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterDirecciones Direcciones
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String>
	 *            Lista de los identificadores de los registros eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param="filter") Direcciones filterDirecciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DireccionesController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples Direccioness");
		this.direccionesService.removeMultiple(filterDirecciones, jqGridRequestDto, false);
		DireccionesController.logger.info("All entities correctly deleted!");
		
		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}
	
	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */
	
	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterDirecciones Direcciones
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Direcciones>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Direcciones>> jerarquia(
			@RequestJsonBody(param="filter") Direcciones filterDirecciones,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		DireccionesController.logger.info("[POST - jerarquia] : Obtener Direccioness jerarquia");
		return this.direccionesService.jerarquia(filterDirecciones, jqGridRequestDto, false);
	}
	
	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion jerarquica.
	 * 
	 * @param filterDirecciones Direcciones
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<Direcciones>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<Direcciones>> jerarquiaChildren(
			@RequestJsonBody(param="filter") Direcciones  filterDirecciones ,
			@RequestJsonBody JQGridRequestDto  jqGridRequestDto) {
		DireccionesController.logger.info("[POST - jerarquia] : Obtener Direccioness jerarquia - Hijos");
		return this.direccionesService.jerarquiaChildren(filterDirecciones, jqGridRequestDto);
	}
	
}	
