package com.ejie.ab61.control.solicitudes;

import com.ejie.ab61.model.solicitudes.MensajesSolicitud;
import com.ejie.ab61.service.solicitudes.MensajesSolicitudService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * MensajesSolicitudController  
 *  
 */
 
@Controller
@RequestMapping (value = "/mensajessolicitud")

public class MensajesSolicitudController  {

	private static final Logger logger = LoggerFactory.getLogger(MensajesSolicitudController.class);

	@Autowired
	private MensajesSolicitudService mensajesSolicitudService;
	
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */
	
	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador indicado.
	 * 
	 * @param idMensaje Integer
	 * @return MensajesSolicitud 
	 *            Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idMensaje}", method = RequestMethod.GET)
	public @ResponseBody MensajesSolicitud get(@PathVariable Integer idMensaje) {
        MensajesSolicitud mensajesSolicitud = new MensajesSolicitud();
		mensajesSolicitud.setIdMensaje(idMensaje);
        mensajesSolicitud = this.mensajesSolicitudService.find(mensajesSolicitud);
        MensajesSolicitudController.logger.info("[GET - findBy_PK] : Obtener MensajesSolicitud por PK");
        return mensajesSolicitud;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterMensajesSolicitud MensajesSolicitud
	 *            Objeto que contiene los parametros de filtrado utilizados en
	 *            la busqueda.
	 * @return List<MensajesSolicitud> 
	 *            Lista de objetos correspondientes a la busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<MensajesSolicitud> getAll(@ModelAttribute MensajesSolicitud filterMensajesSolicitud) {
		MensajesSolicitudController.logger.info("[GET - find_ALL] : Obtener MensajesSolicitud por filtro");
	    return this.mensajesSolicitudService.findAll(filterMensajesSolicitud, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param mensajesSolicitud MensajesSolicitud 
	 *            Bean que contiene la informacion a modificar.
	 * @return MensajesSolicitud 
	 *            Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
    public @ResponseBody MensajesSolicitud edit(@RequestBody MensajesSolicitud mensajesSolicitud) {		
        MensajesSolicitud mensajesSolicitudAux = this.mensajesSolicitudService.update(mensajesSolicitud);
		MensajesSolicitudController.logger.info("[PUT] : MensajesSolicitud actualizado correctamente");
        return mensajesSolicitudAux;
    }

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param mensajesSolicitud MensajesSolicitud 
	 *            Bean que contiene la informacion con la que se va a crear el
	 *            nuevo registro.
	 * @return MensajesSolicitud
	 *            Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody MensajesSolicitud add(@RequestBody MensajesSolicitud mensajesSolicitud) {		
        MensajesSolicitud mensajesSolicitudAux = this.mensajesSolicitudService.add(mensajesSolicitud);
        MensajesSolicitudController.logger.info("[POST] : MensajesSolicitud insertado correctamente");
    	return mensajesSolicitudAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idMensaje Integer
	 *            Identificador del objeto que se desea eliminar.
	 * @return MensajesSolicitud
	 *            Bean eliminado.
	 */
	@RequestMapping(value = "/{idMensaje}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
    public @ResponseBody MensajesSolicitud remove(@PathVariable Integer idMensaje) {
        MensajesSolicitud mensajesSolicitud = new MensajesSolicitud();
        mensajesSolicitud.setIdMensaje(idMensaje);
        this.mensajesSolicitudService.remove(mensajesSolicitud);
       	MensajesSolicitudController.logger.info("[DELETE] : MensajesSolicitud borrado correctamente");
       	return mensajesSolicitud;
    }
    
	
	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	 /**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		MensajesSolicitudController.logger.info("[GET - View] : mensajessolicitud");
		return "mensajessolicitud";
	}
	 
	 /**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterMensajesSolicitud MensajesSolicitud
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<MensajesSolicitud>
	 *            Dto que contiene el resultado del filtrado realizado por el 
	 *            componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<MensajesSolicitud> filter(
			@RequestJsonBody(param="filter") MensajesSolicitud filterMensajesSolicitud,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MensajesSolicitudController.logger.info("[POST - filter] : Obtener MensajesSolicituds");
		return this.mensajesSolicitudService.filter(filterMensajesSolicitud, jqGridRequestDto, false);
	}
	 
	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterMensajesSolicitud MensajesSolicitud
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param searchMensajesSolicitud MensajesSolicitud
	 *            Bean que contiene los parametros de busqueda a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<MensajesSolicitud> 
	 *            Dto que contiene el resultado de la busqueda realizada por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<MensajesSolicitud>> search(
			@RequestJsonBody(param="filter") MensajesSolicitud filterMensajesSolicitud,
			@RequestJsonBody(param="search") MensajesSolicitud searchMensajesSolicitud,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MensajesSolicitudController.logger.info("[POST - search] : Buscar MensajesSolicituds");
		return this.mensajesSolicitudService.search(filterMensajesSolicitud, searchMensajesSolicitud, jqGridRequestDto, false);
	}
	
	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterMensajesSolicitud MensajesSolicitud
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String>
	 *            Lista de los identificadores de los registros eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param="filter") MensajesSolicitud filterMensajesSolicitud,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MensajesSolicitudController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples MensajesSolicituds");
		this.mensajesSolicitudService.removeMultiple(filterMensajesSolicitud, jqGridRequestDto, false);
		MensajesSolicitudController.logger.info("All entities correctly deleted!");
		
		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}
	
	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */
	
	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterMensajesSolicitud MensajesSolicitud
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<MensajesSolicitud>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<MensajesSolicitud>> jerarquia(
			@RequestJsonBody(param="filter") MensajesSolicitud filterMensajesSolicitud,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		MensajesSolicitudController.logger.info("[POST - jerarquia] : Obtener MensajesSolicituds jerarquia");
		return this.mensajesSolicitudService.jerarquia(filterMensajesSolicitud, jqGridRequestDto, false);
	}
	
	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion jerarquica.
	 * 
	 * @param filterMensajesSolicitud MensajesSolicitud
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<MensajesSolicitud>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<MensajesSolicitud>> jerarquiaChildren(
			@RequestJsonBody(param="filter") MensajesSolicitud  filterMensajesSolicitud ,
			@RequestJsonBody JQGridRequestDto  jqGridRequestDto) {
		MensajesSolicitudController.logger.info("[POST - jerarquia] : Obtener MensajesSolicituds jerarquia - Hijos");
		return this.mensajesSolicitudService.jerarquiaChildren(filterMensajesSolicitud, jqGridRequestDto);
	}
	
}	
