package com.ejie.ab61.control.terceros;

import com.ejie.ab61.model.terceros.TiposCanalAviso;
import com.ejie.ab61.service.terceros.TiposCanalAvisoService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * TiposCanalAvisoController  
 *  
 */
 
@Controller
@RequestMapping (value = "/tiposcanalaviso")

public class TiposCanalAvisoController  {

	private static final Logger logger = LoggerFactory.getLogger(TiposCanalAvisoController.class);

	@Autowired
	private TiposCanalAvisoService tiposCanalAvisoService;
	
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */
	
	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador indicado.
	 * 
	 * @param idTipoCanalAviso Long
	 * @return TiposCanalAviso 
	 *            Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idTipoCanalAviso}", method = RequestMethod.GET)
	public @ResponseBody TiposCanalAviso get(@PathVariable Long idTipoCanalAviso) {
        TiposCanalAviso tiposCanalAviso = new TiposCanalAviso();
		tiposCanalAviso.setIdTipoCanalAviso(idTipoCanalAviso);
        tiposCanalAviso = this.tiposCanalAvisoService.find(tiposCanalAviso);
        TiposCanalAvisoController.logger.info("[GET - findBy_PK] : Obtener TiposCanalAviso por PK");
        return tiposCanalAviso;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterTiposCanalAviso TiposCanalAviso
	 *            Objeto que contiene los parametros de filtrado utilizados en
	 *            la busqueda.
	 * @return List<TiposCanalAviso> 
	 *            Lista de objetos correspondientes a la busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<TiposCanalAviso> getAll(@ModelAttribute TiposCanalAviso filterTiposCanalAviso) {
		TiposCanalAvisoController.logger.info("[GET - find_ALL] : Obtener TiposCanalAviso por filtro");
	    return this.tiposCanalAvisoService.findAll(filterTiposCanalAviso, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param tiposCanalAviso TiposCanalAviso 
	 *            Bean que contiene la informacion a modificar.
	 * @return TiposCanalAviso 
	 *            Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
    public @ResponseBody TiposCanalAviso edit(@RequestBody TiposCanalAviso tiposCanalAviso) {		
        TiposCanalAviso tiposCanalAvisoAux = this.tiposCanalAvisoService.update(tiposCanalAviso);
		TiposCanalAvisoController.logger.info("[PUT] : TiposCanalAviso actualizado correctamente");
        return tiposCanalAvisoAux;
    }

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param tiposCanalAviso TiposCanalAviso 
	 *            Bean que contiene la informacion con la que se va a crear el
	 *            nuevo registro.
	 * @return TiposCanalAviso
	 *            Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody TiposCanalAviso add(@RequestBody TiposCanalAviso tiposCanalAviso) {		
        TiposCanalAviso tiposCanalAvisoAux = this.tiposCanalAvisoService.add(tiposCanalAviso);
        TiposCanalAvisoController.logger.info("[POST] : TiposCanalAviso insertado correctamente");
    	return tiposCanalAvisoAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idTipoCanalAviso Long
	 *            Identificador del objeto que se desea eliminar.
	 * @return TiposCanalAviso
	 *            Bean eliminado.
	 */
	@RequestMapping(value = "/{idTipoCanalAviso}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
    public @ResponseBody TiposCanalAviso remove(@PathVariable Long idTipoCanalAviso) {
        TiposCanalAviso tiposCanalAviso = new TiposCanalAviso();
        tiposCanalAviso.setIdTipoCanalAviso(idTipoCanalAviso);
        this.tiposCanalAvisoService.remove(tiposCanalAviso);
       	TiposCanalAvisoController.logger.info("[DELETE] : TiposCanalAviso borrado correctamente");
       	return tiposCanalAviso;
    }
    
	
	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	 /**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		TiposCanalAvisoController.logger.info("[GET - View] : tiposcanalaviso");
		return "tiposcanalaviso";
	}
	 
	 /**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterTiposCanalAviso TiposCanalAviso
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<TiposCanalAviso>
	 *            Dto que contiene el resultado del filtrado realizado por el 
	 *            componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<TiposCanalAviso> filter(
			@RequestJsonBody(param="filter") TiposCanalAviso filterTiposCanalAviso,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposCanalAvisoController.logger.info("[POST - filter] : Obtener TiposCanalAvisos");
		return this.tiposCanalAvisoService.filter(filterTiposCanalAviso, jqGridRequestDto, false);
	}
	 
	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterTiposCanalAviso TiposCanalAviso
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param searchTiposCanalAviso TiposCanalAviso
	 *            Bean que contiene los parametros de busqueda a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<TiposCanalAviso> 
	 *            Dto que contiene el resultado de la busqueda realizada por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<TiposCanalAviso>> search(
			@RequestJsonBody(param="filter") TiposCanalAviso filterTiposCanalAviso,
			@RequestJsonBody(param="search") TiposCanalAviso searchTiposCanalAviso,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposCanalAvisoController.logger.info("[POST - search] : Buscar TiposCanalAvisos");
		return this.tiposCanalAvisoService.search(filterTiposCanalAviso, searchTiposCanalAviso, jqGridRequestDto, false);
	}
	
	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterTiposCanalAviso TiposCanalAviso
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String>
	 *            Lista de los identificadores de los registros eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param="filter") TiposCanalAviso filterTiposCanalAviso,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposCanalAvisoController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples TiposCanalAvisos");
		this.tiposCanalAvisoService.removeMultiple(filterTiposCanalAviso, jqGridRequestDto, false);
		TiposCanalAvisoController.logger.info("All entities correctly deleted!");
		
		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}
	
	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */
	
	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterTiposCanalAviso TiposCanalAviso
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<TiposCanalAviso>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<TiposCanalAviso>> jerarquia(
			@RequestJsonBody(param="filter") TiposCanalAviso filterTiposCanalAviso,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposCanalAvisoController.logger.info("[POST - jerarquia] : Obtener TiposCanalAvisos jerarquia");
		return this.tiposCanalAvisoService.jerarquia(filterTiposCanalAviso, jqGridRequestDto, false);
	}
	
	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion jerarquica.
	 * 
	 * @param filterTiposCanalAviso TiposCanalAviso
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<TiposCanalAviso>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<TiposCanalAviso>> jerarquiaChildren(
			@RequestJsonBody(param="filter") TiposCanalAviso  filterTiposCanalAviso ,
			@RequestJsonBody JQGridRequestDto  jqGridRequestDto) {
		TiposCanalAvisoController.logger.info("[POST - jerarquia] : Obtener TiposCanalAvisos jerarquia - Hijos");
		return this.tiposCanalAvisoService.jerarquiaChildren(filterTiposCanalAviso, jqGridRequestDto);
	}
	
}	
