package com.ejie.ab61.control.terceros;

import com.ejie.ab61.model.terceros.TiposTercero;
import com.ejie.ab61.service.terceros.TiposTerceroService;
import com.ejie.x38.control.bind.annotation.RequestJsonBody;
import com.ejie.x38.dto.JQGridRequestDto;
import com.ejie.x38.dto.JQGridResponseDto;
import com.ejie.x38.dto.JerarquiaDto;
import com.ejie.x38.dto.TableRowDto;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

/**
 * TiposTerceroController  
 *  
 */
 
@Controller
@RequestMapping (value = "/tipostercero")

public class TiposTerceroController  {

	private static final Logger logger = LoggerFactory.getLogger(TiposTerceroController.class);

	@Autowired
	private TiposTerceroService tiposTerceroService;
	
	/*
	 * OPERACIONES CRUD (Create, Read, Update, Delete)
	 * 
	 */
	
	/**
	 * Operacion CRUD Read. Devuelve el bean correspondiente al identificador indicado.
	 * 
	 * @param idTipoTercero Long
	 * @return TiposTercero 
	 *            Objeto correspondiente al identificador indicado.
	 */
	@RequestMapping(value = "/{idTipoTercero}", method = RequestMethod.GET)
	public @ResponseBody TiposTercero get(@PathVariable Long idTipoTercero) {
        TiposTercero tiposTercero = new TiposTercero();
		tiposTercero.setIdTipoTercero(idTipoTercero);
        tiposTercero = this.tiposTerceroService.find(tiposTercero);
        TiposTerceroController.logger.info("[GET - findBy_PK] : Obtener TiposTercero por PK");
        return tiposTercero;
	}

	/**
	 * Devuelve una lista de beans correspondientes a los valores de filtrados
	 * indicados en el objeto pasado como parametro.
	 *
	 * @param filterTiposTercero TiposTercero
	 *            Objeto que contiene los parametros de filtrado utilizados en
	 *            la busqueda.
	 * @return List<TiposTercero> 
	 *            Lista de objetos correspondientes a la busqueda realizada.
	 */
	@RequestMapping(method = RequestMethod.GET)
	public @ResponseBody List<TiposTercero> getAll(@ModelAttribute TiposTercero filterTiposTercero) {
		TiposTerceroController.logger.info("[GET - find_ALL] : Obtener TiposTercero por filtro");
	    return this.tiposTerceroService.findAll(filterTiposTercero, null);
	}

	/**
	 * Operacion CRUD Edit. Modificacion del bean indicado.
	 *
	 * @param tiposTercero TiposTercero 
	 *            Bean que contiene la informacion a modificar.
	 * @return TiposTercero 
	 *            Bean resultante de la modificacion.
	 */
	@RequestMapping(method = RequestMethod.PUT)
    public @ResponseBody TiposTercero edit(@RequestBody TiposTercero tiposTercero) {		
        TiposTercero tiposTerceroAux = this.tiposTerceroService.update(tiposTercero);
		TiposTerceroController.logger.info("[PUT] : TiposTercero actualizado correctamente");
        return tiposTerceroAux;
    }

	/**
	 * Operacion CRUD Create. Creacion de un nuevo registro a partir del bean
	 * indicado.
	 *
	 * @param tiposTercero TiposTercero 
	 *            Bean que contiene la informacion con la que se va a crear el
	 *            nuevo registro.
	 * @return TiposTercero
	 *            Bean resultante del proceso de creacion.
	 */
	@RequestMapping(method = RequestMethod.POST)
	public @ResponseBody TiposTercero add(@RequestBody TiposTercero tiposTercero) {		
        TiposTercero tiposTerceroAux = this.tiposTerceroService.add(tiposTercero);
        TiposTerceroController.logger.info("[POST] : TiposTercero insertado correctamente");
    	return tiposTerceroAux;
	}

	/**
	 * Operacion CRUD Delete. Borrado del registro correspondiente al
	 * identificador especificado.
	 *
	 * @param idTipoTercero Long
	 *            Identificador del objeto que se desea eliminar.
	 * @return TiposTercero
	 *            Bean eliminado.
	 */
	@RequestMapping(value = "/{idTipoTercero}", method = RequestMethod.DELETE)
	@ResponseStatus(value = HttpStatus.OK)
    public @ResponseBody TiposTercero remove(@PathVariable Long idTipoTercero) {
        TiposTercero tiposTercero = new TiposTercero();
        tiposTercero.setIdTipoTercero(idTipoTercero);
        this.tiposTerceroService.remove(tiposTercero);
       	TiposTerceroController.logger.info("[DELETE] : TiposTercero borrado correctamente");
       	return tiposTercero;
    }
    
	
	/*
	 * METODOS COMPONENTE RUP_TABLE
	 * 
	 */

	 /**
	 * Metodo de presentacion del RUP_TABLE.
	 * 
	 * @param model Model
	 * @return String
	 */
	@RequestMapping(value = "/maint", method = RequestMethod.GET)
	public String getFormEdit(Model model) {
		TiposTerceroController.logger.info("[GET - View] : tipostercero");
		return "tipostercero";
	}
	 
	 /**
	 * Operacion de filtrado del componente RUP_TABLE.
	 * 
	 * @param filterTiposTercero TiposTercero
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<TiposTercero>
	 *            Dto que contiene el resultado del filtrado realizado por el 
	 *            componente RUP_TABLE.
	 */
	@RequestMapping(value = "/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<TiposTercero> filter(
			@RequestJsonBody(param="filter") TiposTercero filterTiposTercero,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposTerceroController.logger.info("[POST - filter] : Obtener TiposTerceros");
		return this.tiposTerceroService.filter(filterTiposTercero, jqGridRequestDto, false);
	}
	 
	/**
	 * Operacion de busqueda del componente RUP_TABLE.
	 * 
	 * @param filterTiposTercero TiposTercero
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param searchTiposTercero TiposTercero
	 *            Bean que contiene los parametros de busqueda a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la búsqueda.
	 * @return TableRowDto<TiposTercero> 
	 *            Dto que contiene el resultado de la busqueda realizada por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/search", method = RequestMethod.POST)
	public @ResponseBody List<TableRowDto<TiposTercero>> search(
			@RequestJsonBody(param="filter") TiposTercero filterTiposTercero,
			@RequestJsonBody(param="search") TiposTercero searchTiposTercero,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposTerceroController.logger.info("[POST - search] : Buscar TiposTerceros");
		return this.tiposTerceroService.search(filterTiposTercero, searchTiposTercero, jqGridRequestDto, false);
	}
	
	/**
	 * Borrado multiple de registros
	 * 
	 * @param filterTiposTercero TiposTercero
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en la busqueda.
	 * @return List<String>
	 *            Lista de los identificadores de los registros eliminados.
	 * 
	 */
	@RequestMapping(value = "/deleteAll", method = RequestMethod.POST)
	@ResponseStatus(value = HttpStatus.OK)
	public @ResponseBody List<String> removeMultiple(
			@RequestJsonBody(param="filter") TiposTercero filterTiposTercero,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposTerceroController.logger.info("[POST - search] : [POST - removeMultiple] : Eliminar multiples TiposTerceros");
		this.tiposTerceroService.removeMultiple(filterTiposTercero, jqGridRequestDto, false);
		TiposTerceroController.logger.info("All entities correctly deleted!");
		
		return jqGridRequestDto.getMultiselection().getSelectedIds();
	}
	
	/*
	 * METODOS COMPONENTE RUP_TABLE - JERARQUIA
	 */
	
	/**
	 * Operacion de filtrado del componente RUP_TABLE para presentar los
	 * registros mediante visualizacion jerarquica.
	 * 
	 * @param filterTiposTercero TiposTercero
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<TiposTercero>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquia/filter", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<TiposTercero>> jerarquia(
			@RequestJsonBody(param="filter") TiposTercero filterTiposTercero,
			@RequestJsonBody JQGridRequestDto jqGridRequestDto) {
		TiposTerceroController.logger.info("[POST - jerarquia] : Obtener TiposTerceros jerarquia");
		return this.tiposTerceroService.jerarquia(filterTiposTercero, jqGridRequestDto, false);
	}
	
	/**
	 * Recupera los hijos de los registros desplegados en la visualizacion jerarquica.
	 * 
	 * @param filterTiposTercero TiposTercero
	 *            Bean que contiene los parametros de filtrado a emplear.
	 * @param jqGridRequestDto
	 *            Dto que contiene los parametros de configuracion propios del
	 *            RUP_TABLE a aplicar en el filtrado.
	 * @return JQGridResponseDto<JerarquiaDto<TiposTercero>>
	 *            Dto que contiene el resultado del filtrado realizado por el
	 *            componente RUP_TABLE. 
	 */
	@RequestMapping(value = "/jerarquiaChildren", method = RequestMethod.POST)
	public @ResponseBody JQGridResponseDto<JerarquiaDto<TiposTercero>> jerarquiaChildren(
			@RequestJsonBody(param="filter") TiposTercero  filterTiposTercero ,
			@RequestJsonBody JQGridRequestDto  jqGridRequestDto) {
		TiposTerceroController.logger.info("[POST - jerarquia] : Obtener TiposTerceros jerarquia - Hijos");
		return this.tiposTerceroService.jerarquiaChildren(filterTiposTercero, jqGridRequestDto);
	}
	
}	
